/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*****************************************************************************
  program name: gl_auto.c  (running as a version of lm_auto).
******************************************************************************/

/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_auto"

#include "autozyg.h"
#include "auto_chk.c"

#define AUTO_CKPNTS 1000

void  gl_do_auto_sweep (int, int);
void  prnt_xxx (FILE *, int **, int, int);
void  prnt_fgl (FILE *, int **, int, int);
void  score_gametes (int, int);
void  score_counts (int, int);
void  alloc_xscor (int, double ****);
void  free_xscor (int, double ****);

static int nsw;
static int mc_itno;
                                        /* Reinstate lm_auto variables    */
static int *gaga;                       /* [g]                            */
static int *npt;                        /* [g+1]  no. of patterns for
                                         *        gametes                 */
static int **iindex;                    /* [g+1][cd+1]                    */
static int **label;                     /* [g+1][pt+1]                    */
static int **list;                      /* [g+1][pt+1]                    */
static int nwpt;                        /* number of binary patterns,
                                         *   and of windows               */
static int *gw, *wloc;                  /* binary return for locus in
                                         *   window                       */
static double ****wscore;               /* window scores, each cmpt,
                                         * each scoreset, each window,
                                         *   each pattern
                                         *   assume 1 compnt, 1 scoreset  */
static double ***xscore;                /* [ns][nl][pt] exact prob scores */
static int **count;                     /* [nc][nl]                       */
static double ****nscore;               /* [nc][ns][nl][pt]  ibd counts:
                                         *   ea component, ea scoreset,
                                         *   ea locus, ea pattern         */

                                        /* Counts and log-probs           */
static double *ctot;                    /* CDLL by cmpt                   */
static int tot_count = 0;               /* score by component and locus   */

static double *sss;                     /* [nl]                           */
static double ss;
static int ***recounts;                 /* [nc][nl][6] counts of recombs  */
static double **tot_log_prob_seg;
static double **tot_log_prob_pen;
static double **temp_log_prob_seg;
static double **prob;

static double **prob_fwd;               /* pcf of exact_ivprob now must be
                                           returned to main               */

                                      /* fgl_dump variables                */
static int optout;                      /* if =1, count, etc omitted         */
static int ixxfgl;                      /* indicator of xxx or fgl output    */
static Indlist *pind;                   /* pointer to component indlist */

static FILE *oScorF;

int main (int argc, char *argv[])
{
   int   i, j, jj, k, kk, ng, s, c, t;
   int   nm, nm2, *w2, allox;           /* temp for xact stuff */

   int   nCompnt1, cmp, nctemp;         /* cmp indicators for sampling by cmpt */
   int   mul;                           /* mul is indicator of MM-sampling   */

   FILE *oSeedF, *iXtra, *ibdF;

   nsw = setupAutozyg (argc, argv, PNAM, 0);
   setupSampling ();
   prob = setupInherit (0);

   /* set up probands for gene scoring */
   /* if the number of components read is less than the last one for which
      there are proband gametes, reset the number of components to score      */
   NScCmp = set_probgam (n_compnts);

   tot_count = 0;
   allox = 0;

   /* gaga[], iindex[][], label[][], list[][], npt[] are malloced for the
      number of gametes in the largest set */
   /*  alloc of count and sss added here, EAT 2/2002       */
   /* count total number of patterns                       */

   alloc_ibd (&gaga, &iindex, &label, &list, &npt, &count, &nscore, &sss);

   /* count total number of patterns */

   for (j = 1; j <= MxGam; ++j)
      npt[j] = index_ibd (j, 0, 0, iindex[j], label[j], list[j], ibdF);

   pr_npatsets (npt);

   nwpt = 1;
   for (j = 0; j < NLocWndw; ++j)
      nwpt *= 2;

   if (NLocWndw)
   {
      alloc_wndw (&gw, &wloc, &wscore, NWndw, nwpt);

      for (c = 0; c < NScCmp; ++c)
      {
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NWndw; ++j)
               for (k = 0; k < nwpt; ++k)
                  wscore[c][s][j][k] = 0.0;
      }
   }
   else
   {
      wloc = malloc ((NLoci + 1) * sizeof (int));
      alloc_chk (WHERE, wloc);
   }

/* set up flags for sampling etc.   */
   if (!(iXtra = open_fil (XTRA)))
      give_up (ERR_qit);
   fscanf (iXtra, "%d %d %d %d %d", &mul, &cmp, &optout, &MaxNMexact, &ixxfgl);

/* set up for MM-sampling: from lm_multiple  */
   if (mul == 1)
   {
      Max_update = 8;
      P_meio = 0.2;
      P_ind = 0.3;
      P_sib = 0.3;
      P_3gen = 0.2;
   }

   if (!cmp)
      GlobalMcmc = 1;                   /* first get working for Global MCMC */

   list_switches (nsw);
   list_nghds ();

   printf ("\n done lists ");
   fflush (stdout);
   if (mul)
   {
      setup_marriage ();
      prnt_marriage ();
   }
   printf ("\n done marriages");
   fflush (stdout);

   nCompnt1 = 0;
   if (!GlobalMcmc)
   {
      /* number of components to be iid sampled */
      for (c = 1; c <= nCompnt; c++)
         if (cmplist[c].nmeioses <= MaxNMexact)
            nCompnt1++;
   }

   /* open output scores file, if requested */

   if (FilIt)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   prntStatusBMC ();
   fflush (stdout);

   if (GlobalMcmc)
   {
      cmp = 0;
      /* set up recombination patterns and initial probs */
      /* probabilities computed separately by component  */
      if (!optout)
      {
         ctot = malloc (n_compnts * sizeof (double));
         alloc_chk (WHERE, ctot);
         alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                       &tot_log_prob_pen);
         tot_count = 0;
         for (c = 0; c < n_compnts; ++c)
         {
            for (i = 0; i <= NMarks; ++i)
            {
               tot_log_prob_seg[c][i] = 0.0;
               tot_log_prob_pen[c][i] = 0.0;
            }
            for (j = 0; j <= NMarks; ++j)
            {
               count[c][j] = 0;
               for (jj = 0; jj < 6; ++jj)
                  recounts[c][j][jj] = 0;
               for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
                  for (k = 0; k <= PTgg[NPrGam[c][s]]; ++k)
                     nscore[c][s][j][k] = 0.0;
            }
         }
         alloc_logprob ();
      }

      setup_sampling_cmp (cmp, mul, NMarks);

      /* now do burn-in iterations */

      printf (" Now starting %d iterations of burn-in\n", NBrnIt);
      for (t = 1; t <= NBrnIt; t++)
      {
         if (!(t % ChkIt))
         {
            printf (" Starting burn-in iteration %d\n", t);
            fflush (stdout);
         }
         sampling_cmp (t, cmp, 0, mul, NMarks, nFGL, prob);
      }

      /* now do MCMC sampling, and score results */

      for (mc_itno = 0; mc_itno < NIt; ++mc_itno)
         gl_do_auto_sweep (mul, cmp);   /* do main mcmc iterations            */

      printf ("\n");
      prntStatusFMC ();

      for (c = 0; c < n_compnts; ++c)   /* now print results                  */
      {
         pr_cmpnthead (c);
         if (!optout)
         {
            pr_log_prob_seg (1, NMarks, -cmplist[c + 1].nmeioses,
                             tot_log_prob_seg[c], tot_count);
            pr_log_prob_pen (1, NMarks, tot_log_prob_pen[c], tot_count);
            pr_reco_counts (0, NMarks, recounts[c]);

            /* for ea set, print probabilities for ea pattern for ea window  */

            for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
            {
               /* format and print headings  */

               pr_probhead ();

               pr_pgamset (c, s);

               pr_lochead (chromo);

               /* format and print data  */

               ng = NPrGam[c][s];

               for (k = 0; k < npt[ng]; ++k)
               {
                  ss = 0.0;
                  for (j = 0; j < NLoci; ++j)
                  {
                     sss[j] = nscore[c][s][j][list[ng][k]] / count[c][j];
                     ss += sss[j];
                  }
                  if (ss > 0.0001)
                  {
                     invert_label (label[ng][k], ng, gaga);
                     pr_locprob (ng, gaga, sss, label[ng][k]);
                  }
               }
               printf ("\n");

               if (NLocWndw)            /* **** now output window scores */
               {
                  if (NPtLab[c][s])
                  {
                     pr_prbwhead (c, s);
                     pr_wndwpat (c, s);
                     pr_wndwhead (NWndw);
                     for (i = 0; i < nwpt; ++i)
                     {
                        ss = 0.;
                        for (j = 0; j < NWndw; ++j)
                           if (wscore[c][s][j][i] > ss)
                              ss = wscore[c][s][j][i];
                        if (ss / NIt > 0.0001)
                        {
                           binary (i, NLocWndw, gw);
                           pr_wndwprob (c, s, NWndw, i, gw, wscore);
                        }
                     }
                     printf ("\n");
                  }
               }

            }
         }
      }
      if (!optout)
      {
         free_1d ((void *) &ctot);
         free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                      &tot_log_prob_pen);
         free_logprob ();
      }
      free_sampling_cmp ();
   }
   else
   {
      printf (" Going to do stuff by components");
      fflush (stdout);

      nctemp = n_compnts;
      n_compnts = 1;

      for (cmp = 1; cmp <= NScCmp && cmp <= nctemp; ++cmp)
      {
/* set up recombination patterns and initial probs for n_compnts=1 cmpnt */
         if (!optout)
         {
            ctot = malloc (n_compnts * sizeof (double));
            alloc_chk (WHERE, ctot);
            alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                          &tot_log_prob_pen);
            tot_count = 0;
            c = 0;
            for (i = 0; i <= NMarks; ++i)
            {
               tot_log_prob_seg[c][i] = 0.0;
               tot_log_prob_pen[c][i] = 0.0;
            }
            for (j = 0; j <= NMarks; ++j)
            {
               count[c][j] = 0;
               for (jj = 0; jj < 6; ++jj)
                  recounts[c][j][jj] = 0;
               for (s = 0; s < NScSet[cmp - 1]; ++s)
                  for (k = 0; k <= PTgg[NPrGam[cmp - 1][s]]; ++k)
                     nscore[cmp - 1][s][j][k] = 0.0;
            }

            alloc_logprob ();
         }

         if (cmplist[cmp].nmeioses <= MaxNMexact)
         {
            printf ("\n Component %d will be iid sampled ", cmp);
            printf (" and/or exact marginal computation will be done\n \n");
            fflush (stdout);
            /* fwd computation only */
            exact_smpl (1, 0, cmp, 0, NMarks, nFGL, prob, NMarks, ctot);
            /* iid sampling if reqd  */
            mc_itno = NIt / FilIt;
            for (t = 0; t < mc_itno; ++t)
            {
               exact_smpl (0, 1, cmp, 0, NMarks, nFGL, prob, NMarks, ctot);

               if (!optout)
               {
                  score_gametes (cmp, 0);
                  score_counts (cmp, 0);
               }

               /* TEMP CHECK EAT FIXME : print out meiosis indicators directly

                  fprintf(oScorF, "\n");
                  for (j=0; j<=NMarks; ++j)
                  { pind = cmplist[cmp].people->next;
                  while (pind)
                  { i = pind->current->index;
                  if (!isNullNode(nghd[i].pa))
                  fprintf (oScorF, " %d %d",xx0[j][i],xx1[j][i]);
                  pind = pind -> next;
                  }
                  }
                  fflush(oScorF);
                */

               if (ixxfgl == 0)
               {
                  for (i = cmplist[cmp].meioses[0];
                       i <= cmplist[cmp].meioses[(cmplist[cmp].nmeioses - 1)];
                       ++i)
                  {
                     c = (meiosis[i].ind).index;
                     s = meiosis[i].sex;
                     if (s == 0)
                        prnt_xxx (oScorF, xx0, NMarks, c);
                     else
                        prnt_xxx (oScorF, xx1, NMarks, c);
                  }
               }
               else
               {
                  pind = cmplist[cmp].people->next;
                  while (pind)
                  {
                     i = pind->current->index;
                     /*         if (!isNullNode (nghd[i].pa)) ;  i a non-founder */
                     {
                        prnt_fgl (oScorF, gg0, NMarks, i);
                        prnt_fgl (oScorF, gg1, NMarks, i);
                     }
                     pind = pind->next;
                  }
               }
            }
            /* exact computation */
            if (NScSet[cmp - 1])
            {
               nm = cmplist[cmp].nmeioses;
               nm2 = pow2 (nm);
               if (nm > allox)
               {
                  if (allox)
                  {
                     free_2d ((void *) &prob_fwd);
                     free (w2);
                  }
                  prob_fwd = alloc_dbl2d (NMarks + 1, nm2);
                  w2 = malloc (nm * sizeof (int));
                  allox = nm;
               }

               exact_ivprob (cmp, 0, NMarks, nFGL, prob_fwd);

/*  Here is scoring still to be written  -- see store_pen for cmp    */
/* note set_FGL needs to be by cmpnt  -- if cmpnt, do  not reset founders
      but drop from founders through the indlist of the cmpnt       */

               for (k = 0; k < nm; ++k)
                  w2[k] = 0;
/* then allocate xscore[s][j][ptt] and set to 0  */
               alloc_xscor (cmp - 1, &xscore);

               for (i = 0; i < nm2; ++i)
               {
                  /*   set xx0/xx1 on the component from w2 , and set FGL -- use locus 0  */
                  for (k = 0; k < nm; k++)
                  {
                     kk = cmplist[cmp].meioses[k];
                     if (meiosis[kk].sex) /* male meiosis */
                        xx1[0][(meiosis[kk].ind).index] = w2[k];
                     else               /* female meiosis */
                        xx0[0][(meiosis[kk].ind).index] = w2[k];
                  }
                  reset_FGL (0, cmp);

                  for (j = NMarks; j >= 0; --j) /*   score ea locus       */
                  {
                     if (prob_fwd[j][i]) /*   prob for this mei/loc not 0 */
                     {
                        for (s = 0; s < NScSet[cmp - 1]; ++s) /* for ea score set in cmp */
                        {
                           score_xact (NPrGam[cmp - 1][s], gaga,
                                       iindex[NPrGam[cmp - 1][s]],
                                       list[NPrGam[cmp - 1][s]],
                                       ProbIx[cmp - 1][s], ProbGam[cmp - 1][s],
                                       xscore[s][j], prob_fwd[j][i]);
                        }
                     }
                  }
                  biadd (w2, nm, 1);
               }
            }
         }
         else
         {
            /* sampling /computation to be done separately by component  */

            setup_sampling_cmp (cmp, mul, NMarks);

            /* now do burn-in iterations */

            printf (" Now starting %d iterations of burn-in\n", NBrnIt);
            for (t = 1; t <= NBrnIt; t++)
            {
               if (!(t % ChkIt))
               {
                  printf (" Starting burn-in iteration %d\n", t);
                  fflush (stdout);
               }
               sampling_cmp (t, cmp, 0, mul, NMarks, nFGL, prob);
            }

            /* now do MCMC sampling, and score results */

            for (mc_itno = 0; mc_itno < NIt; ++mc_itno)
               gl_do_auto_sweep (mul, cmp); /* do main mcmc iterations   */

            printf ("\n");
            prntStatusFMC ();
            free_sampling_cmp ();
         }

         /* now print results    */
         c = 0;
         pr_cmpnthead (cmp - 1);
         if (!optout)
         {
            pr_log_prob_seg (1, NMarks, -cmplist[cmp].nmeioses,
                             tot_log_prob_seg[c], tot_count);
            pr_log_prob_pen (1, NMarks, tot_log_prob_pen[c], tot_count);
            pr_reco_counts (0, NMarks, recounts[c]);

            /* for ea set, print probabilities for ea pattern for ea window  */

            for (s = 0; c < NScCmp && s < NScSet[cmp - 1]; ++s)
            {
               /* format and print headings  */

               pr_probhead ();

               pr_pgamset (cmp - 1, s);

               pr_lochead (chromo);

               /* format and print data  */

               ng = NPrGam[cmp - 1][s];

               for (k = 0; k < npt[ng]; ++k)
               {
                  ss = 0.0;
                  for (j = 0; j < NLoci; ++j)
                  {
                     sss[j] = nscore[cmp - 1][s][j][list[ng][k]] / count[c][j];
                     ss += sss[j];
                  }
                  if (ss > 0.0001)
                  {
                     invert_label (label[ng][k], ng, gaga);
                     pr_locprob (ng, gaga, sss, label[ng][k]);
                  }
               }
               printf ("\n");

               if (NLocWndw)            /* **** now output window scores */
               {
                  if (NPtLab[cmp - 1][s])
                  {
                     pr_prbwhead (cmp - 1, s);
                     pr_wndwpat (cmp - 1, s);
                     pr_wndwhead (NWndw);

                     /* format and print data  */

                     for (i = 0; i < nwpt; ++i)
                     {
                        ss = 0.;
                        for (j = 0; j < NWndw; ++j)
                           if (wscore[cmp - 1][s][j][i] > ss)
                              ss = wscore[cmp - 1][s][j][i];
                        if (ss / NIt > 0.0001)
                        {
                           binary (i, NLocWndw, gw);
                           pr_wndwprob (c, s, NWndw, i, gw, wscore);
                        }
                     }
                     printf ("\n");
                  }
               }
            }
            /* print out exact computation results  */

            if (cmplist[cmp].nmeioses <= MaxNMexact)
            {
               for (s = 0; c < NScCmp && s < NScSet[cmp - 1]; ++s)
               {
                  /* format and print headings  */
                  printf ("\n Printing exact marginals for component %d", cmp);
                  pr_probhead ();
                  pr_pgamset (cmp - 1, s);
                  pr_lochead (chromo);

                  /* format and print data  */

                  ng = NPrGam[cmp - 1][s];

                  for (k = 0; k < npt[ng]; ++k)
                  {
                     ss = 0.0;
                     for (j = 0; j < NLoci; ++j)
                     {
                        sss[j] = xscore[s][j][list[ng][k]];
                        ss += sss[j];
                     }
                     if (ss > 0.0001)
                     {
                        invert_label (label[ng][k], ng, gaga);
                        pr_locprob (ng, gaga, sss, label[ng][k]);
                     }
                  }
                  printf ("\n");
               }
               free_xscor (cmp - 1, &xscore);
            }

            free_1d ((void *) &ctot);
            free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                         &tot_log_prob_pen);
            free_logprob ();
         }
         fflush (stdout);
      }
      /* now done with each component */
      n_compnts = nctemp;
   }

   /* ****  now we are done **** */
   fflush (stdout);

   /* Free up allocated memory */
   free_xxgg ();

   locus_peel_free ();
   free_allocate_loci ();
   free_chromo (0);
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);
   free_msmpl ();
   free_traits ();

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (FilIt)
      fclose (oScorF);

   if (Scan)
      free_perm ();

   exit (EXIT_SUCCESS);
}

void gl_do_auto_sweep (int amul, int acmp)
{
/* local variables: */
   int   c, s, i;

   /* now do the next MCMC sampling iteration */

   sampling_cmp (mc_itno, acmp, 0, amul, NMarks, nFGL, prob);

   /* score results */

   if (!((mc_itno + 1) % ScorIt))
   {
      /* score tot log-liks & recombs for this component */
      if (!acmp)
      {
         if (!optout)
         {
            c = 1;
            score_gametes (acmp, c);
            score_counts (acmp, c);
         }
         /* tests for output to scores file */
         if (FilIt && !((mc_itno + 1) % FilIt))
         {
            if (ixxfgl == 0)
            {
               for (i = 1; i <= nsw; ++i)
               {
                  c = (meiosis[i].ind).index;
                  s = meiosis[i].sex;
                  if (s == 0)
                     prnt_xxx (oScorF, xx0, NMarks, c);
                  else
                     prnt_xxx (oScorF, xx1, NMarks, c);
               }
            }
            else
            {
               for (i = 0; i < pedsize; ++i)
                  /*    if (!isNullNode (nghd[i].pa)) ;  i a non-founder */
               {
                  prnt_fgl (oScorF, gg0, NMarks, i);
                  prnt_fgl (oScorF, gg1, NMarks, i);
               }
            }
         }
      }
      else
      {
         if (!optout)
         {
            score_gametes (acmp, 0);
            score_counts (acmp, 0);
         }

         if (FilIt && !((mc_itno + 1) % FilIt)) /* tests for output to scores file */
         {
            if (ixxfgl == 0)
            {
               for (i = cmplist[acmp].meioses[0];
                    i <= cmplist[acmp].meioses[(cmplist[acmp].nmeioses - 1)];
                    ++i)
               {
                  c = (meiosis[i].ind).index;
                  s = meiosis[i].sex;
                  if (s == 0)
                     prnt_xxx (oScorF, xx0, NMarks, c);
                  else
                     prnt_xxx (oScorF, xx1, NMarks, c);
               }
            }
            else
            {
               pind = cmplist[acmp].people->next;
               while (pind)
               {
                  i = pind->current->index;
                  /*       if (!isNullNode (nghd[i].pa)) ;  i a non-founder */
                  {
                     prnt_fgl (oScorF, gg0, NMarks, i);
                     prnt_fgl (oScorF, gg1, NMarks, i);
                  }
                  pind = pind->next;
               }
            }
         }
      }
   }
}

/* score_counts() added as separate routine: EAT Aug 2008 */
/* score_gametes() added as separate routine: EAT Dec 2008 */

void score_gametes (int scmp, int cc)
{
   int   c, j, jj, s;
   if (!scmp)
   {
      for (c = 0; c < n_compnts; ++c)
         for (j = NMarks; j >= 0; --j)
            ++count[c][j];

      for (c = 0; c < NScCmp; ++c)      /* for ea component & score set      */
         for (s = 0; s < NScSet[c]; ++s)
         {
            for (j = NMarks; j >= 0; --j) /* score ea locus                 */
            {
               jj = chromo[0][j];
               wloc[j] = score_genes (jj, NPrGam[c][s], gaga,
                                      iindex[NPrGam[c][s]],
                                      list[NPrGam[c][s]], ProbIx[c][s],
                                      ProbGam[c][s], nscore[c][s][j],
                                      NPtLab[c][s], PatLab[c][s]);
            }
            if (NLocWndw)
               score_windows (wscore[c][s], NLocWndw, wloc, NMarks);
         }
   }
   else
   {
      for (j = NMarks; j >= 0; --j)
         ++count[cc][j];                /* cc or scmp ?? FIXME */

      for (s = 0; s < NScSet[scmp - 1]; ++s) /* for score set  in cmpt scmp */
      {
         for (j = NMarks; j >= 0; --j)  /* score ea locus                 */
         {
            jj = chromo[0][j];
            wloc[j] = score_genes (jj, NPrGam[scmp - 1][s], gaga,
                                   iindex[NPrGam[scmp - 1][s]],
                                   list[NPrGam[scmp - 1][s]],
                                   ProbIx[scmp - 1][s], ProbGam[scmp - 1][s],
                                   nscore[scmp - 1][s][j], NPtLab[scmp - 1][s],
                                   PatLab[scmp - 1][s]);
         }
         if (NLocWndw)
            score_windows (wscore[scmp - 1][s], NLocWndw, wloc, NMarks);
      }
   }

   return;
}

void score_counts (int scmp, int cc)
{
   int   c, j;
   if (!scmp)
   {
      for (c = 0; c < n_compnts; ++c)
      {
         ctot[c] = 0.0;
         full_seg_prob_cmp (NMarks, c, recounts[c], temp_log_prob_seg[c]);
         for (j = 1; j < NMarks; ++j)
         {
            tot_log_prob_pen[c][j] += log (prob[c][j]);
            tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];
            ctot[c] += (log (prob[c][j]) + temp_log_prob_seg[c][j]);
         }
         tot_log_prob_pen[c][NMarks] += log (prob[c][NMarks]);
         ctot[c] += log (prob[c][NMarks]);
      }
      ++tot_count;
   }
   else
   {
      ctot[cc] = 0.0;
      full_seg_prob_cmp (NMarks, scmp - 1, recounts[cc], temp_log_prob_seg[cc]);
      for (j = 1; j < NMarks; ++j)
      {
         tot_log_prob_pen[cc][j] += log (prob[scmp - 1][j]);
         tot_log_prob_seg[cc][j] += temp_log_prob_seg[cc][j];
         ctot[cc] += (log (prob[cc][j]) + temp_log_prob_seg[cc][j]);
      }
      tot_log_prob_pen[cc][NMarks] += log (prob[scmp - 1][NMarks]);
      ctot[cc] += log (prob[scmp - 1][NMarks]);
      ++tot_count;
   }
   return;
}

/* routine added EAT, April 2008  */

void prnt_xxx (FILE * fptr, int **xxx, int nmk, int ii)
{
   int   j, sw;
   sw = 0;
   for (j = 2; j <= nmk; ++j)
      if (xxx[j][ii] != xxx[j - 1][ii])
         ++sw;
   fprintf (fptr, "%d %d %d %d", ii, xxx[0][ii], xxx[1][ii], sw);
   for (j = 2; j <= nmk; ++j)
      if (xxx[j][ii] != xxx[j - 1][ii])
         fprintf (fptr, " %d", j);
   fprintf (fptr, "\n");
   return;
}

/* routine added EAT, December 2008 */

void prnt_fgl (FILE * fptr, int **ggg, int nmk, int ii)
{
   int   j, sw;
   sw = 0;
   for (j = 2; j <= nmk; ++j)
      if (ggg[j][ii] != ggg[j - 1][ii])
         ++sw;
   fprintf (fptr, "%s %d %d %d", nghd[ii].self, ggg[0][ii], ggg[1][ii], sw);
   for (j = 2; j <= nmk; ++j)
      if (ggg[j][ii] != ggg[j - 1][ii])
         fprintf (fptr, " %d %d ", j, ggg[j][ii]);
   fprintf (fptr, "\n");
   return;
}

void alloc_xscor (int c, double ****xsc)
{
   int   s, j, i;
   (*xsc) = malloc (NScSet[c] * sizeof (double **));
   alloc_chk (WHERE, (*xsc));
   for (s = 0; s < NScSet[c]; ++s)
   {
      (*xsc)[s] = malloc (NLoci * sizeof (double *));
      alloc_chk (WHERE, (*xsc)[s]);
      for (j = 0; j < NLoci; ++j)
      {
         (*xsc)[s][j] = malloc ((PTgg[NPrGam[c][s]] + 1) * sizeof (double));
         alloc_chk (WHERE, (*xsc)[s][j]);
      }
   }
   for (s = 0; s < NScSet[c]; ++s)
      for (j = 0; j < NLoci; ++j)
         for (i = 0; i <= PTgg[NPrGam[c][s]]; ++i)
            (*xsc)[s][j][i] = 0.0;

   return;
}

void free_xscor (int c, double ****xsc)
{
   int   s, j;
   if ((*xsc))
   {
      for (s = 0; s < NScSet[c]; ++s)
         if ((*xsc)[s])
         {
            for (j = 0; j < NLoci; ++j)
               if ((*xsc)[s][j])
                  free ((*xsc)[s][j]);
            free ((*xsc)[s]);
         }
      free ((*xsc));
      *xsc = NULL;
   }
   return;
}
