/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*****************************************************************************
  program name: lm_auto.c

     Version to try to adapt to a quantitative trait.

******************************************************************************/

/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_auto"

#include "autozyg.h"
#include "auto_chk.c"

#define AUTO_CKPNTS 1000

void  do_auto_sweep (void);

static int nsw;
static int mc_itno;

static int *gaga;                       /* [g]                            */
static int *npt;                        /* [g+1]  no. of patterns for
                                         *        gametes                 */
static int **iindex;                    /* [g+1][cd+1]                    */
static int **label;                     /* [g+1][pt+1]                    */
static int **list;                      /* [g+1][pt+1]                    */
static int nwpt;                        /* number of binary patterns,
                                         *   and of windows               */
static int *gw, *wloc;                  /* binary return for locus in
                                         *   window                       */
static double ****wscore;               /* window scores, each cmpt,
                                         * each scoreset, each window,
                                         *   each pattern
                                         *   assume 1 compnt, 1 scoreset  */
static int **count;                     /* [nc][nl]                       */
static double ****nscore;               /* [nc][ns][nl][pt]  ibd counts:
                                         *   ea component, ea scoreset,
                                         *   ea locus, ea pattern         */
static int tot_count = 0;               /* score by component and locus   */

static double *sss;                     /* [nl]                           */
static double ss;
static int ***recounts;                 /* [nc][nl][6] counts of recombs  */
static double **tot_log_prob_seg;
static double **tot_log_prob_pen;
static double **temp_log_prob_seg;
static double **prob;

static int chkitvl = AUTO_CKPNTS;

static FILE *oScorF;

int main (int argc, char *argv[])
{
   int   i, j, jj, k, ng, s, c, t;

   FILE *oSeedF;
   FILE *ibdF = NULL;

   nsw = setupAutozyg (argc, argv, PNAM, 0);
   setupSampling ();
   prob = setupInherit (0);

   /* set up probands for gene scoring */
   /* if the number of components read is less than the last one for which
      there are proband gametes, reset the number of components to score      */

   NScCmp = set_probgam (n_compnts);

   /* set up recombination patterns and initial probs */

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);

   tot_count = 0;

   /* gaga[], iindex[][], label[][], list[][], npt[] are malloced for the
      number of gametes in the largest set */
   /*  alloc of count and sss added here, EAT 2/2002       */

   /*   alloc_az_prob (&count, &sss);  */
   alloc_ibd (&gaga, &iindex, &label, &list, &npt, &count, &nscore, &sss);

   /* count total number of patterns ----
      modified EAT, March 2001, now will do only full set of patterns
      for each number of gametes  */

   /* probability now computed separately by component  */

   for (c = 0; c < n_compnts; ++c)
   {
      for (i = 0; i <= NMarks; ++i)
      {
         tot_log_prob_seg[c][i] = 0.0;
         tot_log_prob_pen[c][i] = 0.0;
      }
      for (j = 0; j <= NMarks; ++j)
      {
         count[c][j] = 0;
         for (jj = 0; jj < 6; ++jj)
            recounts[c][j][jj] = 0;
         for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
            for (k = 0; k <= PTgg[NPrGam[c][s]]; ++k)
               nscore[c][s][j][k] = 0.0;
      }
   }

   tot_count = 0;

   /* count total number of patterns */

   for (j = 1; j <= MxGam; ++j)
      npt[j] = index_ibd (j, 0, 0, iindex[j], label[j], list[j], ibdF);

   pr_npatsets (npt);

   nwpt = 1;
   for (j = 0; j < NLocWndw; ++j)
      nwpt *= 2;

   if (NLocWndw)
   {
      alloc_wndw (&gw, &wloc, &wscore, NWndw, nwpt);

      for (c = 0; c < NScCmp; ++c)
      {
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NWndw; ++j)
               for (k = 0; k < nwpt; ++k)
                  wscore[c][s][j][k] = 0.0;
      }
   }
   else
   {
      wloc = malloc ((NLoci + 1) * sizeof (int));
      alloc_chk (WHERE, wloc);
   }

   alloc_logprob ();

   /* open output scores file, if requested */

   if (FilIt)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   /* try to restart if a rescue file is specified */

   t = 0;
   auto_restart (nscore, tot_log_prob_pen, tot_log_prob_seg, prob,
                 count, recounts, &tot_count, &t);

   prntStatusBMC ();

   /* now do burn-in iterations */

   printf (" Now starting %d iterations of burn-in\n", NBrnIt);
   for (t = 1; t <= NBrnIt; t++)
   {
      if (!(t % ChkIt))
      {
         printf (" Starting burn-in iteration %d\n", t);
         fflush (stdout);
      }
      sampling (t, 0, NMarks, nsw, nFGL, prob);
   }

   /* now do MCMC sampling, and score results */

#ifdef AUTO_DISP                        /* may not want to link graphics fncs */
   if (GDispOpt)
   {
      i = 0;
      while (!NGDispNams[i])
         ++i;
      getIndIxs (i + 1, NGDispNams[i], GDispNam[i], GDispIx[i]);
      glutInit (&argc, argv);           /* initialize "glut" for X Windows    */
      gfmInitGFM ();                    /* set up display callbacks           */
      glutMainLoop ();                  /* start graphics; idle callback      */
   }                                    /*   function calls do_auto_sweep ()  */
#else
   if (GDispOpt)
      give_up (ERA_ngd);
#endif

   if (!GDispOpt)                       /* ignore graphics                    */
      for (mc_itno = 0; mc_itno < NIt; ++mc_itno)
         do_auto_sweep ();              /* do main mcmc iterations            */

   printf ("\n");
   prntStatusFMC ();

   for (c = 0; c < n_compnts; ++c)      /* now print results                  */
   {
      pr_cmpnthead (c);

      pr_log_prob_seg (0, NMarks, -cmplist[c + 1].nmeioses,
                       tot_log_prob_seg[c], tot_count);

      pr_log_prob_pen (0, NMarks, tot_log_prob_pen[c], tot_count);

      pr_reco_counts (0, NMarks, recounts[c]);

      /* for ea set, print probabilities for ea pattern for ea window  */

      for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
      {
         /* format and print headings  */

         pr_probhead ();

         pr_pgamset (c, s);

         pr_lochead (chromo);

         /* format and print data  */

         ng = NPrGam[c][s];

         for (k = 0; k < npt[ng]; ++k)
         {
            ss = 0.0;
            for (j = 0; j < NLoci; ++j)
            {
               sss[j] = nscore[c][s][j][list[ng][k]] / count[c][j];
               ss += sss[j];
            }
            if (ss > 0.0001)
            {
               invert_label (label[ng][k], ng, gaga);
               pr_locprob (ng, gaga, sss, label[ng][k]);
            }
         }
         printf ("\n");

         if (NLocWndw)                  /* **** now output window scores */
         {
            if (NPtLab[c][s])
            {
               pr_prbwhead (c, s);
               pr_wndwpat (c, s);
               pr_wndwhead (NWndw);

               /* format and print data  */

               for (i = 0; i < nwpt; ++i)
               {
                  ss = 0.;
                  for (j = 0; j < NWndw; ++j)
                     if (wscore[c][s][j][i] > ss)
                        ss = wscore[c][s][j][i];
                  if (ss / NIt > 0.0001)
                  {
                     binary (i, NLocWndw, gw);
                     pr_wndwprob (c, s, NWndw, i, gw, wscore);
                  }
               }
               printf ("\n");
            }
         }
      }
   }
   /* ****  now we are done **** */
   fflush (stdout);

   /* Free up allocated memory */
   free_rescue_filenames (orescName);
   free_xxgg ();

   free_ibd (&gaga, &iindex, &label, &list, &npt, &count, &nscore, &sss);
   if (NLocWndw)
      free_wndw (&gw, &wloc, &wscore, NWndw, nwpt);
   else
      free (wloc);

   free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                &tot_log_prob_pen);
   locus_peel_free ();
   free_allocate_loci ();
   free_chromo (0);
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);
   free_logprob ();
   free_msmpl ();
   free_traits ();

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (FilIt)
      fclose (oScorF);

   if (Scan)
      free_perm ();

   exit (EXIT_SUCCESS);
}

void do_auto_sweep (void)
{
/* local variables: */
   int   t, c, s, i, j, jj, k;
   int   ng;

   /* now do the next MCMC sampling iteration */

   sampling (mc_itno, 0, NMarks, nsw, nFGL, prob);

   /* MCMC sampling step or scan */

   /* score results */

   if (!((mc_itno + 1) % ScorIt))
   {
      for (c = 0; c < n_compnts; ++c)
         for (j = NMarks; j >= 0; --j)
            ++count[c][j];

      for (c = 0; c < NScCmp; ++c)      /* for ea component & score set      */
         for (s = 0; s < NScSet[c]; ++s)
         {
            for (j = NMarks; j >= 0; --j) /* score ea locus                 */
            {
               jj = chromo[0][j];
               wloc[j] = score_genes (jj, NPrGam[c][s], gaga,
                                      iindex[NPrGam[c][s]],
                                      list[NPrGam[c][s]], ProbIx[c][s],
                                      ProbGam[c][s], nscore[c][s][j],
                                      NPtLab[c][s], PatLab[c][s]);
            }
            if (NLocWndw)
               score_windows (wscore[c][s], NLocWndw, wloc, NMarks);
         }
      /* score tot log-liks & recombs for all components */
      for (c = 0; c < n_compnts; ++c)
      {
         full_seg_prob (NMarks, c, recounts, temp_log_prob_seg);
         for (j = 0; j < NMarks; ++j)
         {
            tot_log_prob_pen[c][j] += log (prob[c][j]);
            tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];
         }
         tot_log_prob_pen[c][NMarks] += log (prob[c][NMarks]);
      }
      ++tot_count;
   }
   if (FilIt && !((mc_itno + 1) % FilIt)) /* tests for output to scores file */
      for (c = 0; c < n_compnts; ++c)
      {                                 /* outputs by component            */
         fprintf (oScorF, "%d ", c + 1);
         for (jj = 0; jj <= NMarks; ++jj)
         {
            j = revchro[0][jj];
            fprintf (oScorF, "%d ", count[c][j]); /* (counts, by name)  */
         }
         fprintf (oScorF, "\n");
         for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
         {                              /* then by proband gamete set      */
            ng = NPrGam[c][s];
            for (k = 0; k < npt[ng]; ++k)
            {                           /* then by label and pattern       */
               fprintf (oScorF, "%d ", s + 1);
               invert_label (label[ng][k], ng, gaga);
               fprintf (oScorF, "%d  ", label[ng][k]);
               for (i = 0; i < ng; ++i)
                  fprintf (oScorF, "%d ", gaga[i]);
               for (jj = 0; jj <= NMarks; ++jj) /* then by name            */
               {
                  j = revchro[0][jj];
                  fprintf (oScorF, " %e", nscore[c][s][j][list[ng][k]]);
               }
               fprintf (oScorF, "\n");
            }
         }
      }
   if (chkitvl && ((mc_itno + 1) % chkitvl == 0))
   {
      auto_dump (nscore, tot_log_prob_pen, tot_log_prob_seg, prob,
                 count, recounts, tot_count, t);
   }
}
