/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*************************************************************************
  program name: markerdrop.c    (previously make_data: EAT 1998-9)
  
  Program to simulate multi-point marker data conditional on a trait model
  (penetrances of the trait genotypes, trait allele frequencies) 
  or conditional on a partially specified trait-locus segregation pattern.

  Originally written to generate the icemod data -- see book section 10.1

  Updated from MORGAN_F1 to MORGAN 2.3.2  (EAT, Jan 2002)
    temporarily needs own  read_gen_dat3, trait_segs3 and next_marker_segs3
      as these are gone from MORGAN 2.3.2 and/or have not been updated
      in Msample/realize.c

  Updated to Morgan 2.4 (EAT, Feb 2002)
    get rid of extra routines. Use lm_auto statements. Renamed markerdrop.

  Version using trait model by Saonli Basu (10/2004) and 
          updated to MORGAN 2.6
  
  Updated to new nghd.g_dat structures; cleaned up;      EAT 10/2004
  Updated to Morgan 2.7
  
**************************************************************************/

/**
 * @defgroup markerdrop markerdrop
 * @ingroup GDrop
 * @{
 */

#define PNAM  "markerdrop"
#define IBD
#define MDP
#include "genedrop.h"                   /* MORGAN 2.5 version  EAT */
#include "autozyg.h"

int main (int argc, char *argv[])
{
   int   i, j, k;
   double new_like = 0;
   int   nt;
   int   nsw;

   FILE *oSeedF;

   nsw = setupAutozyg (argc, argv, PNAM, 0);
   setSampling ();

   /* (for sampler setup, trait is unlinked) */

   /* now set trait segregations from pedigree data for trait
      or from partial nheritance indicators at trait locus */

   if (PedDatTyp != INHERITANCE)        /* data is discrete trait values */
   {
      n_markers = NMarks;
      n_tloci = 1;
      locus_peel_init (0, 1);
      set_transmission (0, 0, 0);

      new_like = locus_mjrpeel (1.0, 1.0, 1, 0, NULL, &loci[0]);
      printf (" Likelihood = %f\n\n", new_like);
   }
   else
      set_trait_segs (nsw);             /* data is inheritance indicators
                                         * at trait locus */
#if 0
   printf ("\n list of init segs");
   for (i = 0; i < pedsize; ++i)
   {
      printf ("\n %d %s;", nghd[i].index, nghd[i].self);
      printf ("  %2d %2d", xx1[0][i], xx0[0][i]);
   }
   fflush (stdout);
#endif

#if 0
   printf ("%d  %d  %d  %d\n", atoi (nghd[k].self), trtphen[k],
           loci[0].seg[0][k], loci[0].seg[1][k]);
   fflush (stdout);
#endif

   /* set seeds for marker simulation */

   getSeeds (TRAIT);                    /* saves trait seeds, */
   assgnSeeds (MARK);                   /* sets marker seeds  */
   setSeeds (MARK);

#if 0
   printf ("\n check %d switches and trait indicators", nsw);
   fflush (stdout);
   for (i = 1; i <= nsw; ++i)
   {
      j = (meiosis[i].ind).index;
      if (meiosis[i].sex == 0)
         xxx = xx0;
      else
         xxx = xx1;
      printf ("\n sw %d ind %d %d %s sx %d xxx %d", i, j,
              nghd[j].index, nghd[j].self, meiosis[i].sex, xxx[0][j]);
   }
   fflush (stdout);

   nt = revchro[0][0];                  /* position of trait */
   printf (" \n trait interval %d recom to next marker malrec= %f femrec = %f",
           nt, xecomo[1][0][nt], xecomo[0][0][nt]);
   fflush (stdout);
#endif

   nt = revchro[0][0];                  /* position of trait */

/* set other markers to the right  */
   if (nt < NMarks)
   {
      next_marker_segs (nsw, 0, nt + 1, nt);
      for (j = nt + 1; j < NMarks; ++j)
         next_marker_segs (nsw, j, j + 1, j);
   }

/* set other markers to the left */
   if (nt > 0)
   {
      next_marker_segs (nsw, 0, nt, nt - 1);
      for (j = nt; j > 1; --j)
         next_marker_segs (nsw, j, j - 1, j - 2);
   }

#if 0
   printf ("\n \n segregations at all loci on whole pedigree:");
   for (i = 0; i < pedsize; ++i)
   {
      if (nghd[i].pa->self != UNKNOWN)
      {
         printf ("\n %s", nghd[i].self);
         for (j = 0; j <= NMarks; ++j)
            printf ("  %d %d", xx1[j][i], xx0[j][i]);
      }
   }
   fflush (stdout);
#endif

/* Now set gene identities (founder gene labels) in all individuals 
   at all marker loci, in accordance with simulated meiosis indicators.
 */

   for (j = 0; j <= NMarks; ++j)
      set_FGL (j);
   fflush (stdout);

   if (PrintLevel >= 2)
   {
      printf ("\n assigned FGL in all listed individuals:");
      printf ("\n trait locus, followed by %d marker loci", NMarks);
      for (i = 0; i < NObs; ++i)
      {
         k = ObsIx[i];
         printf ("\n %s", nghd[k].self);
         for (j = 0; j <= NMarks; ++j)
            printf ("  %d %d", gg1[j][k], gg0[j][k]);
      }
      fflush (stdout);
   }

/* Now set the simulated genotypes at marker loci, in accord with original
   data availability.
 */

   for (j = 1; j <= NMarks; ++j)
      set_genotypes (nFound, j, Qq[j]);

   printf
      ("\n\n assigned marker genotypes in accordance with data availability:");
   fflush (stdout);

   for (i = 0; i < NObs; ++i)
   {
      k = ObsIx[i];
      printf ("\n %s", nghd[k].self);
      for (j = 1; j <= NMarks; ++j)
         printf ("  %d %d", gg1[j][k], gg0[j][k]);
   }
   printf ("\n\n");

   free_xxgg ();

   if (MarkSeedsOut || TraitSeedsOut)   /* if seeds are to be output,       */
   {
      getSeeds (MARK);                  /* saves marker seeds               */
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   fflush (stdout);

   exit (EXIT_SUCCESS);
}

/** @} */
