/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*         errspars.h:  error messages produced by Pars/ functions            */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef ERRSPARS_H
#define ERRSPARS_H

/**
 * @addtogroup parse
 * @{
 */

#define E 'E'
#define W 'W'
#define F 'F'
#define Q 'Q'

static int      n_warn = 0;
static int      n_err = 0;

/**
 * Parse error codes. error codes which result in the error messages and error
 * operations defined in proc_errors below.
 */
enum parsEnum {
      ERR_NUL,
      ERR_FNU,
      ERR_PCL,
      ERR_FCL,
      ERF_EPF,
      ERR_EIF,
      ERR_OPF,
      ERR_CLF,
      ERR_SIF,
      ERR_LTL,
      ERR_KEY,
      ERR_OLD,
      ERR_DAT,
      ERR_HEX,
      ERR_STR,
      ERR_NML,
      ERR_BGM,
      ERR_PGP,
      ERR_BNN,
      ERR_AIL,
      ERR_XTR,
      ERR_NOT,
      ERR_RED,
      ERR_PED,
      ERR_PEX,
      ERR_ALU,
      ERR_ALP,
      ERR_ALC,
      ERR_ALO,
      ERR_PPI,
      ERR_PPR,
      ERR_PF1,
      ERR_TRI,
      ERR_TRK,
      ERR_WTI,
      ERR_WT2,
      ERR_PF2,
      ERR_PN3,
      ERR_GND,
      ERR_OPA,
      ERR_ASG,
      ERR_IGG,
      ERR_ORD,
      ERR_STT,
      ERR_SCN,
      ERR_MNW,
      ERR_MXW,
      ERR_SSU,
      ERR_NSI,
      ERR_SIP,
      ERR_SCI,
      ERR_SBI,
      ERR_SPI,
      ERR_BVE,
      ERR_SNI,
      ERA_CET,
      ERA_NAI,
      ERA_NCI,
      ERA_NGI,
      ERA_NRI,
      ERA_PCE,
      ERA_PCS,
      ERR_PRI,
      ERR_FFO,
      ERR_FFD,
      ERR_MNR,
      ERR_FFR,
      ERR_LVM,
      ERR_VAM,
      ERR_COM,
      ERR_CFM,
      ERR_CFO,
      ERD_CHO,
      ERD_MK0,
      ERD_MKM,
      ERD_TR0,
      ERD_TRX,
      ERD_HRX,
      ERD_TLX,
      ERD_TRM,
      ERD_RSM,
      ERD_RCM,
      ERD_RST,
      ERD_RSL,
      ERD_RSI,
      ERD_RMP,
      ERA_RLW,
      ERK_KNN,
      ERK_RCP,
      ERK_DNM,
      ERK_NRF,
      ERK_XRF,
      ERA_PPW,
      ERA_PPN,
      ERD_RCS,
      ERD_MDM,
      ERD_MRM,
      ERD_MPM,
      ERD_MXT,
      ERD_O1M,
      ERD_TMM,
      ERD_MDO,
      ERD_MFO,
      ERA_MES,
      ERA_MXA,
      ERA_LBR,
      ERA_GER,
      ERA_MTR,
      ERA_RST,
      ERA_RIS,
      ERA_RLS,
      ERA_RIT,
      ERA_RLT,
      ERA_SPI,
      ERA_RPI,
      ERA_BGP,
      ERD_MPO,
      ERD_MPD,
      ERD_TDO,
      ERD_TFO,
      ERA_TIO,
      ERA_TFO,
      ERA_TDO,
      ERA_TPO,
      ERA_TID,
      ERA_TFD,
      ERA_TDD,
      ERA_TIN,
      ERA_WJP,
      ERA_OXP,
      ERA_OXC,
      ERA_OJP,
      ERA_OJC,
      ERA_OUP,
      ERA_OUC,
      ERA_WDX,
      ERA_OAP,
      ERA_OAC,
      ERA_OIP,
      ERA_OIC,
      ERA_TPW,
      ERD_MKX,
      ERD_MKO,
      ERD_TRO,
      ERD_IHO,
      ERD_TLO,
      ERD_DIH,
      ERD_DTR,
      ERD_DTW,
      ERD_REF,
      ERD_RTL,
      ERD_RIL,
      ERD_DMK,
      ERD_TTL,
      ERD_ITL,
      ERD_MTL,
      ERR_TRP,
      ERR_IRP,
      ERD_MLO,
      ERD_TLA,
      ERD_AFO,
      ERD_SMA,
      ERD_MAF,
      ERD_TAF,
      ERD_WNM,
      ERD_TGM,
      ERD_TGR,
      ERD_AVR,
      ERD_RVR,
      ERD_VMN,
      ERA_BLS,
      ERA_MDL,
      ERA_BMD,
      ERA_RMD,
      ERA_RMN,
      ERA_MNO,
      ERA_PAL,
      ERA_UAL,
      ERA_DIN,
      ERA_DMN,
      ERA_CMP,
      ERA_SCS,
      ERA_WPO,
      ERA_OAI,
      ERA_OSC,
      ERA_OSP,
      ERA_PGL,
      ERA_PGX,
      ERA_BGM,
      ERA_DPG,
      ERA_DAI,
      ERR_EIM,
      ERF_PE0,
      ERF_PE1,
      ERF_PE2,
      ERF_IPR,
      ERF_NIN,
      ERF_NRL,
      ERR_STN,
      ERR_SFN,
      ERR_TRF,
      ERR_FFF,
      ERR_SEL,
      ERR_STV,
      ERR_VA0,
      ERR_VR0,
      ERR_VSA,
      ERR_VSR,
      ERR_VAN,
      ERR_VRN,
      ERR_CSA,
      ERR_CSR,
      ERR_CFA,
      ERR_CFR,
      ERR_CAF,
      ERR_CRF,
      ERR_CAZ,
      ERR_CRZ,
      ERR_PDA,
      ERR_PDR,
      ERR_ET1,
      ERR_ET2,
      ERR_PT1,
      ERK_KIN,
      ERK_DRF,
      ERD_SIM,
      ERD_CHR,
      ERD_MTT,
      ERD_MIT,
      ERD_TTM,
      ERD_TIM,
      ERD_MHT,
      ERD_THM,
      ERD_MCT,
      ERD_TNF,
      ERD_TUL,
      ERD_MSM,
      ERD_NFM,
      ERD_NMA,
      ERD_NFA,
      ERD_MMM,
      ERA_1MA,
      ERA_MN1,
      ERA_MNL,
      ERD_MTM,
      ERD_TFM,
      ERD_TMA,
      ERD_TFA,
      ERA_MXM,
      ERA_MBM,
      ERA_ITI,
      ERA_DBE,
      ERA_NBE,
      ERA_XEI,
      ERA_DEI,
      ERA_NDI,
      ERA_MIO,
      ERA_WIP,
      ERA_NAT,
      ERD_MMX,
      ERD_TID,
      ERD_MID,
      ERA_TPM,
      ERA_IPM,
      ERA_NMP,
      ERA_GMP,
      ERD_LUL,
      ERD_NML,
      ERD_MFR,
      ERD_MFN,
      ERD_MFX,
      ERD_TAV,
      ERD_TRV,
      ERD_NAR,
      ERD_SVR,
      ERD_NTL,
      ERD_TFR,
      ERD_TFN,
      ERD_TFX,
      ERD_NGM,
      ERD_MTD,
      ERA_ZMD,
      ERA_SMD,
      ERD_SEE,
      ERD_OTS,
      ERD_OMS,
      ERA_OSS,
      ERA_OPS,
      ERD_SSD,
      ERD_STD,
      ERA_SSP,
      ERA_SSS,
      ERF_OPF,
      ERD_OST,
      ERD_OSM,
      ERA_USS,
      ERA_USP,
      ERA_SIM,
      ERA_SEL,
      ERA_MSL,
      ERA_MTI,
      ERA_MST,
      ERA_OOM,
      ERA_OOS,
      ERA_NMD,
      ERA_BND,
      ERA_BNM,
      ERA_BMA,
      ERA_WMA,
      ERA_MPG,
      ERA_PIG,
      ERA_MLW,
      ERA_MSS,
      ERA_BPC,
      ERA_NPP,
      ERA_ZUT,
      ERA_LWO,
      ERA_NPV,
      ERA_RPV,
      ERA_OPV,
      ERA_MPV,
      ERA_PNI,
      ERA_MMT,
      ERA_MMA,
      ERA_NGE,
      ERA_NIT,
      ERF_OSF,
      ERA_NCS,
      ERA_WIM,
      ERA_WLM,
      ERA_NSU,
      ERA_WIT,
      ERA_WLT,
      ERA_IPD,
      ERA_IPT,
      ERA_BIR,
      ERA_BI2,
      ERA_BH2,
      ERA_BRR,
      ERA_NEI,
      ERA_NER,
      ERA_NEJ,
      ERA_NEK,
      ERA_DRP,
      ERA_TEI,
      ERA_TE2,
      ERA_TER,
      ERG_NDN,
      ERG_TMC,
      ERG_TMI,
      ERR_FFL,
      ERR_FFN,
      ERR_CHG,
      ERR_EIP,
      ERR_OEF,
      ERR_EIE,
      ERR_OBS,
      ERR_MAX,
      ERR_FAT,
      ERR_QIT };

static int err_typ = 0;

static char errnumb[80];
static char * err_txt;             /*ptr to text for txtflg below       */

/* error message table                                                  */

/* error level is   'F' causes immediate termination of program         */
/*                  'E' causes termination of program after all input   */
/*                      statements are processed                        */
/*                  'W' warning statement only                          */

struct _errors {
    char * msg;                         /*text for error                */
    char   level;                       /*error level                   */
    int    tokflg;                      /*whether tokens are to be prntd*/
    int    txtflg;                      /*whether an arg is to be prntd */
    int    statflg;                     /*whether correct form printed  */
               } *perr;

static struct _errors proc_errors[] = {
{"***"                                                          ,F,N,N,N}, /*ERR_NUL*/
{" File type \"%s\" from command line not recognized"           ,F,N,Y,N}, /*ERR_FNU*/
{" File type \"%s\" from command line not used by this program" ,W,N,Y,N}, /*ERR_PCL*/
{" Incorrect specification of files in command line.  Correct form is:\n\n"
 "   %s"                                                        ,F,N,Y,N}, /*ERR_FCL*/
{" No pedigree file specified in command line or \"input pedigree file\" statement"
                                                                ,E,N,N,N}, /*ERF_EPF*/
{" No parameter file specified in command line"                 ,F,N,N,N}, /*ERR_EIF*/
{" Can't open %s"                                               ,E,N,Y,N}, /*ERR_OPF*/
{" Error when closing %s"                                       ,W,N,Y,N}, /*ERR_CLF*/
{" No parameter statements in file \"%s\""                      ,W,N,Y,N}, /*ERR_SIF*/
{" %s\n\n"
 "    Line is too long;  the maximum number of characters is " PMAXLINLEN
                                                                ,F,N,Y,N}, /*ERR_LTL*/
{" Unrecognizable word or word order"                           ,E,Y,N,N}, /*ERR_KEY*/
{" This statement has been replaced;\n"
 "    Please refer to user documentatation for current statements"
                                                                ,E,Y,N,N}, /*ERR_OLD*/
{" Missing data in parameter statement"                         ,E,Y,N,Y}, /*ERR_DAT*/
{" Format error in hexadecimal data value"                      ,E,Y,N,Y}, /*ERR_HEX*/
{" Expected string in statement"                                ,E,Y,N,Y}, /*ERR_STR*/
{" Names of individuals may have a maximum of %s characters:\n"
 "    name(s) truncated"                                        ,W,Y,Y,N}, /*ERR_NML*/
{" \"%s\" not an integer;  expected pairs of names and integers"
                                                                ,E,Y,Y,Y}, /*ERR_BGM*/
{" Expected pairs of names and integers"                        ,E,Y,N,Y}, /*ERR_PGP*/
{" Expected a positive integer, not \"%s\""                     ,E,Y,Y,Y}, /*ERR_BNN*/
{" Wrong number of data;  expected sets of names and 2 x %s marker(s)"
                                                                ,E,Y,Y,Y}, /*ERR_AIL*/
{" Too many parameters in input statement"                      ,E,Y,N,Y}, /*ERR_XTR*/
{" This statement is not implemented for program \"%s\""        ,W,Y,Y,N}, /*ERR_NOT*/
{" Overrides previous statement of same type"                   ,W,Y,N,N}, /*ERR_RED*/
{" Pedigree size must be greater than 2"                        ,E,Y,N,N}, /*ERR_PED*/
{" Pedigree size is greater than default maximum size of %s"    ,W,Y,Y,N}, /*ERR_PEX*/
{" Overrides default maximum pedigree size of %s\n"
 "    Program \"unibig\" is recommended instead for large pedigrees"
                                                                ,W,Y,Y,N}, /*ERR_ALU*/
{" Overrides default maximum pedigree size of %s"               ,W,Y,Y,N}, /*ERR_ALP*/
{" Overrides default maximum component size of %s"              ,W,Y,Y,N}, /*ERR_ALC*/
{" Overrides default maximum number of observed individuals of %s"
                                                                ,W,Y,Y,N}, /*ERR_ALO*/
{" Preferred form of this statement is:\n"
 "        \"input pedigree record names  3  integers  I\""      ,W,Y,N,N}, /*ERR_PPI*/
{" Preferred form of this statement is:\n"
 "    \"input pedigree record names  3  integers  I  reals  J\""
                                                                ,W,Y,N,N}, /*ERR_PPR*/
{" Number of integers out of range"                             ,E,Y,N,N}, /*ERR_PF1*/
{" The position in the record for the integer trait data is out of range"
                                                                ,E,Y,N,N}, /*ERR_TRI*/
{" The position in the record for real (quantitative) trait data\n"
 "         is out of range"                                     ,E,Y,N,N}, /*ERR_TRK*/
{" There must be the same number of traits and corresponding record positions"
                                                                ,E,Y,N,N}, /*ERR_WTI*/
{" There must be the same number of traits and corresponding pairs of record positions"
                                                                ,E,Y,N,N}, /*ERR_WT2*/
{" Number of reals is out of range"                             ,E,Y,N,N}, /*ERR_PF2*/
{" Number of names must be 3"                                   ,E,Y,N,N}, /*ERR_PN3*/
{" Overrides previous gender present/absent statement"          ,W,Y,N,N}, /*ERR_GND*/
{" Overrides previous observed present/absent statement"        ,W,Y,N,N}, /*ERR_OPA*/
{" Overrides previous ignore gender statement"                  ,W,Y,N,N}, /*ERR_ASG*/
{" Overrides previous assign gender statement"                  ,W,Y,N,N}, /*ERR_IGG*/
{" Overrides previous record order statement"                   ,W,Y,N,N}, /*ERR_ORD*/
{" Overrides previous set trait data statement"                 ,W,Y,N,N}, /*ERR_STT*/
{" Overrides previous sample by scan/step statement"            ,W,Y,N,N}, /*ERR_SCN*/
{" Locus window size out of range;  must be > 1"                ,E,Y,N,N}, /*ERR_MNW*/
{" Locus window size out of range;  must be <= number of loci"  ,E,Y,N,N}, /*ERR_MXW*/
{" Overrides previous sampler setup statement"                  ,W,Y,N,N}, /*ERR_SSU*/
{" Number of sequential imputation samples for setup is out of range"
                                                                ,E,Y,N,N}, /*ERR_NSI*/
{" Frequency of sequential proposals is out of range"           ,E,Y,N,N}, /*ERR_SIP*/
{" Frequency of scores computation is out of range"             ,E,Y,N,N}, /*ERR_SCI*/
{" Number of burn-in iterations is out of range"                ,E,Y,N,N}, /*ERR_SBI*/
{" Number of pseudo-prior iterations is out of range"           ,E,Y,N,N}, /*ERR_SPI*/
{" Number of batches for MC variance estimation is out of range",E,Y,N,N}, /*ERR_VBE*/
{" Number of MCMC iterations is out of range"                   ,E,Y,N,N}, /*ERR_SNI*/
{" Number of times to compute estimates is out of range"        ,E,Y,N,N}, /*ERA_CET*/
{" Number of ascent iterations is out of range"                 ,E,Y,N,N}, /*ERA_NAI*/
{" Number of curvature iterations is out of range"              ,E,Y,N,N}, /*ERA_NCI*/
{" Number of gradient iterations is out of range"               ,E,Y,N,N}, /*ERA_NGI*/
{" Number of LRT statistics iterations is out of range"         ,E,Y,N,N}, /*ERA_NRI*/
{" Percent change for EM estimates is out of range"             ,E,Y,N,N}, /*ERA_PCE*/
{" Percent change for SA estimates is out of range"             ,E,Y,N,N}, /*ERA_PCS*/
{" Print level is out of range"                                 ,E,Y,N,N}, /*ERR_PRI*/
{" Fixed effect number is out of range"                         ,E,Y,N,N}, /*ERR_FFO*/
{" Repeated fixed effect number \"%s\" in statement"            ,W,Y,Y,N}, /*ERR_FFD*/
{" Overrides previous mean for the trait"                       ,W,Y,N,N}, /*ERR_MNR*/
{" Overrides previous fixed effect levels for the trait"        ,W,Y,N,N}, /*ERR_FFR*/
{" Too many starting values for fixed effect levels"            ,E,Y,N,N}, /*ERR_LVM*/
{" Too many starting values for variances"                      ,E,Y,N,N}, /*ERR_VAM*/
{" Too many starting values for covariances or correlations"    ,E,Y,N,N}, /*ERR_COM*/
{" Too many fit flags for covariances or correlations"          ,E,Y,N,N}, /*ERR_CFM*/
{" Statement ignored.  %s specified in command line"            ,W,Y,Y,N}, /*ERD_CFO*/
{" Chromosome number is out of range"                           ,E,Y,N,N}, /*ERD_CHO*/
{" At least one marker expected"                                ,E,Y,N,N}, /*ERD_MK0*/
{" Too many markers"                                            ,E,Y,N,N}, /*ERD_MKM*/
{" Bad number of traits"                                        ,E,Y,N,N}, /*ERD_TR0*/
{" Too many traits are to be simulated or selected for this program"
                                                                ,E,Y,N,N}, /*ERD_TRX*/
{" Too many inheritance indicators are selected for this program"
                                                                ,E,Y,N,N}, /*ERD_HRX*/
{" Too many tlocs are to be simulated or selected for this program"
                                                                ,E,Y,N,N}, /*ERD_TLX*/
{" Too many traits"                                             ,E,Y,N,N}, /*ERD_TRM*/
{" Overrides previous marker simulation or selection statement" ,W,Y,N,N}, /*ERD_RSM*/
{" Overrides previous marker simulation or selection statement\n"
 "        for this chromosome"                                  ,W,Y,N,N}, /*ERD_RCM*/
{" Overrides previous select traits or simulate traits statement"
                                                                ,W,Y,N,N}, /*ERD_RST*/
{" Overrides previous tloc simulation or selection statement"   ,W,Y,N,N}, /*ERD_RSL*/
{" Overrides previous inheritance selection statement"          ,W,Y,N,N}, /*ERD_RSI*/
{" Overrides previous map statement"                            ,W,Y,N,N}, /*ERD_RMP*/
{" Overrides previous locus window statement"                   ,W,Y,N,N}, /*ERA_RLW*/
{" Wrong number of names:  %s of probands required"             ,E,Y,Y,N}, /*ERK_KNN*/
{" Overrides previous statement for this component"             ,W,Y,N,N}, /*ERK_RCP*/
{" Duplicate name \"%s\" for computation of kinship coefficient",E,Y,Y,N}, /*ERK_DNM*/
{" Recombination frequencies cannot be less than 0"             ,E,Y,N,N}, /*ERK_NRF*/
{" Recombination frequencies must be <= .5"                     ,E,Y,N,N}, /*ERK_XRF*/
{" A small positive value was substituted for pseudo-priors of 0.0"
                                                                ,W,Y,N,N}, /*ERA_PPW*/
{" Pseudo-priors may not be negative"                           ,E,Y,N,N}, /*ERA_PPN*/
{" Overrides previous statement for this chromosome"            ,W,Y,N,N}, /*ERD_RCS*/
{" Too many marker distances"                                   ,E,Y,N,N}, /*ERD_MDM*/
{" Too many marker recombination fractions"                     ,E,Y,N,N}, /*ERD_MRM*/
{" Too many marker positions"                                   ,E,Y,N,N}, /*ERD_MPM*/
{" Too many tlocs (trait loci)"                                 ,E,Y,N,N}, /*ERD_MXT*/
{" Since a single marker position is specified, there is no marker map"
                                                                ,W,Y,N,N}, /*ERD_O1M*/
{" There must be the same number of tlocs, markers\n"
 "        and distances, positions or recombination fractions"  ,E,Y,N,N}, /*ERD_TMM*/
{" Marker distances must be >= 0. and <= 500. cM"               ,E,Y,N,N}, /*ERD_MDO*/
{" Marker recombination fractions must be >= 0. and < .5"       ,E,Y,N,N}, /*ERD_MFO*/
{" Overrides previous statement specifying map estimation model"
                                                                ,W,Y,N,N}, /*ERA_MES*/
{" Overrides previous statement selecting algorithm for maximization "
                                                                ,W,Y,N,N}, /*ERA_MXA*/
{" Lower bounds for recombination fractions must be > 0. and < .1"
                                                                ,E,Y,N,N}, /*ERA_LBR*/
{" Genotyping error rate must be >= 0. and < 1."                ,E,Y,N,N}, /*ERA_GER*/
{" Overrides previous map type statement"                       ,W,Y,N,N}, /*ERA_MTR*/
{" Overrides previous \"compute ... statistics\" statement"     ,W,Y,N,N}, /*ERA_RST*/
{" Overrides previous IBD statistics measures"                  ,W,Y,N,N}, /*ERA_RIS*/
{" Overrides previous likelihood-ratio statistics measures"     ,W,Y,N,N}, /*ERA_RLS*/
{" Overrides previous IBD statistics tests"                     ,W,Y,N,N}, /*ERA_RIT*/
{" Overrides previous likelihood-ratio statistics tests"        ,W,Y,N,N}, /*ERA_RLT*/
{" Number of permutations must be > 0"                          ,E,Y,N,N}, /*ERA_SPI*/
{" It is recommended that the number of permutations is at least 50"
                                                                ,W,Y,N,N}, /*ERA_RPI*/
{" Number of gridpoints must be > 1"                            ,E,Y,N,N}, /*ERA_BGP*/
{" Marker positions must be > -500. and <= 500. cM"             ,E,Y,N,N}, /*ERD_MPO*/
{" Marker positions must be in increasing order"                ,E,Y,N,N}, /*ERD_MPD*/
{" Tloc distances must be >= 0. and <= 500. cM"                 ,E,Y,N,N}, /*ERD_TDO*/
{" Tloc recombination fractions must be >= 0. and < .5"         ,E,Y,N,N}, /*ERD_TFO*/
{" Tloc interval proportions must be > 0. and < 1."             ,E,Y,N,N}, /*ERA_TIO*/
{" Tloc recombination fractions must be > 0. and < .5"          ,E,Y,N,N}, /*ERA_TFO*/
{" Tloc distances must be > 0. and <= 500."                     ,E,Y,N,N}, /*ERA_TDO*/
{" Tloc positions must be >= 0. and <= 500."                    ,E,Y,N,N}, /*ERA_TPO*/
{" Tloc interval proportions must be strictly increasing"       ,E,Y,N,N}, /*ERA_TID*/
{" Tloc beginning, ending or external recombination fractions\n"
 "        must be strictly increasing"                          ,E,Y,N,N}, /*ERA_TFD*/
{" Tloc beginning, ending or external distances must be strictly increasing"
                                                                ,E,Y,N,N}, /*ERA_TDD*/
{" Interval number must be > 0 and < number of markers"         ,E,Y,N,N}, /*ERA_TIN*/
{" Pairs of tloc positions are expected"                        ,E,Y,N,N}, /*ERA_WJP*/
{" Overrides previous statement for beginning and/or ending map positions"
                                                                ,W,Y,N,N}, /*ERA_OXP*/
{" Overrides previous statement for this chromosome for beginning\n"
 "        and/or ending map positions"                          ,W,Y,N,N}, /*ERA_OXC*/
{" Overrides previous statement for joint test positions for these tlocs"
                                                                ,W,Y,N,N}, /*ERA_OJP*/
{" Overrides previous statement for this chromosome for joint test positions\n"
 "        for these tlocs"                                      ,W,Y,N,N}, /*ERA_OJC*/
{" Overrides previous statement for joint test positions for these tlocs\n"
 "        (one is unlinked)"                                    ,W,Y,N,N}, /*ERA_OUP*/
{" Overrides previous statement for this chromosome for joint test positions\n"
 "        for these tlocs (one is unlinked)"                    ,W,Y,N,N}, /*ERA_OUC*/
{" The number of beginning or ending map positions for the tloc\n"
 "        is not the same for males and females"                ,E,N,N,N}, /*ERA_WDX*/
{" Overrides interval proportions previously specified for all intervals"
                                                                ,W,Y,N,N}, /*ERA_OAP*/
{" Overrides proportions previously specified for this chromosome\n"
 "        for all intervals"                                    ,W,Y,N,N}, /*ERA_OAC*/
{" Overrides proportions previously specified for one or more intervals"
                                                                ,W,Y,N,N}, /*ERA_OIP*/
{" Overrides proportions previously specified for this chromosome\n"
 "        for one or more intervals"                            ,W,Y,N,N}, /*ERA_OIC*/
{" Window width, the number of hypothesized trait positions on either side\n"
 "        of current position, is out of range"                 ,E,Y,N,N}, /*ERA_TPW*/
{" Too many markers to be simulated or selected for this program"
                                                                ,E,Y,N,N}, /*ERD_MKX*/
{" Marker number out of range"                                  ,E,Y,N,N}, /*ERD_MKO*/
{" Trait number out of range"                                   ,E,Y,N,N}, /*ERD_TRO*/
{" Inheritance number out of range"                             ,E,Y,N,N}, /*ERD_IHO*/
{" Tloc (trait locus) number out of range"                      ,E,Y,N,N}, /*ERD_TLO*/
{" Duplicate inheritance number \"%s\" in list"                 ,E,Y,Y,N}, /*ERD_DIH*/
{" Duplicate trait number \"%s\" in list"                       ,E,Y,Y,N}, /*ERD_DTR*/
{" Duplicate tloc number \"%s\" in list"                        ,W,Y,Y,N}, /*ERD_DTW*/
{" Replaces effect(s) already set for trait \"%s\""             ,W,Y,Y,N}, /*ERD_REF*/
{" Replaces tloc(s) already set for trait(s)"                   ,W,Y,N,N}, /*ERD_RTL*/
{" Replaces tloc(s) already set for inheritance(s)"             ,W,Y,N,N}, /*ERD_RIL*/
{" Duplicate marker number \"%s\" in list"                      ,E,Y,Y,N}, /*ERD_DMK*/
{" There must be the same number of traits and tlocs"           ,E,Y,N,N}, /*ERD_TTL*/
{" There must be the same number of inheritances and tlocs"     ,E,Y,N,N}, /*ERD_ITL*/
{" Too many tlocs (trait loci) for trait;  maximum is \"%s\""   ,E,Y,Y,N}, /*ERD_MTL*/
{" Trait data record position redefined for one or more traits"
                                                                ,W,Y,N,N}, /*ERR_TRP*/
{" Inheritance data record position redefined for one or more indicators"
                                                                ,W,Y,N,N}, /*ERR_IRP*/
{" Too few or too many marker alleles"                          ,E,Y,N,N}, /*ERD_MLO*/
{" Expected two alleles for trait locus"                        ,E,Y,N,N}, /*ERD_TLA*/
{" Allele frequencies cannot be less than 0."                   ,E,Y,N,N}, /*ERD_AFO*/
{" Sum of allele frequencies is not in range " Q_LIMITS         ,W,Y,N,N}, /*ERD_SMA*/
{" Overrides previous allele frequencies for marker(s)"         ,W,Y,N,N}, /*ERD_MAF*/
{" Overrides previous allele frequencies for tloc(s)"           ,W,Y,N,N}, /*ERD_TAF*/
{" There must be the same number of traits and corresponding tlocs"
                                                                ,E,Y,N,N}, /*ERD_WNM*/
{" There must be 3 mean values, one each for the (1 1), the (1 2) or (2 1),\n"
 "           and the (2 2) genotypes"                           ,E,Y,N,N}, /*ERD_TGM*/
{" Overrides previous genotype means for trait at tloc"         ,W,Y,N,N}, /*ERD_TGR*/
{" Overrides previous additive variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_AVR*/
{" Overrides previous residual variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_RVR*/
{" Negative variance"                                           ,E,Y,N,N}, /*ERD_VMN*/
{" L sampler probability must be >= 0 and <= 1."                ,E,Y,N,N}, /*ERA_BLS*/
{" Marker data error;  provide for each observed individual:\n"
 "     name and a pair of alleles for each marker"              ,E,Y,N,Y}, /*ERA_MDL*/
{" Negative name or allele \"%s\" in marker data"               ,E,Y,Y,Y}, /*ERA_BMD*/
{" Overrides previous marker data statement"                    ,W,Y,N,N}, /*ERA_RMD*/
{" Overrides previous marker names statement"                   ,W,Y,N,N}, /*ERA_RMN*/
{" No. of markers \"%d\" out of range"                          ,E,Y,Y,Y}, /*ERA_MNO*/
{" Marker data error for individual %s:  negative allele"       ,E,Y,Y,N}, /*ERA_PAL*/
{" Marker data error for individual %s:  both alleles for a marker \n"
 "    must be observed (non-zero) or unobserved (zero)"         ,E,Y,Y,N}, /*ERA_UAL*/
{" Duplicate name \"%s\" in marker data"                        ,E,N,Y,N}, /*ERA_DIN*/
{" Duplicate marker name \"%s\""                                ,E,N,Y,N}, /*ERA_DMN*/
{" Bad component number \"%s\""                                 ,E,Y,Y,N}, /*ERA_CMP*/
{" Bad scoreset number \"%s\""                                  ,E,Y,Y,N}, /*ERA_SCS*/
{" One or more IBD pattern labels are out of range, including \"%s\""
                                                                ,E,Y,Y,N}, /*ERA_WPO*/
{" Overrides previous list of individuals for this component and scoreset"
                                                                ,W,Y,N,N}, /*ERA_OAI*/
{" Overrides previous proband gametes for this component and scoreset"
                                                                ,W,Y,N,N}, /*ERA_OSC*/
{" Overrides previous IBD pattern labels for this component and scoreset"
                                                                ,W,Y,N,N}, /*ERA_OSP*/
{" There must be more than one proband gamete"                  ,E,Y,N,N}, /*ERA_PGL*/
{" Too many proband gametes;  %s permitted"                     ,E,Y,Y,N}, /*ERA_PGX*/
{" \"%s\" not a haplotype indicator;\n"
 "    must be \"0\" (maternal) or \"1\" (paternal)"             ,E,Y,Y,N}, /*ERA_BGM*/
{" Duplicate name and haplotype indicator:  %s"                 ,E,Y,Y,N}, /*ERA_DPG*/
{" Duplicate name \"%s\""                                       ,E,Y,Y,N}, /*ERA_DAI*/
{" Number of eigenvalues greater than pedigree size"            ,E,N,N,N}, /*ERR_EIM*/
{" \"input pedigree size\" statement missing"                   ,E,N,N,N}, /*ERF_PE0*/
{" Pedigree size is too large.  Use \"unibig\" instead for big pedigrees"
                                                                ,E,N,N,N}, /*ERF_PE1*/
{" Pedigree size too large.  Use \"allow pedigree size\" statement"
   "to override default"                                        ,E,N,N,N}, /*ERF_PE2*/
{" \"input pedigree record names ...\" statement missing"
                                                                ,E,N,N,N}, /*ERF_IPR*/
{" Expected \"input pedigree record names ...\" to specify at least %s integer(s)"
                                                                ,E,N,Y,N}, /*ERF_NIN*/
{" Expected \"input pedigree record names ...\" to specify at least %s real(s)"
                                                                ,E,N,Y,N}, /*ERF_NRL*/
{" Incorrect number of traits selected for program \"%s\""      ,E,N,Y,N}, /*ERR_STN*/
{" Incorrect number of fixed effects selected for program \"%s\""
                                                                ,E,N,Y,N}, /*ERR_SFN*/
{" A trait number is > the number of real numbers in the pedigree file"
                                                                ,E,N,N,N}, /*ERR_TRF*/
{" A fixed effect number is > the number of integers - 1 (for gender) in the pedigree file"
                                                                ,E,N,N,N}, /*ERR_FFF*/
{" Starting values given for non-selected traits or non-selected fixed effects"
                                                                ,W,N,N,N}, /*ERR_SEL*/
{" Starting value(s) given for trait mean or fixed effect levels\n"
 "     (will not be computed)"                                  ,W,N,N,N}, /*ERR_STV*/
{" Starting values must be given for the additive variances"    ,E,N,N,N}, /*ERR_VA0*/
{" Starting values must be given for the residual variances"    ,E,N,N,N}, /*ERR_VR0*/
{" Wrong number of starting values for additive variances"      ,E,N,N,N}, /*ERR_VSA*/
{" Wrong number of starting values for residual variances"      ,E,N,N,N}, /*ERR_VSR*/
{" Starting values for additive variances must be non-zero"     ,E,N,N,N}, /*ERR_VAN*/
{" Starting values for residual variances must be non-zero"     ,E,N,N,N}, /*ERR_VRN*/
{" Wrong no. of starting values for additive genetic covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CSA*/
{" Wrong no. of starting values for residual covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CSR*/
{" Wrong no. of fit/no-fit indicators for additive genetic covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CFA*/
{" Wrong no. of fit/no-fit indicators for residual covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CFR*/
{" Inconsistent independent groups in \"fit additive covariances/correlations\" statement"
                                                                ,E,N,N,N}, /*ERR_CAF*/
{" Inconsistent independent groups in \"fit residual covariances/correlations\" statement"
                                                                ,E,N,N,N}, /*ERR_CRF*/
{" Non-zero starting values for independent additive covariances ignored"
                                                                ,W,N,N,N}, /*ERR_CAZ*/
{" Non-zero starting values for independent residual covariances ignored"
                                                                ,W,N,N,N}, /*ERR_CRZ*/
{" Additive genetic covariance matrix is not positive definite" ,E,N,N,N}, /*ERR_PDA*/
{" Residual covariance matrix is not positive definite"         ,E,N,N,N}, /*ERR_PD1*/
{" \"compute eigenvalues\" overrides \"set eigenvalues\" statement"
                                                                ,W,N,N,N}, /*ERR_ET1*/
{" Eigenvalues will be computed"                                ,W,N,N,N}, /*ERR_ET2*/
{" Command line specification of peeling sequence file \n"
 "       overrides \"input peeling sequence file\""             ,W,N,N,N}, /*ERR_PT1*/
{" Expected at least one \"compute\" statement"                 ,E,N,N,N}, /*ERK_KIN*/
{" Duplicate recombination frequency in list"                   ,W,N,N,N}, /*ERK_DRF*/
{" Expected at least one \"simulate\" statement for markers and/or traits"
                                                                ,E,N,N,N}, /*ERD_SIM*/
{" Cannot specify selection/simulation of both unnamed and named chromosomes"
                                                                ,E,N,N,N}, /*ERD_CHR*/
{" \"set traits ... tlocs ...\" statement is missing"           ,E,N,N,N}, /*ERD_MTT*/
{" \"set inheritances ... tlocs ...\" statement is missing"     ,E,N,N,N}, /*ERD_MIT*/
{" Tlocs are not specified for simulated or selected trait(s);\n"
 "     please include \"set traits ... tlocs ...\" statement for trait %s"
                                                                ,E,N,Y,N}, /*ERD_TTM*/
{" Tlocs are not specified for the selected inheritance;\n"
 "     please include \"set inheritances ... tlocs ...\" statement for inheritance %s"
                                                                ,E,N,Y,N}, /*ERD_TIM*/
{" \"set inheritances ... tlocs ...\" statement is missing"     ,E,N,N,N}, /*ERD_MHT*/
{" Tlocs are not specified for selected inheritance;\n"
 "     please include \"set inheritance ... tlocs ...\" statement for inheritance %s"
                                                                ,E,N,Y,N}, /*ERD_THM*/
{" Tloc %s is mapped on more than one chromosome"               ,E,N,Y,N}, /*ERD_MCT*/
{" Map or unlinked specification is missing for tloc %s"        ,E,N,Y,N}, /*ERD_TNF*/
{" Tloc %s cannot be both mapped and unlinked"                  ,E,N,Y,N}, /*ERD_TUL*/
{" Statement missing for simulating/selecting markers %s\n"
 "    tlocs cannot be mapped without markers"                   ,E,N,Y,N}, /*ERD_MSM*/
{" No. of markers mapped differs for females and males %s"      ,E,N,Y,N}, /*ERD_NFM*/
{" No. of markers differs in the male and gender-unspecified maps %s"
                                                                 ,E,N,Y,N}, /*ERD_NMA*/
{" No. of markers differs in the female and gender-unspecified maps %s"
                                                                ,E,N,Y,N}, /*ERD_NFA*/
{" Marker map is missing %s"                                    ,E,N,Y,N}, /*ERD_MMM*/
{" One marker only is assumed, as there is no marker map %s"    ,W,N,Y,N}, /*ERA_1MA*/
{" Marker map expected, as selected marker's no. is > 1 %s"
                                                                ,E,N,Y,N}, /*ERA_MN1*/
{" Marker no. in select statement is greater than the number of markers mapped %s"
                                                                ,E,N,Y,N}, /*ERA_MNL*/
{" Tloc map (or unlinked specification) is missing %s"          ,E,N,Y,N}, /*ERD_MTM*/
{" No. of tlocs mapped differs for females and males %s"        ,E,N,Y,N}, /*ERD_TFM*/
{" No. of tlocs differs in the male and gender-unspecified maps %s"
                                                                ,E,N,Y,N}, /*ERD_TMA*/
{" No. of tlocs differs in the female and gender-unspecified maps %s"
                                                                ,E,N,Y,N}, /*ERD_TFA*/
{" Map statements are missing for the trait test positions %s"  ,E,N,Y,N}, /*ERA_MXM*/
{" There are no map statements for the starting and ending trait"
 " test positions %s"                                           ,E,N,Y,N}, /*ERA_MBM*/
{" Tloc id's differ for gender-specific or sex-averaged maps %s"
                                                                ,E,N,Y,N}, /*ERA_ITI*/
{" Beginning and/or ending tloc test positions are not specified %s\n"
 "    default values will be used"                              ,W,N,Y,N}, /*ERA_DBE*/
{" Beginning and/or ending tloc test positions are not specified %s\n"
 "    nor were default values requested"                        ,W,N,Y,N}, /*ERA_NBE*/
{" Interval proportions for tloc test positions will be ignored %s\n"
 "    (there is only one marker selected)"                      ,W,N,Y,N}, /*ERA_XEI*/
{" No interval proportions are specified %s\n"
 "    default tloc test positions will be used between markers" ,W,N,Y,N}, /*ERA_DEI*/
{" No interval proportions are specified %s\n"
 "    nor were default tloc test positions requested"           ,W,N,Y,N}, /*ERA_NDI*/
{" Marker interval number out of range %s\n"
 "    it may not be greater than one less than the number of markers mapped"
                                                                ,E,N,Y,N}, /*ERA_MIO*/
{" The number of interval proportions for mapping the tloc\n"
 "    is not the same for males and females %s"                 ,E,N,Y,N}, /*ERA_WIP*/
{" One trait must be specified in the \"select markers ...\" statement"
                                                                ,E,N,Y,N}, /*ERA_NAT*/
{" Marker no. in tloc map %s > no. of markers mapped on chromosome"
                                                                ,E,N,Y,N}, /*ERD_MMX*/
{" Tloc numbers in corresponding gender-specific and/or gender-unspecified maps\n"
 "   %s must be the same and in the same order"                 ,E,N,Y,N}, /*ERD_TID*/
{" Tlocs in corresponding gender-specific and/or gender-unspecified maps\n"
 "    %s must be in the same marker intervals"                  ,E,N,Y,N}, /*ERD_MID*/
{" There are no test (tloc and/or non-selected marker) positions;\n"
 "    program %s requires at least 1 test position"             ,E,N,Y,N}, /*ERA_TPM*/
{" Test (tloc and/or non-selected marker) positions missing for some intervals;\n"
 "    program %s requires test positions between (selected) markers"
                                                                ,E,N,Y,N}, /*ERA_IPM*/
{" There are no test (tloc and/or non-selected marker) positions;\n"
 "    program %s will compute likelihood at marker loci only"   ,W,N,Y,N}, /*ERA_NMP*/
{" There are no test (tloc and/or non-selected marker) positions;\n"
 "    program %s requires test positions for likelihood computation"
 "    for genotypic trait"                                      ,E,N,Y,N}, /*ERA_GMP*/
{" Tloc %s is specified as both mapped and unlinked"            ,E,N,Y,N}, /*ERD_LUL*/
{" Allele frequencies missing %s:  defaults are .5 .5"          ,W,N,Y,N}, /*ERD_NML*/
{" Allele frequencies missing %s"                               ,E,N,Y,N}, /*ERD_MFR*/
{" Allele frequencies normalized %s"                            ,W,N,Y,N}, /*ERD_MFN*/
{" Sum of allele frequencies %s is out of range;\n"
 "    correct or \"set normalized frequencies\""                ,E,N,Y,N}, /*ERD_MFX*/
{" Additive variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TAV*/
{" Residual variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TRV*/
{" Neither the additive nor residual variance is given for trait %s"
                                                                ,E,N,Y,N}, /*ERD_NAR*/
{" Sum of additive and residual variance for trait %s is 0.0"   ,E,N,Y,N}, /*ERD_SVR*/
{" Allele frequencies missing for tloc %s:  defaults are .5 .5" ,W,N,Y,N}, /*ERD_NTL*/
{" Allele frequencies missing for tloc %s"                      ,E,N,Y,N}, /*ERD_TFR*/
{" Allele frequencies normalized for tloc %s"                   ,W,N,Y,N}, /*ERD_TFN*/
{" Sum of allele frequencies for tloc %s is out of range;\n"
 "    correct or \"set normalized frequencies\""                ,E,N,Y,N}, /*ERD_TFX*/
{" Genotype means not given %s:  defaults are 0. 0. 0."         ,W,N,Y,N}, /*ERD_NGM*/
{" Marker to tloc distance is > marker to marker distance for tloc %s"
                                                                ,E,N,Y,N}, /*ERD_MTD*/
{" The distance between adjacent markers must be greater than 0. cM"
                                                                ,E,N,N,N}, /*ERA_ZMD*/
{" Distance between adjacent markers < %s cM;\n"
 "     See \"limit recombination fractions ...\" statement"     ,W,N,Y,N}, /*ERA_SMD*/
{" Congruential seed (1st seed) must be odd,\n"
 " Tausworthe seed (2nd seed) must be non-zero"                 ,W,Y,N,N}, /*ERD_SEE*/
{" Overrides \"output tloc seeds only\" statement"              ,W,Y,N,N}, /*ERD_OTS*/
{" Overrides \"output marker seeds only\" statement"            ,W,Y,N,N}, /*ERD_OMS*/
{" Overrides \"output sampler seeds only\" statement"           ,W,Y,N,N}, /*ERA_OSS*/
{" Overrides \"output permutation seeds only\" statement"       ,W,Y,N,N}, /*ERA_OPS*/
{" Default marker seeds are used"                               ,W,N,N,N}, /*ERD_SSD*/
{" Default trait seeds are used"                                ,W,N,N,N}, /*ERD_STD*/
{" Default permutation seeds are used"                          ,W,N,N,N}, /*ERA_SSP*/
{" Default sampler seeds are used"                              ,W,N,N,N}, /*ERA_SSS*/
{" Output pedigree file is not named in parameter statement or command line"
                                                                ,E,N,N,N}, /*ERF_OPF*/
{" No output seed file named.  \"output tloc seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERD_OST*/
{" No output seed file named.  \"Output marker seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERD_OSM*/
{" No output seed file named.  \"output sampler seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERA_USS*/
{" No output seed file named.  \"output permutation seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERA_USP*/
{" The \"simulate ... markers ...\" statement is missing"       ,E,N,N,N}, /*ERA_SIM*/
{" The \"select ... markers ...\" statement is missing"         ,E,N,N,N}, /*ERA_SEL*/
{" The \"simulate tloc ...\" statement is missing"              ,E,N,N,N}, /*ERA_MSL*/
{" The \"select trait ...\"  or \"select inheritance ...\" statement"
 " is missing"                                                  ,E,N,N,N}, /*ERA_MTI*/
{" The \"select trait ...\"  statement is missing"              ,E,N,N,N}, /*ERA_MST*/
{" Only one chromosome may be simulated"                        ,E,N,N,N}, /*ERA_OOM*/
{" Only one chromosome may be selected"                         ,E,N,N,N}, /*ERA_OOS*/
{" No marker data given %s"                                     ,W,N,Y,N}, /*ERA_NMD*/
{" Wrong number of marker data:  provide name of each observed individual\n"
 "    together with a pair of alleles for each marker"          ,E,N,N,N}, /*ERA_BND*/
{" The number of marker names is not the same as the number of markers\n"
 "    mapped on the chromosome"                                 ,E,N,N,N}, /*ERA_BNM*/
{" Marker data error for individual %s:  alleles must be integers"
                                                                ,E,N,Y,N}, /*ERA_BMA*/
{" Marker data error for individual %s:\n"
 "    specified allele > number of allele frequencies for marker"
                                                                ,E,N,Y,N}, /*ERA_WMA*/
{" No proband gametes specified"                                ,E,N,N,N}, /*ERA_MPG*/
{" Too many proband gametes:  with locus window size, use pairs of proband gametes"
                                                                ,E,N,N,N}, /*ERA_PIG*/
{" Window IBD pattern labels for scoring are ignored,\n"
 "    as no locus window size is specified"                     ,W,N,N,N}, /*ERA_MLW*/
{" IBD pattern labels, but not proband gametes, are given for\n"
 "    one or more scoresets, including %s"                      ,W,N,Y,N}, /*ERA_MSS*/
{" IBD pattern label \"%s\" is out of range"                    ,E,N,Y,N}, /*ERA_BPC*/
{" Wrong number of pseudo-priors;  expected %s, one greater than the number of"
 "    test positions"                                           ,E,N,Y,N}, /*ERA_NPP*/
{" Pseudo-prior for the unlinked position (0) must be non-zero" ,E,N,N,N}, /*ERA_ZUT*/
{" Locus window size out of range:  must be > 1 and <= %s, the number of loci"
                                                                ,E,N,Y,N}, /*ERA_LWO*/
{" Wrong number of incomplete penetrances.\n"
 "    Provide one each for the (1 1) (1 2) and (2 2) genotypes" ,E,Y,N,Y}, /*ERA_NPV*/
{" Incomplete penetrances out of range;  values must be between 0. and 1."
                                                                ,E,Y,N,N}, /*ERA_RPV*/
{" Check incomplete penetrances:\n"
 "    expected genotype (1 1) <= genotype (1 2) <= genotype (2 2)"
                                                                ,W,Y,N,N}, /*ERA_OPV*/
{" Incomplete penetrances are missing.\n"
 "    Use \"set incomplete penetrances ...\" statement"         ,E,N,N,N}, /*ERA_MPV*/
{" Incomplete penetrances are ignored as there is no trait data",W,N,N,N}, /*ERA_PNI*/
{" \"set map estimation model ..\" statement is missing"        ,E,N,N,N}, /*ERA_MMT*/
{" Please choose hybrid algorithm or MCEM for maximization"     ,E,N,N,N}, /*ERA_MMA*/
{" Genotyping error rate must be specified for map estimation model with mistyping"
                                                                ,E,N,N,N}, /*ERA_NGE*/
{" Number of MC iterations must be specified"                   ,E,N,N,N}, /*ERA_NIT*/
{" Interim scores will not be saved:\n"
 "    output scores file not named in parameter statement or command line"
                                                                ,W,N,N,N}, /*ERF_OSF*/
{" Statement to \"compute IBD statistics\" \n"
 "    or \"compute likelihood-ratio statistics\" is missing"    ,E,N,N,N}, /*ERA_NCS*/
{" Specified likehood-ratio statistics measures are ignored\n"
 "    as IBD statistics are to be computed"                     ,W,N,N,N}, /*ERA_WIM*/
{" Specified IBD statistics measures are ignored\n"
 "    as likelihood-ratio statistics are to be computed"        ,W,N,N,N}, /*ERA_WLM*/
{" Cannot presently use Saffunaff measure with IBD normal test" ,E,N,N,N}, /*ERA_NSU*/
{" Specified likehood-ratio statistics tests are ignored\n"
 "    as IBD statistics are to be computed"                     ,W,N,N,N}, /*ERA_WIT*/
{" Specified IBD statistics tests are ignored\n"
 "    as likelihood-ratio statistics are to be computed"        ,W,N,N,N}, /*ERA_WLT*/
{" Trait type is assumed to be discrete"                        ,W,N,N,N}, /*ERA_IPD*/
{" Discrete trait type specification ignored:\n"
 "    data are inheritance indicators at trait locus"           ,W,N,N,N}, /*ERA_IPT*/
{" Incompatible pedigree data type for trait %s,\n"
 "    (see \"input pedigree record traits ... integers ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BIR*/
{" Incompatible pedigree data type for trait %s,\n"
 "    (see \"input pedigree record traits ... integer pairs ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BI2*/
{" Incompatible pedigree data type for inheritance indicator %s,\n"
 "    (see \"input pedigree record inheritance ... integer pairs ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BH2*/
{" Incompatible pedigree data type for quantitative trait %s,\n"
 "    (see \"input pedigree record traits ...  reals ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BRR*/
{" Not enough integers in pedigree record to include trait number %s"
                                                                ,E,N,Y,N}, /*ERA_NEI*/
{" Not enough reals in pedigree record to include trait number %s"
                                                                ,E,N,Y,N}, /*ERA_NER*/
{" Trait data was specified at position %s in the pedigree record\n"
 "    this is fewer than the number of integers in the record"  ,E,N,Y,N}, /*ERA_NEJ*/
{" A quantitative trait was specified at position %s in the pedigree record\n"
 "    this is fewer than the number of reals in the record"     ,E,N,Y,N}, /*ERA_NEK*/
{" Duplicate record position %s for trait\n"
 "    (see \"input pedigree record traits ...  integer pairs ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_DRP*/
{" Record position for trait %s assumed, as not specified in\n"
 "    (\"input pedigree record traits ...  integers ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TEI*/
{" Record position for inheritance indicator %s assumed, as not specified\n"
 "    in (\"input pedigree record inheritnce ...  integer pairs ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TE2*/
{" Record position for trait %s assumed, as not specified in\n"
 "    (\"input pedigree record traits ...  reals ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TER*/
{" Display of segregations and/or founder labels requested;\n"
 "    please specify names of individuals to be displayed"      ,E,N,N,N}, /*ERG_NDN*/
{" Graphical display requested for more than one pedigree component\n"
 "     The component %d only will be displayed"                 ,E,N,Y,N}, /*ERG_TMC*/
{" The display frequency is greater than the total number of MCMC iterations"
                                                                ,E,N,N,N}, /*ERG_TMI*/
{" The number of fixed effect levels in the pedigree file \n"
 "    is > the maximum no. defined in limdefs.h"                ,F,N,N,N}, /*ERR_FFL*/
{" The number of starting values given for fixed effect levels\n"
 "    is not the same as the number of levels found in the pedigree file"
                                                                ,W,N,N,N}, /*ERR_FFN*/
{" The largest component exceeds the maximum size permitted\n"
 "    Rerun with the \"allow component size\" statement"        ,F,N,N,N}, /*ERR_CHG*/
{" The number of eigenvalues in the \"set eigenvalues\" statement\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIP*/
{" Can't open eigenvalues file \"%s\""                          ,F,N,Y,N}, /*ERR_OEF*/
{" The number of eigenvalues in the eigenvalues file\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIE*/
{" The number of observed individuals exceeds the maximum number permitted\n"
 " Rerun with the \"allow observed individuals\" statement"     ,F,N,N,N}, /*ERR_OBS*/
{" Program quitting due to too many warnings and errors for parameter"
 " statements"                                                  ,Q,N,N,N}, /*ERR_MAX*/
{" Program quitting due to fatal error"                         ,Q,N,N,N}, /*ERR_FAT*/
{" Program quitting due to errors in parameter statements"      ,Q,N,N,N}};/*ERR_QIT*/

#undef E
#undef W
#undef F
#undef Q

#endif  /*ERRSPARS_H*/
