/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*           parsepars.h:  header file for the Pars/ library                  */
/*                         -  of file scope                                   */
/*                         -  included by Pars/ functions                     */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEPARS_H
#define  PARSEPARS_H

/**
 * @defgroup parse libpars
 * @ingroup Libraries
 * @{
 */

#define SIGSQ(r, c)  SigSq[(r) * (r - 1) /2 + c - 1]
#define TAUSQ(r, c)  TauSq[(r) * (r - 1) /2 + c - 1]
#define SIG_MODL(r, c)  SigModel[(r) * (r - 1) /2 + c - 1]
#define TAU_MODL(r, c)  TauModel[(r) * (r - 1) /2 + c - 1]

static FILTAB_PTR  pfloop, pft;
static FILE * pfile;


/***    limits overriding variables                                         ***/

static int    allw_pedsiz,  allw_cmpsiz,   allw_obsiz;

/***    record variables                                                    ***/

static int parentOrdIn  = UNDEF;
static int parentOrdOut = UNDEF;

/*** the following macro constants are used by proc_pars() and proc_drop()  ***/

#define NCH  0      /* no chromosome            */
#define CH   1      /* chromosome               */
#define NCMP 0      /* no component             */
#define CMP  1      /* component                */
#define NSC  0      /* no scoreset              */
#define SC   1      /* scoreset                 */
#define TLMAP  0    /* tloc map                 */
#define TLVMAP 1    /* tloc test position map   */

#define NO_CH_NO -1 /* chrom not needed         */

/* options for simulating or selecting markers, traits, tlocs, inheritance    */

#define SS_MKALL       1     /* all markers                                   */
#define SS_MKLIST      2     /* list of markers (or single marker)            */
#define SS_TRLIST      3     /* list of traits (or single trait)              */
#define SS_TLLIST      4     /* list of tlocs (or single tloc)                */
#define SS_INHER       5     /* list of inheritance indicators (or single)    */

/* options for setting trait loci                                             */

#define INHLIST_TLLIST 7     /* list of inheritances w list of corresp tlocs  */
#define TRLIST_TLLIST  8     /* list of traits w list of corresponding tlocs  */
#define TR_TLLIST      9     /* trait no. w list of tlocs                     */

#define TRAIT_SELCTD    (sim_traits && sim_traits->trait_typ == SS_TRLIST)
#define TLOC_SELCTD     (sim_traits && sim_traits->trait_typ == SS_TLLIST)
#define INHER_SELCTD    (sim_traits && sim_traits->trait_typ == SS_INHER)
#define UNLTLOC_SELCTD  (sim_unlnkd_tlocs)

/* distances and positions are Haldane           */
#define RM     0       /* recomb fraction        */
#define DM     1       /* distance               */
#define PM     2       /* position               */
#define JPM    3       /* joint positions        */
#define JUPM   4       /* pos joint w/ unlnkd pos*/
#define IM     5       /* interval proportions   */
#define AM     6       /* all interval props     */
#define XRM    7       /* external recomb fracts */
#define XDM    8       /* external distance      */
#define BRM    9       /* begin recomb fracts    */
#define ERM   10       /* ending recomb fracts   */
#define BDM   11       /* begin distances        */
#define EDM   12       /* ending distances       */

/* distances and positions are Kosambi           */
#define KDM   13       /* distance               */
#define KPM   14       /* position               */
#define JKPM  15       /* joint positions        */
#define JUKPM 16       /* pos joint w/ unlnkd pos*/
#define KXDM  17       /* external distance      */
#define KBDM  18       /* begin distances        */
#define KEDM  19       /* ending distances       */

#define INTDAT   1   /* trait data is integer   */
#define INTDAT2  2   /* trait data is int pairs */
#define REALDAT  3   /* trait data is real      */
#define INHDAT2  4   /* inher data is int pairs */

#define AV  0       /* additive variance      */
#define RV  1       /* residual variance      */

#define AFFIND 1    /* names of affecteds     */
#define DISPIND 2   /* names for display      */


/* **  for PedComp/kinship computing requests                              ** */

struct kinship_node {                    /*Kinship_node:                      */
    int component;                       /*for this comp,                     */
    int no_nams;                         /*the no. of probands                */
    char * p_nams;                       /*and a ptr to the list of names     */
    struct kinship_node * p_next;
                    };
typedef struct kinship_node KINS;
typedef KINS * KINS_PTR;
typedef KINS * INBR_PTR;
typedef KINS * INBR2_PTR;
static KINS_PTR  first_kins_node = NULL;
static INBR_PTR  first_inbreed_node = NULL;
static INBR2_PTR first_2loc_node = NULL;


/* ****** simulation or selections of markers and of traits at tloci ******** */

struct sim_chrom_node {                 /*Sim_chrom_node:                     */
    int chrom_no;                       /*chromosome no.                      */
    int mark_typ;                       /*marker sim/sel statement type       */
    int no_mark_sel;                    /*no. markers to select/simulate      */
                                        /*(for all, added by test_mark_map()) */
                                        /*(-1, for all markers)               */
    int no_mark_map;                    /*no. markers mapped (from chrom map) */
                                        /*(added by test_mark_map())          */
                                        /*(0, for only 1 marker)              */
    int * p_mark;                       /*ptr to list of marker no.'s         */
                                        /*(no. is no_mark_sel)                */
    int no_tloc_sel;                    /*no. tloc sim/sel for this chrom     */
                                        /*(added by (updt_sim_lnkd_tloc)      */
                                        /*or by (updt_sim_vtloc))             */
    int * p_tloc;                       /*ptr to list of tlocs in num. order  */
                                        /*(added by fncs named above)         */
    struct sim_chrom_node * p_next;
                      };
typedef struct sim_chrom_node SIM;
typedef SIM * SIM_PTR;
static SIM_PTR first_sim_chrom_node = NULL;

struct sim_trait_node {                 /*Sim_trait node:                     */
    int trait_typ;                      /*sim/sel statement type              */
                                        /* (traits, tlocs, or inheritance     */
    int no_traits;                      /*no. traits to select/simulate       */
    int * p_trait;                      /*ptr to array of trait, tloc or      */
                      };                /*iheritance no's                     */
typedef struct sim_trait_node SIMTRAIT;
typedef SIMTRAIT * SIMTRAIT_PTR;
static SIMTRAIT_PTR sim_traits = NULL;

struct trait_tloc_node {                /*Trait_tloc_node:                    */
    int trait_no;                       /*trait no.                           */
    int list_typ;                       /*tr/tloc or tr/mult-tloc             */
    int selectd;                        /*trait selected/simulated?           */
                                        /*added by (find_trait_tloc())        */
    int no_loci;                        /*no. of loci for this trait          */
    int * p_tloc;                       /*ptr to array of tloc no's for trait */
    struct trait_tloc_node * p_next;
                       };
typedef struct trait_tloc_node TRAITTLOC;
typedef TRAITTLOC * TRAITTLOC_PTR;
static TRAITTLOC_PTR first_trait_tloc_node = NULL;

struct inher_tloc_node {                /*Inher_tloc_node:                    */
    int inher_no;                       /*inher no.                           */
    int list_typ;                       /*inh/tloc (only choice now)          */
    int selectd;                        /*inher selected/simulated?           */
                                        /*added by (find_inher_tloc())        */
    int no_loci;                        /*no. of loci for this inher (1)      */
    int * p_tloc;                       /*ptr to list of tloc no's for inher  */
    struct inher_tloc_node * p_next;
                       };
typedef struct inher_tloc_node INHERTLOC;
typedef INHERTLOC * INHERTLOC_PTR;
static INHERTLOC_PTR first_inher_tloc_node = NULL;

struct sim_unlnkd_tloc_node {           /*Sim_unlnkd_tloc:                    */
    int no_tloc_sel;                    /*no. tloc sim/sel (updt_sim_tloc())  */
    int * p_tloc;                       /*ptr to list of tloc no.'s           */
                            };
typedef struct sim_unlnkd_tloc_node SIM_UNLNKD;
typedef SIM_UNLNKD * SIM_UNLNKD_PTR;
static SIM_UNLNKD_PTR sim_unlnkd_tlocs = NULL;


/* ****************** mapping of markers and trait loci  ******************** */

struct map_node {                       /*Map_node:                           */
    int chrom_no;                       /*for this chromosome no.             */
    int no_loci;                        /*no. markers or tlocs                */
    struct dist_list * p_dist;          /*ptr to dist lists for loci          */
    struct map_node * p_next;           /*3 lists:  female, male and sex-avgd */
                };
typedef struct map_node MAP;
typedef MAP * MAP_PTR;
static MAP_PTR first_mark_map_node[3] = {NULL, NULL, NULL};
static MAP_PTR first_tloc_map_node[3] = {NULL, NULL, NULL};

struct dist_list {                      /*Dist_node:                          */
    int no;                             /*marker or tloc no.                  */
    int selectd;                        /*marker or tloc selectd? (NEW)       */
                                        /*(added by find_tloc_map())          */
    int map_mark;                       /*preceding marker on this chrom      */
    double pos;                         /*dist of mark or tloc from 1st marker*/
                                        /*(NEW)**/
    double dist;                        /*dist of marker or tloc from the loc */
    double prob;                        /*and the recomb fract                */
                 };
typedef struct dist_list DIST;
typedef DIST * DIST_PTR;

struct unlnkd_map_node {                /*Unlnkd_map_node:                    */
    int no_loci;                        /*no. tlocs sel (updt_sim_tloc))      */
    struct udist_list * p_udist;        /*ptr to list of tloc no.'s           */
                       };
typedef struct unlnkd_map_node UMAP;
typedef UMAP * UMAP_PTR;
static UMAP_PTR unlnkd_tloc_map_node = NULL;

struct udist_list {                     /*Udist_node:                         */
    int no;                             /*(unlinked) tloc no.                 */
    int selectd;                        /*tloc selectd? (NEW)                 */
                  };                    /*(added by find_tloc_map())          */
typedef struct udist_list UDIST;
typedef UDIST * UDIST_PTR;

struct vmap_node {                      /*Vmap_node:                          */
    int chrom_no;                       /*for this chromosome **AND**         */
    int no;                             /*tloc no. (w varying test pos)       */
    int selectd;                        /*tloc selectd? (NEW)                 */
    struct xt_map * p_pos;              /*ptr to positions relative to 1st mrk*/
                                        /* *NEW * */
    struct xt_map * p_beg;              /*ptr to starting positions           */
    struct xt_map * p_end;              /*ptr to ending positions             */
    struct nt_map * p_intv;             /*ptr to list of positns beteewn marks*/
    struct vmap_node * p_next;          /*3 sets:  female, male and sex-avgd  */
                 };
typedef struct vmap_node VMAP;
typedef VMAP * VMAP_PTR;
static VMAP_PTR first_tloc_vmap_node[3] = {NULL, NULL, NULL};

struct xt_map {                         /*Xt_map:                             */
    int no_pos;                         /*no. positions relative to 1st mark, */
                                        /*or no. pos before 1st marker,       */
                                        /*or no. pos after last marker        */
    double * p_pos;                     /*ptr to positions (incr order)       */
                                        /* NEW */
    double * p_dist;                    /*ptr to distances (incr order)       */
    double * p_rf;                      /*and recomb fractions                */
              };                        /*(with first or last marker)         */
typedef struct xt_map XMAP;
typedef XMAP * XMAP_PTR;

struct nt_map {                         /*Nt_map:                             */
    int intv_no;                        /*interval no. (0 for all),           */
    int no_prop;                        /*no. proportions for this interval,  */
    double * p_prop;                    /*ptr to list of proportions of the   */
                                        /*distance between markers            */
    struct nt_map * p_next;             /*ptr to node for next interval       */
              };
typedef struct nt_map IMAP;
typedef IMAP * IMAP_PTR;

struct {                                /*vmap_options:                       */
    int default_xt;                     /*use default begin or ending pos     */
    int default_nt;                     /*use default interval proportions    */
       } vmap_opt = {TRUE, TRUE};

struct jmap_node {                      /*J_map node:                         */
    int chrom_no;                       /*for this chromosome AND             */
    int no[2];                          /*pair of tlocs                       */
    struct jnt_map * p_jnt;             /*ptr to joint pos for tloc 1, tloc 2 */
    struct ujnt_map * p_ujnt[2];        /*ptr to pos for tloc 1, tloc 2 unlnkd*/
                                        /*and to pos for tloc 2, tloc 1 unlnkd*/
    struct jmap_node * p_next;          /*3 sets:  female, male and sex-avgd  */
                 };
typedef struct jmap_node JMAP;
typedef JMAP * JMAP_PTR;
static JMAP_PTR first_tloc_jmap_node[3] = {NULL, NULL, NULL};

struct jnt_map {                        /*Jnt_map:                            */
    int no_jnt_pos;                     /*no. joint pos for tloc 1, tloc 2    */
    double ** p_jntpos;                 /*ptr to n pos pairs for tlocs 1 and 2*/
               };                       /*(sorted by tloc 1 position)         */
typedef struct jnt_map JNTMAP;
typedef JNTMAP * JNTMAP_PTR;

struct ujnt_map {                       /*UJnt_map:                           */
    int no_ujnt_pos;                    /*no. unlnkd positions                */
    double * p_ujntpos;                 /*ptr to pos for tloc, other unlinked */
                };
typedef struct ujnt_map UJNTMAP;
typedef UJNTMAP * UJNTMAP_PTR;

struct locus_node {                     /*Locus_node:                         */
    int chrom_no;                       /*chromosome no.                      */
    int no;                             /*the marker or tloc no.              */
    int no_alleles;                     /*the no. alleles                     */
    struct freq_node * p_freq_node;     /*ptr to the allele freq node         */
    struct locus_node * p_next;
                  };
typedef struct locus_node LOCUS;
typedef LOCUS * LOCUS_PTR;
static LOCUS_PTR first_mark_locus_node = NULL;
static LOCUS_PTR first_tloc_locus_node = NULL;

/* ******************* marker and tloc allele frequencies ******************* */

static int alle_frq_norm = N;           /*Flag - normalize allele freqs       */
static int alle_frq_deflts = N;         /*Flag - use default allele freqs     */

struct freq_node {                      /*Freq_node:                          */
    int no_loci;                        /*the no. loci with these allele freqs*/
    double sum;                         /*sum of allele frequencies           */
    double * p_freq;                    /*and ptr to list of freqs            */
                 };
typedef struct freq_node FREQ;
typedef FREQ * FREQ_PTR;

/* ******************* genotype means for traits at tlocs ******************* */

struct geno_means_node {                /*Geno_means_node:                    */
    int trait_no;                       /*trait no.                           */
    int tloc_no;                        /*tloc no.                            */
    double mean[3];                     /*genotype means for trait at tloc    */
    struct geno_means_node * p_next;
                       };
typedef struct geno_means_node GENO_MEANS;
typedef GENO_MEANS * GENO_MEANS_PTR;
static GENO_MEANS_PTR first_geno_means_node = NULL;

struct vari_node {                      /*Vari_node:                          */
    int no;                             /*trait no.                           */
    double vari;                        /*additive or residual variance       */
    struct vari_node * p_next;
                 };
typedef struct vari_node VARI;
typedef VARI * VARI_PTR;
static VARI_PTR first_addi_node = NULL;
static VARI_PTR first_resi_node = NULL;

struct traitped_node {                  /*Traitped_node:                      */
    int no;                             /*trait no.                           */
    int trait_typ;                      /*trait type:  trait, inheritance     */
    int dat_typ;                        /*data type: INTDAT, INTDAT2, REALDAT */
                                        /*           INHDAT2                  */
    int pos[2];                         /*1 or 2 record pos for trait data,   */
                                        /*relative to 1st int or real in rec  */
    struct traitped_node * p_next;
                     };
typedef struct traitped_node TRTPED;
typedef TRTPED * TRTPED_PTR;
static TRTPED_PTR first_trtped_node = NULL;

/***  The following data structures are used by proc_pars() and proc_auto() ***/
/***  for Autozyg/ specifications.                                          ***/
/***  A few of these are also used for ibddrop.                             ***/

struct markdat_node {                   /*Markdat_node:                       */
    int chrom_no;                       /*for ea chrom                        */
    int no_obs;                         /*no. observed individuals            */
    int no_marks;                       /*no. of markers (1 or no. mapped)    */
    char * p_obs;                       /*ptr to the list of ped member names */
    int * p_alls;                       /*ptr to the observed alleles         */
    struct markdat_node * p_next;
                    };
typedef struct markdat_node MARKDAT;
typedef MARKDAT * MARKDAT_PTR;
static MARKDAT_PTR first_markdat_node = NULL;

struct marknam_node {                   /*Marknam_node:                       */
    int chrom_no;                       /*for ea chrom                        */
    int no_marks;                       /*no. of markers                      */
    char * p_nams;                      /*ptr to the list of marker names     */
    struct marknam_node * p_next;
                    };
typedef struct marknam_node MARKNAM;
typedef MARKNAM * MARKNAM_PTR;
static MARKNAM_PTR first_marknam_node = NULL;

struct probgam_node {                   /*Probgam_node:                       */
    int component;                      /*for this component                  */
    int scoreset;                       /*and scoreset,                       */
    int no_pgams;                       /*the no. of proband gametes          */
    struct probgam_list *  p_pglist;    /*and a ptr to the list               */
    struct probgam_node * p_next;
                    };
typedef struct probgam_node PROBGAM;
typedef PROBGAM * PROBGAM_PTR;
static PROBGAM_PTR first_probgam_node = NULL;

struct probgam_list {                    /*Probgam_list:                      */
    char nam[MAXNAMLEN+1];               /*the list of individuals (names)    */
    int gamete;                          /*and gametes                        */
                    };
typedef struct probgam_list PROBANDS;
typedef PROBANDS * PROBLIST_PTR;

struct indnam_node {                    /*Indnam_node:                        */
    int component;                      /*for this component                  */
    int no_nams;                        /*the no. of selected individuals     */
    char * p_namlist;                   /*and a ptr to the list of names      */
    struct indnam_node * p_next;
                   };
typedef struct indnam_node INDNAM;
typedef INDNAM * INDNAM_PTR;
static INDNAM_PTR first_affnam_node = NULL;
static INDNAM_PTR first_dispnam_node = NULL;

struct locwind_node {                   /*Locwind_node:                       */
    int chrom_no;                       /*for this chromosome,                */
    int window;                         /*the no. of loci in ea window        */
    struct locwind_node * p_next;
                    };
typedef struct locwind_node LOCWND;
typedef LOCWND * LOCWND_PTR;
static LOCWND_PTR first_locwind_node = NULL;

struct windpat_node {                   /*Windpat_node:                       */
    int component;                      /*for this component                  */
    int scoreset;                       /*and scoreset,                       */
    int no_pats;                        /*the no. of pattern labels           */
    int *p_pats;                        /*and a ptr to the set                */
    struct windpat_node * p_next;
                    };
typedef struct windpat_node WNDPAT;
typedef WNDPAT * WNDPAT_PTR;
static WNDPAT_PTR first_windpat_node = NULL;

/***     The following variables and tables, used by proc_pars(),           ***/
/***     proc_poly() and check_pars(), contain PolyEM/ modelling            ***/
/***     specifications.                                                    ***/

struct trait_eff_node {                 /*Trait_effects_node:                 */
    int trait_num;                      /*Trait number for this node.         */
    int set_trait_eff_inp;              /*Set to 1 if found and to 0 if not.  */
    int start_tr_mean_inp;              /*Set to 1 if the "start trait K1     */
    /*                                    mean X1" statement has been read    */
    /*                                    and to 0 if not.                    */
    double start_tr_mean;               /*Starting value of mean for this     */
    /*                                    trait.                              */
    int num_eff;                        /*Number of effects for this trait.   */
    int * eff_nums;                     /*Pointer to an array of fixed effect */
    /*                                    numbers set for this trait.         */
    struct fixed_eff_node * p_eff_list; /*Pointer to a list of fixed_effect_  */
    /*                                    nodes given starting values by the  */
    /*                                    user for this trait.                */
    struct trait_eff_node * p_next;
                      };
typedef struct trait_eff_node TRAITEFF;
typedef TRAITEFF * TRAITEFF_PTR;
static TRAITEFF_PTR first_trait_eff_node = NULL;

struct fixed_eff_node {                 /*Fixed_effect_node:                  */
    int eff_num;                        /*Fixed effect number for this node.  */
    int nstartff;                       /*Number of starting values for this  */
    /*                                    effect (not including mean) found   */
    /*                                    from the "start trait K1 effect M1  */
    /*                                    X1 X2 ..." statement.               */
    double * startff;                   /*Pointer to an array of starting     */
    /*                                    values input for each level of this */
    /*                                    fixed effect.                       */
    struct fixed_eff_node * p_next;
                      };
typedef struct fixed_eff_node FIXEDEFF;
typedef FIXEDEFF * FIXEDEFF_PTR;

static int    **seltrff;                /*Selection versus nonselection of    */
                                        /*quantitative traits and their       */
                                        /*corresponding (discrete) fixed      */
                                        /*effects.  This information is the   */
                                        /*same as FfDex[][], but not          */
                                        /*condensed for traits selected.      */

static int    *selff;                   /*Selection versus nonselection of    */
                                        /*fixed effects (with mean included). */

static int    *cum_selff;               /*This array is the cumulative sum of */
                                        /*the entries in selff.  There are a  */
                                        /*number of places in proc_poly and   */
                                        /*check_pars where these cumulative   */
                                        /*sums are required to subscript in   */
                                        /*various other arrays, like FfDex.   */

static int    add_cor_flag,             /*Starting values are correlations,   */
              res_cor_flag;             /*not covariances.                    */

static int    naddvar, naddcov,         /*Number of starting values input for */
              nresvar, nrescov;         /*variances and covariances.          */

static int    nfitadd, nfitres;         /*Number of fit flags for covariances */

static int    eigen_set, eigen_compute; /*Eigenvalue options.                 */

static int    n_eigen;                  /*Number of eigenvalues given.        */

static int    n_pseudo;                 /*Number of pseudo-priors given.      */

/** @} */

#endif  /*PARSEPARS_H*/
