/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#ifndef TWOQTL_H
#define TWOQTL_H

#include <assert.h>
#include "markers.h"
#include "rans.h"
#include "quant.h"
#include "peelie.h"

/** batch mean at each position under the alternative and the null.
    instead of storing likelihood,
    store foo and max for stable computation. **/
struct Eval {
      double **foo_alt;  /* mean (exp (log_likelihood - max))  */
      double **max_alt;  /* max (log_likelihood)              */

      double foo_null;
      double max_null;

      int n_loci_1;
      int n_loci_2;
};
typedef struct Eval mc_eval_t;

void update_S_trait (int comp_no, int nsw, int debug_level,
                     char ***S_trait_1, char ***S_trait_0);
void free_update_S_trait (void);
char ***alloc_S_trait (int n_pos, int *size_c);
void free_S_trait (char ***S);

void write_lod_score (mc_eval_t *mc_eval, int debug_level);
void write_batch_means (mc_eval_t *mc_eval, int batch);
void update_batch (double value, int start, double *foo_p, double *max_p);
mc_eval_t *alloc_mc_eval (int n_pos, int n_qtl);
void free_mc_eval (mc_eval_t *mc_eval);

void log_prob_alt (double *theta, int new,
                   char ***S_trait_1, char ***S_trait_0, int comp_no,
                   int nsw, int debug_level, int start, mc_eval_t *mc_eval_p);
void log_prob_null (double *theta, int new,
                    int comp_no, int nsw, int debug_level,
                    int start, mc_eval_t *mc_eval_p);

double evaluate (int **FGL_a, int **FGL_b, int comp_no);
void free_evaluate (void);


/* printing functions for intermediate results (debug_level > 0) */
void print_S (char **S_0, char **S_1, int n_locus, int comp_no);
void print_S_marker (int comp_no);
void print_mc_eval (mc_eval_t out);

/* functions for getting variables for componentwise computation */
int *get_ibase (void);
int *get_obs_ibase (void);
int *get_size_c (void);
int *get_obs_size_c (void);
int **get_obs_index (void);
int *get_mbase (int nsw);
int *get_n_meiosis (int nsw);
int *get_nFGL_c (void);
void free_component (void);

int nghd_index (int i, int comp_no);
int comp_index (int i, int comp_no);


/* setting and getting parameters of lm_twoqtl */
void set_twoqtl_parm_int (char *key, int value);
void set_twoqtl_parm_dbl (char *key, double value);
void set_geno_means (char *key, double *value);
int    get_twoqtl_parm_int (char *key);
double get_twoqtl_parm_dbl (char *key);
double *get_geno_means (char *key);
void free_twoqtl_parm (void);
void set_twoqtl_parm (void); /* TEMP (before getting parameter statement) */


/* functions that need to be moved to Peel */
void peel_poly (double *y, double *log_prob);
void sample_poly (double *y, double *poly);
double tq_gauss_quant (double *y, int comp_no,
                       double sigd, double lambda, int im);
void free_tq_gauss (void);
void test_tq_gauss_quant (double *, double *);


/* allocate and free arrays in lm_twoqtl
   (a little different from those in Stuff) */
char **alloc_char2d (int, int);
char ***alloc_char3d (int, int, int);
int ***alloc_int3d_two (int dim1, int dim2, int dim3);
double ***alloc_dbl3d_two (int dim1, int dim2, int dim3);
void   free_char2d (void *);
void   free_dbl3d (void *);

/* another rans using different multiplier */
double rans_alt(void);
int intran_alt(int, int);
void setsd_alt(uint32, uint32);
void getsd_alt(uint32 *, uint32 *);
int getranscount_alt ();

#ifndef __GNUC__
void die(const char *format, ...);
#else
void die(const char *format, ...) __attribute__ ((__noreturn__));
#endif /* __GNUC__ */

#endif  /* TWOQTL_H */
