/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
static double **prob_new = NULL;

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~           */
void update_position (double p_lnkd,
                      int window_size,
                      int nummrks,
                      int nped, double **prob, int ngenes, int n_compnts)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~            */
     /*  M-H update for changing the position of the trait locus.             */
     /*  Three changes of position are possible:                              */
     /*  1. a change of trait locus position within an marker interval        */
     /*  2. a change of trait locus position and also a change in             */
     /*     locus ordering (i.e trait locus is moved to a new marker intervl) */
     /*  3. moving the trait locus to the unlinked case.                      */
     /* Note:  window_size => total window width = 2 * window_size + 1        */
{
   int   c, ii;
   int   sx;                            /* sex */
   int   cndt_posit;
   int   cndt_intvl;
   int   cuml_posit_new;
   int   cuml_posit_old;
   double q_old2new;
   double q_new2old;
   double prb_Yt_cndtl;                 /* log P(Y_T | S_{T-1}, S_{T+1})  */
   double M_H_prob;

   /* Allocate prob_new on first entry                                        */

   if (!prob_new)
   {
      prob_new = malloc (sizeof (double) * (n_compnts + 1));
      alloc_chk (WHERE, prob_new);
      for (ii = 0; ii < n_compnts; ii++)
      {
         prob_new[ii] = malloc (sizeof (double) * (nummrks + 1));
         alloc_chk (WHERE, prob_new[ii]);
      }
   }
   for (c = 0; c < n_compnts; c++)
      for (ii = 0; ii <= nummrks; ii++)
         prob_new[c][ii] = prob[c][ii];

   if (rans () <= p_lnkd)

      /* +-------------------------------------------+                        */
      /* |   MOVE TO LINKED POSITION PROPOSED        |                        */
      /* +-------------------------------------------+                        */

      /* Generate Candidate Position                                          */

   {
      linked_proposal (window_size, previous_realiz.tposit,
                       previous_realiz.intvl, &cndt_posit,
                       &cndt_intvl, &cuml_posit_old, &cuml_posit_new,
                       &q_old2new, &q_new2old);

      /* Generate S_T given candidate position                                */

      for (sx = FEM; sx <= MAL; sx++)
         set_chromosomes (nummrks, Marek[sx], 0, &IntVal[cuml_posit_new],
                          &RecVal[sx][cuml_posit_new], sx);
      set_positions (nummrks);

      prb_Yt_cndtl = do_lsmpl (nummrks, 0, prob_new, ngenes);

      /* Calculate M-H ratio                                                  */
      /* To avoid under/over flow errors, logs taken                          */

      M_H_prob = (prb_Yt_cndtl + log (q_new2old)
                  + log (PseudoPrior[cuml_posit_new]))
         - (previous_realiz.prb_Yt_condtl + log (q_old2new)
            + log (PseudoPrior[cuml_posit_old]));

      M_H_prob = exp (M_H_prob);

   }
   else
      /* +-------------------------------------------+                        */
      /* |   MOVE TO UNLINKED POSITION PROPOSED      |                        */
      /* +-------------------------------------------+                        */

   {

      /*  Generate Candidate Position                                         */

      unlinked_proposal (&cndt_posit, &cndt_intvl, &cuml_posit_new);

      /*   Generate S_T given candidate position                              */

      for (sx = FEM; sx <= MAL; sx++)
         set_chromosomes (nummrks, Marek[sx], 0, &IntVal[cuml_posit_new],
                          &RecVal[sx][cuml_posit_new], sx);
      set_positions (nummrks);

      prb_Yt_cndtl = do_lsmpl (nummrks, 0, prob_new, ngenes);

      /*   Calculate M-H ratio for a move from linked to unlinked state       */

      /*  alpha ((S_T, \lamba, \alpha), (S_T*, \lamba*, \alpha*))
         = 1/(Num linked test pos) * P(Y_T | \lamba* = 0.5, \alpha* = 0) 
         * Prior(0.5,0)                                                       */
      /* -------------------------------------------------------------------  */
      /*  p_unlinked * P(Y_T | S_i, S_j, \lamba, \alpha) 
       * Prior(\lamba, \alpha)                                              */

      /* Determine the current cumulative position                            */
      cuml_posit_old = 0;
      for (ii = 0; ii < previous_realiz.intvl; ii++)
         cuml_posit_old += NTPosInt[ii];
      cuml_posit_old += previous_realiz.tposit;

      M_H_prob = (log (1.0 / NTPos) + prb_Yt_cndtl + log (PseudoPrior[0]))
         - (log (1.0 - p_lnkd) + previous_realiz.prb_Yt_condtl
            + log (PseudoPrior[cuml_posit_old]));

      M_H_prob = exp (M_H_prob);
   }

   /*   Accept move with prob M_H_prob                                        */

   if (M_H_prob > 1)
      M_H_prob = 1;
   if (rans () <= M_H_prob)
      /* accept candiate move                                                 */
   {
      set_previous_realiz (cndt_intvl, cndt_posit, RecVal[FEM][cuml_posit_new],
                           RecVal[MAL][cuml_posit_new],
                           prb_Yt_cndtl, xx0, xx1, nummrks, nped);

      /* Set prob                                                             */
      for (ii = 0; ii <= nummrks; ii++)
         for (c = 0; c < n_compnts; c++)
            prob[c][ii] = prob_new[c][ii];

      /* Adjust tposit frequency                                              */
      tposit_freq[cuml_posit_new]++;
   }

   else
      /* Else move is rejected and previous values retained                   */
   {

      /* NEED TO THINK ABOUT THIS /....... now that we have 0 IntVal 
         not just implying unlinked case                                      */

      if (previous_realiz.intvl == 0 && previous_realiz.rec_lft[FEM] == 0.5)
         cuml_posit_old = 0;            /* Unlinked                           */
      else
      {
         cuml_posit_old = 0;
         for (ii = 0; ii < previous_realiz.intvl; ii++)
            cuml_posit_old += NTPosInt[ii];

         cuml_posit_old += previous_realiz.tposit;
      }
      for (sx = FEM; sx <= MAL; sx++)
         set_chromosomes (nummrks, Marek[sx], 0, &IntVal[cuml_posit_old],
                          &RecVal[sx][cuml_posit_old], sx);
      set_positions (nummrks);

      /* reset xx0 and xx1 to previous.realiz.xx0 and previous.realiz.xx1     */
      assign_xx (nummrks, nped);

      /* based on reset xx0 and xx1  reset gg0 and gg1 to previous values     */
      for (ii = 0; ii <= nummrks; ii++)
         set_FGL (ii);

      /* Adjust tposit frequency                                              */
      tposit_freq[cuml_posit_old]++;
   }

   return;
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                      */
void linked_proposal (int WindWdth,
                      int posit_old,
                      int intvl_old,
                      int *posit_new,
                      int *intvl_new,
                      int *cuml_posit_old,
                      int *cuml_posit_new, double *q_old2new, double *q_new2old)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                      */
{
   int   ii, dummy;

   int   X_upper;                       /* upper limit of window    */
   int   X_lower;                       /* lower limit of window    */
   int   h_upper;                       /* upper width of window    */
   int   h_lower;                       /* lower width of window    */

   /* Determine what the old cumulative position is - needed by Morgan routines */
   *cuml_posit_old = 0;
   for (ii = 0; ii < intvl_old; ii++)
      *cuml_posit_old += NTPosInt[ii];
   *cuml_posit_old += posit_old;

   /*   1. Determine if window extends past upper or lower test positions     */
   /*   2. Sample a new canidate test position from the (adjusted) window     */
   /*   3. Calculate q(old -> new)                                            */

   /* Test if window extends beyond right most test position                  */
   if (*cuml_posit_old + WindWdth > NTPos) /* NTPos does not include
                                              unlinked pos (unlike NTPos1)   */
   {
      X_upper = NTPos;
      h_upper = NTPos - *cuml_posit_old;
   }
   else
   {
      X_upper = *cuml_posit_old + WindWdth;
      h_upper = WindWdth;
   }

   /* Test if window extends beyond the left most test position               */
   if (*cuml_posit_old - WindWdth < 1)
   {
      X_lower = 1;
      h_lower = *cuml_posit_old - 1;
   }
   else
   {
      X_lower = *cuml_posit_old - WindWdth;
      h_lower = WindWdth;
   }

   /* Randomly accept position within interval                                */
   *cuml_posit_new = intran (X_lower, X_upper);

   *q_old2new = 1.0 / (h_lower + h_upper + 1.0);

   /* work out the new marker interval                                        */
   *intvl_new = IntVal[*cuml_posit_new];

   dummy = *cuml_posit_new;
   for (ii = 0; ii < *intvl_new; ii++)  /* derive new position within
                                           marker intvl                       */
      dummy -= NTPosInt[ii];
   *posit_new = dummy;

   /* 1. Given the new test position, determine if window extends             */
   /*    beyond test position boundaries                                      */
   /* 2. Calculate  q(new -> old)                                             */

   /* Test if window extends beyond right most test position                  */
   if ((*cuml_posit_new) + WindWdth > NTPos) /* NTPos does not include
                                                unlinked pos                 */
   {
      X_upper = NTPos;
      h_upper = NTPos - (*cuml_posit_new);
   }
   else
   {
      X_upper = (*cuml_posit_new) + WindWdth;
      h_upper = WindWdth;
   }

   /* Test if window extends beyond the left most test position               */
   if ((*cuml_posit_new) - WindWdth < 1)
   {
      X_lower = 1;
      h_lower = (*cuml_posit_new) - 1;
   }
   else
   {
      X_lower = (*cuml_posit_new) - WindWdth;
      h_lower = WindWdth;
   }

   *q_new2old = 1.0 / (h_lower + h_upper + 1.0);

   return;

}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                     */
void unlinked_proposal (int *posit_new, int *intvl_new, int *cuml_posit_new)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                     */
{
   /* A deterministic move to the unlinked case                               */
   *posit_new = 0;
   *intvl_new = 0;
   *cuml_posit_new = 0;
   return;
}
