/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/* --------------------------------------------------------------------------- *
 *       pr_npatsets(), pr_pgamset(),   pr_cmpnthead(), pr_probhead(),         *
 *       pr_prbihead(), pr_prbwhead(),  pr_wndwpat(),                          *
 *       pr_lochead(),  pr_wndwhead(),  pr_locprob(),   pr_wndwprob()          *
 *    pr_histo()  added  EAT 01/2005
 * --------------------------------------------------------------------------- *
 */
#include "markers.h"
#include "traits.h"

/* no. chars for label column,    no. chars for probability data column,
 * no. chars for pattern column,  no. data columns per line
 */
static int wL, wD, wP, nDC;

/**
 * prints number of pattern sets for specified number of gametes;
 * saves no. characters for labels.
 */
void pr_npatsets (int *npt)
{
   int   j, k, c, s;

   if (npt[MxGam] > 0)
      wL = log10 ((double) npt[MxGam]) + 1.;

   printf (" Number of pattern sets to be scored:\n");
   for (j = 1; j <= MxGam; ++j)
   {
      k = 0;
      for (c = 0; c < n_compnts && c < NScCmp; ++c)
         for (s = 0; s < NScSet[c]; ++s)
            if (j == NPrGam[c][s])
               k = 1;
      if (k)
         printf ("    on %2d gametes:  %*d \n", j, wL, npt[j]);
   }
   printf ("\n\n");
   fflush (stdout);

   if (wL < 5)
      wL = 5;

   return;
}

/**
 * prints set of proband gametes for specified component and scoreset.
 */
void pr_pgamset (int c, int s)
{

   int   i, nc = 0;

   if (c < NScCmp && NScSet[c])
   {
      printf ("       Proband gamete set %d:", ScSetNo[c][s]);
      for (i = 0; i < NPrGam[c][s]; ++i)
      {
         nc = nc + strlen (ProbNam[c][s][i]) + 4;
         if (nc > 50)
         {
            printf ("\n                            ");
            nc = 0;
         }
         printf ("  %s", ProbNam[c][s][i]);
         printf (" %d", ProbGam[c][s][i]);
      }
      printf ("\n\n");
      fflush (stdout);
   }
   return;
}

/**
 * prints set of patterns for windows for specified component and scoreset.
 */
void pr_wndwpat (int c, int s)
{
   int   i, nd, np;

   nd = intWid (1, MxGam);
   np = 59 / (nd + 2);

   if (c < NScCmp && NScSet[c])
   {
      for (i = 0; i < NPtLab[c][s]; ++i) /* prints pattern labels            */
      {
         if (!i)
            printf ("       Pattern set:  ");
         else if (!(i % np))
            printf ("\n                    ");
         printf (" %*d ", nd, PatLab[c][s][i]);
      }
      if (NPtLab[c][s])
      {
         printf ("\n\n");
         fflush (stdout);
      }
   }
   return;
}

/**
 * prints heading for component.
 */
void pr_cmpnthead (int c)
{
   printf ("\n Summary for component %d:\n\n", c + 1);
   fflush (stdout);

   return;
}

/**
 * prints heading for ibd patterns for each locus.
 */
void pr_probhead (void)
{
   printf ("    Probabilities of IBD patterns\n\n");
   fflush (stdout);

   return;
}

/**
 * prints heading for ibd patterns for each locus.
 */
void pr_prbihead (void)
{
   printf ("    Probabilities of IBD patterns for windows of %d loci\n\n",
           NLocWndw);
   fflush (stdout);

   return;
}

/**
 * prints heading for a set of ibd patterns for windows of loci.
 */
void pr_prbwhead (int c, int s)
{
   if (c < NScCmp && NScSet[c])
   {
      printf ("    Probabilities of IBD for pattern set for windows"
              " of %d loci\n\n", NLocWndw);
      printf ("       Proband gamete set %d\n\n", ScSetNo[c][s]);
      fflush (stdout);
   }

   return;
}

/**
 * prints column headings for ibd probabilities for loci;
 * saves no. chars for pattern column, no. chars for probability column
 * and no. probability columns / line.
 */
void pr_lochead (int **chromo)
{
   int   j;

   wP = 2 * (MxGam > 4 ? MxGam : 4);

   get_colfmt (80 - 6 - wP - 1, LocNamLen, 6, 1, &nDC, &wD);
   if (wL < wD)
      wL = wD;

   for (j = 0; j < NLoci; ++j)
   {
      if (!(j % nDC))
      {
         if (j)
            printf ("\n");
         printf ("     ");
         if (!j)
            printf (" %*s", wP, "pattern");
         else
            printf (" %*s", wP, " ");
      }
      if (traits)
         printf (" %*s", wD, traits[chromo[0][j]].self);
      else
      {
         if (chromo[0][j] == 0)
            printf (" %*s", wD, TLocNam[0]);
         else
            printf (" %*s", wD, MarkNam[chromo[0][j] - 1]);
      }
   }
   if (NLoci)
   {
      printf (" %*s\n\n", wL, "label");
      fflush (stdout);
   }
   return;
}

/**
 * prints probabilities for each locus.
 */
void pr_locprob (int ng, int *gaga, double *p, int lab)
{
   int   i, j;

   for (j = 0; j < NLoci; ++j)
   {
      if (!(j % nDC))
      {
         if (j)
            printf ("\n");
         printf ("     %*s", wP - 2 * ng, " ");
         if (!j)
            for (i = 0; i < ng; ++i)
               printf (" %d", gaga[i]);
         else
            for (i = 0; i < ng; ++i)
               printf ("  ");
      }
      printf (" %*s", wD, dub2nlz (p[j], wD, 4));
   }
   if (NLoci)
   {
      if (!(j % nDC))
         printf ("\n       %*s", wP, " ");
      printf (" %*d\n", wL, lab);
      fflush (stdout);
   }
   return;
}

/**
 * prints column headings for ibd probabilities for windows.
 */
void pr_wndwhead (int nw)
{
   int   i, j;

   for (j = 0; j < nw; ++j)
   {
      if (!(j % 8))
      {
         if (j)
            printf ("\n");
         printf ("      ");
         for (i = 2; i < NLocWndw; ++i)
            printf ("  ");
         if (!j)
            printf (" IBD ");
         else
            printf ("     ");
      }
      printf (" wndw%2d", j + 1);
   }
   printf ("\n\n");
   fflush (stdout);

   return;
}

/**
 * prints probabilities for each window.
 */
void pr_wndwprob (int c, int s, int nw, int p, int *g, double ****ns)
{
   int   i, j;

   for (j = 0; j < nw; ++j)
   {
      if (!(j % 8))
      {
         if (j)
            printf ("\n");
         printf ("       ");
         for (i = 0; i < NLocWndw; ++i)
            if (!j)
               printf ("%d ", g[i]);
            else
               printf ("  ");
      }
      printf ("%s", dub2nlz (ns[c][s][j][p] / NIt, 7, 4));
   }
   printf ("\n");
   fflush (stdout);

   return;
}

void pr_pvals (int nl, double **pval, int nt, int nq)
/* nt is total number of quantiles (rows) in array **pval
 * nq is total number to be printed, and interval nt/nq
 * they are equally spaced, so pval increment is float(nq/nt)
 */
{
   int   i, ii, j, ncol, wcol;

   if (nl == NLoci - 1)
   {
      get_colfmt (70, LocNamLen, 6, 1, &ncol, &wcol);
      for (i = 0, j = 1; j <= nl; ++i, ++j)
      {
         if ((!(i % ncol)) && (i))
            printf ("\n          ");
         printf (" %*s", wcol, traits[chromo[0][j]].self);
      }
   }
   else
   {
      ncol = 10;
      wcol = 6;
      for (i = 0, j = 1; j <= nl; ++i, ++j)
      {
         if ((!(i % ncol)) && (i))
            printf ("\n          ");
         printf (" %5d", j);
      }
   }

   fflush (stdout);
   printf ("\n");
   for (ii = 0; ii < nq; ++ii)
   {
      printf ("%4.2f %5.3f", 0.5 * ii * 2 / nq, pval[0][ii * nt / nq]);
      for (i = 0, j = 1; j <= nl; ++i, ++j)
      {
         if ((!(i % ncol)) && (i))
            printf ("\n          ");
         printf (" %*.3f", wcol, pval[j][ii * nt / nq]);
      }
      printf ("\n");
      fflush (stdout);
   }
   printf ("\n\n");
   return;
}

void pr_histo (int c, int ***hist)
{
   int   i, j, k, low, high, maximp, scol, ncol, wcol;
   printf ("    %d affected individuals\n\n", cmplist[c + 1].naff);

   low = cmplist[c + 1].naff;
   high = 0;
   maximp = 0;
   for (j = 0; j < NLoci; ++j)
      for (k = 0; k < cmplist[c + 1].naff; ++k)
      {
         if (hist[c][j][k] > 0 && k < low)
            low = k;
         if (hist[c][j][k] > 0 && k > high)
            high = k;
         if (hist[c][j][k] > maximp)
            maximp = hist[c][j][k];
      }

   scol = 7 + LocNamLen + 1;
   get_colfmt (80 - scol - 1, intWid (0, high), intWid (0, maximp),
               1, &ncol, &wcol);

   printf ("    Number of imputations with values:\n");

   for (i = 0, k = low; k <= high; ++i, ++k)
   {
      if (!(i % ncol))
         printf ("\n%*s ", scol, " ");
      printf (" %*d", wcol, k + 1);
   }
   printf ("\n");

   for (j = 0; j < NLoci; ++j)
   {
      printf ("\n       %*s: ", LocNamLen, traits[chromo[0][j]].self);
      for (i = 0, k = low; k <= high; ++i, ++k)
      {
         if (i && !(i % ncol))
            printf ("\n%*s ", scol, " ");
         printf (" %*d", wcol, hist[c][j][k]);
      }
   }
   printf ("\n\n");
   return;

}
