/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "markers.h"

/* routine to determine if individual is only child of a parent  */

int only_kid (Ind * parent, Ind * child)
{
   Marlist *mlpt;
   Indlist *ilpt;

   for (mlpt = parent->marriages; mlpt != NULL; mlpt = mlpt->next)
   {
      for (ilpt = mlpt->current->kids; ilpt != NULL; ilpt = ilpt->next)
         if (strncmp (ilpt->current->self, child->self, MAXNAMLEN))
            return (0);
   }
   return (1);
}

/* generate a list of the 2(pedsize-n_founder) switches (max), so can choose
   a random switch  -- switch is indicated by individual and mat/pat.
   Also, set up a linked list of the switches in a given component for
   later use in sampling and scoring routines -- this bit added, EAT 11/2000
   Also, move some switch variables to msample.h so do not have to be passed
   -- now moved likewise to Markers and markers.h : EAT June 2002    */

/* add component info to each meiosis (Switch)      EAT  10/2004     */
/* Replace these linked lists of meioses by component, by vector
   of meosis indices, in the Component structure    EAT  10/2004     */
/* get rid of *nns argument of make_switches -- info now in Component strct */
/* get rid of setting xx0 and xx1 to -1 for non-meioses -- do elsewhere */
/*   -- hence get rid of argument nl of make_switches    EAT  11/2004   */
/* This meiosis set-up is now moved to Nghds library     EAT 10/2004    */

int make_switches ()
{
   int   i, k, kk, ns;
   Ind  *dad, *mum;
   int  *nns;

   /* The maximum number of switches is: 2 * (PedSize - nFound) */

   meiosis = malloc ((2 * (PedSize - nFound) + 1) * sizeof (Switch));
   alloc_chk (WHERE, meiosis);

   ns = 0;

   for (k = 1; k <= nCompnt; ++k)
      cmplist[k].nmeioses = 0;

   for (i = 0; i < pedsize; ++i)
   {
      if (!isNullNode (nghd[i].pa))
      {
         dad = nghd[i].pa;
         mum = nghd[i].ma;

         if (!(isNullNode (nghd[mum->index].ma) && only_kid (mum, &nghd[i])))
         {
            ++ns;
            meiosis[ns].ind = nghd[i];
            meiosis[ns].sex = 0;
            meiosis[ns].cmpnt = nghd[i].component;
            ++cmplist[nghd[i].component].nmeioses;
         }

         if (!(isNullNode (nghd[dad->index].pa) && only_kid (dad, &nghd[i])))
         {
            ++ns;
            meiosis[ns].ind = nghd[i];
            meiosis[ns].sex = 1;
            meiosis[ns].cmpnt = nghd[i].component;
            ++cmplist[nghd[i].component].nmeioses;
         }
      }
   }
   /* now set space and initialize the meiosis info in individual structures */
   /*     added EAT 4/2005 */
   for (i = 1; i <= ns; ++i)
   {
      k = (meiosis[i].ind).index;
      if (meiosis[i].sex == 1)
      {
         nghd[k].patmeio = i;
         dad = nghd[k].pa;
         ++nghd[dad->index].nkidmeio;
      }
      if (meiosis[i].sex == 0)
      {
         nghd[k].matmeio = i;
         mum = nghd[k].ma;
         ++nghd[mum->index].nkidmeio;
      }
   }

   nghd[0].kidmeio = (int *) malloc (ns * sizeof (int));
   alloc_chk (WHERE, nghd[0].kidmeio);

   for (k = 1; k < pedsize; ++k)
      nghd[k].kidmeio = nghd[k - 1].kidmeio + nghd[k - 1].nkidmeio;

   for (k = 0; k < pedsize; ++k)
      if (nghd[k].nkidmeio == 0)
         nghd[k].kidmeio = NULL;

   nns = (int *) malloc (pedsize * sizeof (int));
   alloc_chk (WHERE, nns);

   for (k = 0; k < pedsize; ++k)
      nns[k] = 0;

   for (i = 1; i <= ns; ++i)
   {
      k = (meiosis[i].ind).index;
      if (meiosis[i].sex == 1)
      {
         dad = nghd[k].pa;
         kk = dad->index;
         nghd[kk].kidmeio[nns[kk]] = i;
         ++nns[kk];
      }
      if (meiosis[i].sex == 0)
      {
         mum = nghd[k].ma;
         kk = mum->index;
         nghd[kk].kidmeio[nns[kk]] = i;
         ++nns[kk];
      }
   }

   /* now set up the space for lists of meiosis switches by component */
   /*  note that cmplist and components are indexed from 1 (not 0)    */
   /*  note that meioses (Switches)  are indexed from 1 (not 0)    */

   cmplist[0].meioses = (int *) malloc (ns * sizeof (int));
   alloc_chk (WHERE, cmplist[0].meioses);

   cmplist[1].meioses = cmplist[0].meioses;
   for (k = 2; k <= nCompnt; ++k)
      cmplist[k].meioses = cmplist[k - 1].meioses + cmplist[k - 1].nmeioses;

   for (k = 0; k < pedsize; ++k)        /* actually will now use only
                                           indices 1,...,n_compnts */
      nns[k] = 0;

   for (i = 1; i <= ns; ++i)
   {
      k = meiosis[i].cmpnt;
      cmplist[k].meioses[nns[k]] = i;
      ++nns[k];
   }
   free (nns);

   return (ns);
}

void prnt_switches (int nsw)
{
   int   k;

   if (PrintLevel >= 3)
   {
      printf ("\n The number of meiosis switches to sample is %d\n\n", nsw);
      printf ("    compnt  switches   first    last\n");
      for (k = 1; k <= nCompnt; ++k)
         printf ("    %6d %9d %7d %7d\n",
                 cmplist[k].index, cmplist[k].nmeioses, cmplist[k].meioses[0],
                 cmplist[k].meioses[(cmplist[k].nmeioses) - 1]);
      printf ("\n\n");
      fflush (stdout);
   }

   if (DebugNghds)
      list_switches (nsw);

   return;
}

/* print the pedigree meiosis info  */
void list_nghds ()
{
   int   i;
   Ind  *mum, *dad;
   printf ("\n Listing of pedigree meioses by individual\n");
   printf (" name      name.pa      name.ma");
   printf (" Compnt pat.meio mat.meio\n");
   for (i = 0; i < pedsize; ++i)
   {
      if (!isNullNode (nghd[i].pa))
      {
         dad = nghd[i].pa;
         mum = nghd[i].ma;
         printf (" %14s %14s %14s",
                 nghd[i].self, nghd[dad->index].self, nghd[mum->index].self);
      }
      else
         printf (" %14s             0              0 ", nghd[i].self);
      printf (" %6d %6d %6d",
              nghd[i].component, nghd[i].patmeio, nghd[i].matmeio);
      printf ("\n");
   }
   printf ("\n");
   return;
}

/* list the details of all switches  */

void list_switches (int nsw)
{
   int   i;
   printf ("\n List of switches:\n");
   for (i = 1; i <= nsw; ++i)
      printf (" %d %d %d %s %d\n", i,
              (meiosis[i].ind).index, meiosis[i].cmpnt,
              (meiosis[i].ind).self, meiosis[i].sex);
   return;
}

/* free the data structures for the switches   */

void free_switches (void)
{

   if (!meiosis)
      return;

   free (meiosis);                      /* frees (Switch *) meiosis */
   meiosis = NULL;

}
