/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* these functions are called from proc_auto().
 * some are also called from proc_lods().
 */

/**
 * checks simulate/select marker/trait/tloc/inheritance statements for errors.
 */
int test_auto_sel (void)
{
   if (!first_sim_chrom_node)           /*select/simu ... markers .. required */
   {
      if (IBDDROP || MARKERDROP)
         err_typ = ERA_SIM;
      else
         err_typ = ERA_SEL;
      return FALSE;
   }

   else if (first_sim_chrom_node->p_next) /*checks for only one chromosome    */
   {
      if (IBDDROP || MARKERDROP)
         err_typ = ERA_OOM;
      else
         err_typ = ERA_OOS;
      return FALSE;
   }

   if (!sim_traits)                     /* checks for sel trait/tloc/inher    */
   {
      if (IBDDROP)                      /* ibddrop:  sim tloc                 */
         err_typ = ERA_MSL;
      else if (MARKERDROP)              /* markerdrop:  sel trait or inher    */
         err_typ = ERA_MTI;
      else if (LM_IBDTESTS || LM_PVAL || LM_AUTO) /* Autozyg/:  sel trait     */
         err_typ = ERA_MST;

      if (err_typ)
         return FALSE;
   }
   return TRUE;
}

/**
 * checks marker data statements for errors
 */
int test_markdat (SIM_PTR p_sel)
{
   int   i, j, k;
   MARKDAT_PTR p_mdt;
   LOCUS_PTR p_locus;
   int  *p_a;
   char *p_o;

   NObs = 0;
   if (!(p_mdt = first_markdat_node))   /* finds marker data for this chrom   */
   {
      err_typ = ERA_NMD;
      for_chrom (p_sel->chrom_no);
   }
   else
   {
      while (p_mdt->p_next && p_mdt->chrom_no < p_sel->chrom_no)
         p_mdt = p_mdt->p_next;
      if (p_mdt->chrom_no != p_sel->chrom_no)
      {
         err_typ = ERA_NMD;
         for_chrom (p_sel->chrom_no);
      }
      else
      {                                 /* checks no. of markers              */
         if ((p_sel->no_mark_map && p_mdt->no_marks != p_sel->no_mark_map)
             || (!p_sel->no_mark_map && p_mdt->no_marks != 1))
         {
            err_typ = ERA_BND;
            for_chrom (p_mdt->chrom_no);
         }
         else
            NObs = p_mdt->no_obs;
      }
   }
   if (err_typ)
      return FALSE;

   p_o = p_mdt->p_obs;                  /* tests marker alleles within range  */
   p_a = p_mdt->p_alls;                 /* ... for each observed individual   */
   for (i = 0; i < NObs; ++i)
   {
      p_locus = first_mark_locus_node;
      for (j = 0; j < p_sel->no_mark_map; ++j) /* ... and each marker    */
      {
         while (((p_locus->p_next && (p_locus->chrom_no < p_sel->chrom_no))
                 || (p_locus->chrom_no == p_sel->chrom_no && p_locus->no <= j)))
            p_locus = p_locus->p_next;

         for (k = 0; k < 2; ++k)
         {
            if (*p_a < 0 || *p_a > p_locus->no_alleles)
            {
               sprintf (errnumb, "%s", p_o);
               err_txt = errnumb;
               err_typ = ERA_WMA;
            }
            ++p_a;
         }
      }
      p_o += MAXNAMLEN + 1;
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks marker names statements for errors
 */
int test_marknam (SIM_PTR p_sel)
{
   MARKNAM_PTR p_mnm;

   if (!(p_mnm = first_marknam_node))   /* looks for mrk names for this chrom */
      return TRUE;
   else
   {
      while (p_mnm->p_next && p_mnm->chrom_no < p_sel->chrom_no)
         p_mnm = p_mnm->p_next;
      if (p_mnm->chrom_no != p_sel->chrom_no)
         return TRUE;
      else
         if ((p_sel->no_mark_map && p_mnm->no_marks != p_sel->no_mark_map)
             || (!p_sel->no_mark_map && p_mnm->no_marks != 1))
      {
         err_typ = ERA_BNM;
         for_chrom (p_mnm->chrom_no);
      }
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks program gamete statements for errors
 */
int test_probgam (SIM_PTR p_sel)
{
   PROBGAM_PTR p_pg;
   LOCWND_PTR p_node;

   if (!(p_pg = first_probgam_node))    /* expects proband gametes            */
      err_typ = ERA_MPG;

   else if ((p_node = first_locwind_node)) /* checks for locus window         */
   {                                    /* for this chromosome                */
      while (p_node->p_next && p_node->chrom_no < p_sel->chrom_no)
         p_node = p_node->p_next;

      if (p_node->chrom_no == p_sel->chrom_no)
      {
         NLocWndw = p_node->window;
         if (NLocWndw < 2 || NLocWndw > NLoci)
         {
            err_typ = ERA_LWO;
            sprintf (errnumb, "%d", NLoci);
            err_txt = errnumb;
         }
      }
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

int test_probgam_jnt (SIM_PTR p_sel)
{
   PROBGAM_PTR p_pg;

   if (!(p_pg = first_probgam_node))    /* expects proband gametes            */
      err_typ = ERA_MPG;

   if (IBDDROP && NLocWndw)             /* if ibddrop, and locus window,      */
      while (p_pg)                      /* tests for one pair of gametes      */
      {                                 /* in each set                        */
         if (p_pg->no_pgams != 2)
            err_typ = ERA_PIG;
         p_pg = p_pg->p_next;
      }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks the pattern label statements for errors.
 *
 * expects a locus window size if there are window patterns to score;
 * if not, a warning is issued                                    
 * Expects at least one proband gamete scoreset for each component for which 
 * there are pattern labels; if not, a warning is issued                    
 * Each pattern label must be in the correct range, given the number of    
 * probands                                                               
 */
int test_probgam_pat (SIM_PTR p_sel)
{
   int   i;

   PROBGAM_PTR p_pg;
   WNDPAT_PTR p_wp;

   if (!(p_wp = first_windpat_node))
      return TRUE;

   if (!NLocWndw)
   {
      err_typ = ERA_MLW;
      return FALSE;
   }

   p_pg = first_probgam_node;

   while (p_wp)
   {
      while ((p_pg->p_next && p_pg->component < p_wp->component)
             || (p_pg->p_next && p_pg->component == p_wp->component
                 && p_pg->scoreset < p_wp->scoreset))
         p_pg = p_pg->p_next;

      if (p_wp->component != p_pg->component
          || p_wp->scoreset != p_pg->scoreset)
      {
         err_typ = ERA_MSS;
         sprintf (errnumb, "component %d scoreset %d", p_wp->component,
                  p_wp->scoreset);
         err_txt = errnumb;
      }
      else
      {
         for (i = 1; i < p_wp->no_pats; ++i)
            if (*(p_wp->p_pats + i) > CDgg[p_pg->no_pgams])
            {
               err_typ = ERA_BPC;
               sprintf (errnumb, "%d", *(p_wp->p_pats + i));
               err_txt = errnumb;
            }
      }
      p_wp = p_wp->p_next;
   }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks scalar parameters and sets defaults
 */
int test_auto_pars (void)
{
   if (MARKERDROP)
      return TRUE;

   if (NIt == UNDEF)
   {
      if (!LM_MAP)
      {
         err_typ = ERA_NIT;
         return FALSE;
      }
      else
         NIt = MCEMIT;
   }
   if (IBDDROP)
   {
      if (err_typ)
         return FALSE;
      return TRUE;
   }

   if (SmplSetup == UNDEF)
      SmplSetup = SEQIMP;

   if (NSeqImp == UNDEF)
   {
      NSeqImp = RSEQIMP * NIt;
      if (NSeqImp < 2)
         NSeqImp = 2;
   }

   if (Scan == UNDEF)
      Scan = Y;
   if (ScorIt == UNDEF)
   {
      if (!LM_MAP && !LM_IBDTESTS)
         ScorIt = 1;
      else if (LM_IBDTESTS)
         ScorIt = NIt / 50 > 10 ? 50 : NIt / 10;
   }

   if (SeqImpIt == UNDEF)
      SeqImpIt = 0;

   if (GlobalMcmc == UNDEF)
      GlobalMcmc = 0;

   if (NBrnIt == UNDEF)
   {
      if (LM_MAP)
         NBrnIt = RBRNIT_MAP * NIt;
      else if (LM_PVAL)
         NBrnIt = RBRNIT_PVAL * NIt;
      else if (LM_IBDTESTS)
         NBrnIt = RBRNIT_IBDTESTS * NIt;
      else if (LM_AUTO)
         NBrnIt = RBRNIT_AUTO * NIt;
   }

   if (LM_MAP)
   {
      if (MapEstMod == UNDEF)           /* checks for with or wthout mistypng */
         err_typ = ERA_MMT;
      else if (MapEstMod != MISTYP)     /* no mistyping, checks algorithm     */
      {
         if (MaximAlgo == UNDEF)
            err_typ = ERA_MMA;
      }
      else if (MapEstMod == MISTYP)     /* mistyping, checks genotyping       */
      {                                 /* error rate                         */
         if (GenoErrRat == 0.)
            err_typ = ERA_NGE;
      }
      if (MapTyp == UNDEF)              /* if output map type no specified,   */
         MapTyp = BOTH;                 /* sets default                       */

      if (err_typ)
         return FALSE;
   }

   if (LM_PVAL)
   {
      if (SpairsOn == UNDEF && RobdomOn == UNDEF && MaxFGLOn == UNDEF)
      {
         MaxFGLOn = 1;                  /* assigns default values */
         SpairsOn = 0;
         RobdomOn = 0;
      }
   }

   if (LM_IBDTESTS)
   {
      if (IBDStats == UNDEF && LikRatStats == UNDEF)
      {
         err_typ = ERA_NCS;
         return FALSE;
      }
      else if (IBDStats == 1)           /* checks IBD statistics options      */
      {
         LikRatStats = 0;
         if (DeltaOn != UNDEF || Lambda_pOn != UNDEF) /* first measures     */
            err_typ = ERA_WIM;
         DeltaOn = 0;
         Lambda_pOn = 0;

         if (NormalEnable == UNDEF || PermutEnable == UNDEF)
         {
            NormalEnable = 1;           /* assigns tests defaults             */
            PermutEnable = 0;
         }

         if (SpairsOn == UNDEF || RobdomOn == UNDEF || SAffUnaffOn == UNDEF
             || LambdaOn == UNDEF || MaxFGLOn == UNDEF)
         {
            SpairsOn = 1;               /* assigns meas defaults              */
            RobdomOn = 0;
            SAffUnaffOn = 0;
            LambdaOn = 0;
            MaxFGLOn = 0;
         }

         if (NormalEnable && SAffUnaffOn)
            err_typ = ERA_NSU;

         else if (ChiSqEnable != UNDEF) /* checks tests                       */
            err_typ = ERA_WIT;
         ChiSqEnable = 0;
      }

      else if (LikRatStats == 1)        /* checks lhood-ratio stat options    */
      {
         IBDStats = 0;
         if (SpairsOn != UNDEF || RobdomOn != UNDEF /* first measures         */
             || SAffUnaffOn != UNDEF || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
            err_typ = ERA_WLM;
         SpairsOn = 0;
         RobdomOn = 0;
         SAffUnaffOn = 0;
         LambdaOn = 0;
         MaxFGLOn = 0;

         if (DeltaOn == UNDEF || Lambda_pOn == UNDEF)
         {
            DeltaOn = 1;                /* assigns meas defaults              */
            Lambda_pOn = 0;
         }
         if (NormalEnable != UNDEF || PermutEnable != UNDEF) /* checks tests  */
            err_typ = ERA_WLT;
         NormalEnable = 0;              /* assigns tests defaults             */
         PermutEnable = 0;

         if (ChiSqEnable == UNDEF)
            ChiSqEnable = 1;

         if (Lambda_pOn)                /* assigns grid spacing               */
         {
            IntervalSegL = .5 / (NumOfSegL - 1.);
            IntervalSegP = .8 / (NumOfSegP - 1.);
         }
      }
   }

   if (ChkIt == UNDEF)
      ChkIt = NIt / 5;

   if (FilIt == UNDEF)
      FilIt = NIt;

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks and sets TraitTyp (trait data type)
 * and PedDatTyp (integers, integer pairs, reals)
 */
int test_trait_typ (void)
{
   if (TraitTyp == UNDEF)
   {
      if (MARKERDROP)
      {
         if (TRAIT_SELCTD)
            TraitTyp = DISCRETE;
         else if (INHER_SELCTD)
            TraitTyp = INHERITANCE;
      }

      if (LM_IBDTESTS)
         TraitTyp = DISCRETE;

      if (LM_AUTO || LM_PVAL)
         TraitTyp = GENOTYPIC;

      if (LM_MARKERS || LM_BAYES || LM_LODS)
         TraitTyp = GENOTYPIC;

      if (LM_TWOQTL || LM_SCHNELL)
         TraitTyp = QUANTITATIVE;

      if (TRAIT_DATA)
         PedDatTyp = TraitTyp;
   }
   else if (MARKERDROP)
   {
      if (TRAIT_SELCTD && TraitTyp != DISCRETE)
      {
         TraitTyp = DISCRETE;
         PedDatTyp = TraitTyp;
         err_typ = ERA_IPD;
      }
      else if (INHER_SELCTD && TraitTyp == DISCRETE)
      {
         TraitTyp = INHERITANCE;
         PedDatTyp = TraitTyp;
         err_typ = ERA_IPT;
      }
   }

   if (!TRAIT_DATA)
      PedDatTyp = UNDEF;

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks pedigree record parameters for trait.
 * tests for trait record positions compatible with no. of integers or reals.
 * allows for gender added by Pedchk/ function in pedigree checking step.
 */
int test_ped_trtdat (void)
{
   int   nd = 1;
   int   tno = 0;
   TRTPED_PTR p_p;

   if (sim_traits)                      /*gets trait or inheritance no.       */
      tno = *(sim_traits->p_trait);

   if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB
       || PedDatTyp == DISCR_COVAR)
      ++nd;

   if ((p_p = find_trtped (tno)))       /* find record position for data      */
   {
      switch (PedDatTyp)
      {
          case QUANTITATIVE:
             if (p_p->dat_typ != REALDAT)
             {
                err_typ = ERA_BRR;
                sprintf (errnumb, "%d", tno);
                err_txt = errnumb;
             }
             else if (p_p->pos[0] > NReals) /* checks pos against no reals    */
             {
                err_typ = ERA_NEK;
                sprintf (errnumb, "%d", p_p->pos[0]);
                err_txt = errnumb;
             }
             break;

          case INHERITANCE:
          case DISCR_LIAB:
          case DISCR_COVAR:
             if ((PedDatTyp == DISCR_LIAB || PedDatTyp == DISCR_COVAR)
                 && p_p->dat_typ != INTDAT2)
             {
                err_typ = ERA_BI2;
                sprintf (errnumb, "%d", tno);
                err_txt = errnumb;
             }
             else if (PedDatTyp == INHERITANCE && p_p->dat_typ != INHDAT2)
             {
                err_typ = ERA_BH2;
                sprintf (errnumb, "%d", tno);
                err_txt = errnumb;
             }

             else if (p_p->pos[0] > NInts) /* checks pos against no. ints     */
             {
                err_typ = ERA_NEJ;
                sprintf (errnumb, "%d", p_p->pos[0]);
                err_txt = errnumb;
             }
             else if (p_p->pos[1] > NInts) /* checks 2nd pos and no. ints     */
             {
                err_typ = ERA_NEJ;
                sprintf (errnumb, "%d", p_p->pos[1]);
                err_txt = errnumb;
             }
             else if (p_p->pos[0] == p_p->pos[1])
             {
                err_typ = ERA_DRP;
                sprintf (errnumb, "%d", p_p->pos[0]);
                err_txt = errnumb;
             }
             break;

          case GENOTYPIC:
          case DISCRETE:
          default:
             if (p_p->dat_typ != INTDAT)
             {
                err_typ = ERA_BIR;
                sprintf (errnumb, "%d", p_p->no);
                err_txt = errnumb;
             }
             if (p_p->pos[0] > NInts)   /* checks position against no. ints   */
             {
                err_typ = ERA_NEJ;
                sprintf (errnumb, "%d", p_p->pos[0]);
                err_txt = errnumb;
             }
             break;
      }
      if (err_typ)
         return FALSE;
   }
   else                                 /* didn't find data pos for trait/inh */
   {
      switch (TraitTyp)
      {
          case QUANTITATIVE:
             err_typ = ERA_TER;
             break;
          case INHERITANCE:
             err_typ = ERA_TE2;
             break;
          default:
             err_typ = ERA_TEI;
             break;
      }
      sprintf (errnumb, "%d", tno);
      err_txt = errnumb;

      if (PedDatTyp != QUANTITATIVE)    /* note: SexIn may be UNDEF           */
      {
         if (tno + (SexIn != N ? 1 : 0) > NInts - nd + 1)
         {
            err_typ = ERA_NEI;
            sprintf (errnumb, "%d", tno);
            err_txt = errnumb;
         }
      }
      else if (tno > NReals)
      {
         err_typ = ERA_NER;
         sprintf (errnumb, "%d", tno);
         err_txt = errnumb;
      }
   }
   if (err_typ)
      return FALSE;

   return TRUE;
}

/**
 * sets mapping information for tloc relative to the "selected" marker.
 * determines:
 *    the selected reference marker for mapping
 *    the dist from the referenced marker
 */
void sel_tloc_locus (PRDIST_PTR p_pl)
{
   p_pl->selectd = 0;                   /* presently no selection for tlocs   */

   if (p_pl->map_mark == 0)             /* tloc precedes 1st marker:          */
   {
      if (p_pl->p_next->selectd)        /* sets selected marker and dist      */
         p_pl->sel_map_mark = p_pl->p_next->no;
      else
         p_pl->sel_map_mark = p_pl->p_next->sel_map_mark;
      p_pl->sel_dist = p_pl->dist + p_pl->p_next->sel_dist;
   }

   else if (p_pl->p_prev)               /* tloc follows 1st marker:           */
   {
      if (p_pl->p_prev->selectd)        /* preceding marker selected:         */
      {
         p_pl->sel_map_mark = p_pl->p_prev->no;
         p_pl->sel_dist = p_pl->dist;   /* sets sel dist to map dist          */
      }
      else if (p_pl->p_prev->sel_map_mark < p_pl->p_prev->no) /* some prev    */
      {                                 /* marker selected:                   */
         p_pl->sel_map_mark = p_pl->p_prev->sel_map_mark; /* adds sel dist    */
         p_pl->sel_dist = p_pl->dist + p_pl->p_prev->sel_dist; /* to map dist */
      }
      else if (p_pl->p_prev->sel_map_mark > p_pl->p_prev->no) /* no prev      */
      {                                 /* marker selected                    */
         p_pl->sel_map_mark = p_pl->p_prev->sel_map_mark; /* subtrcts map dst */
         p_pl->sel_dist = p_pl->p_prev->sel_dist - p_pl->dist; /* from sel    */
      }
   }
   return;
}

/**
 * removes prntmap locus nodes for non-selected markers.
 * adjusts forward and backward ptrs 
 */
void sel_prntmap_mark (PRNTMAP_PTR p_pm)
{
   PRDIST_PTR p, p_sav;

   while ((p = p_pm->p_locus) && (p->locus_typ == MARK_LOC) && (!p->selectd))
   {                                    /* looks for start of list            */
      p_pm->p_locus = p->p_next;        /* advances ptr to first locus        */
      free (p);                         /* removes this locus                 */
   }
   p->p_prev = NULL;

   p_sav = p;                           /* checks loci for deletion           */
   while ((p = p_sav->p_next))
   {
      if (p->locus_typ == MARK_LOC && !p->selectd)
      {
         p_sav->p_next = p->p_next;     /* advances last locus' forward ptr   */
         free (p);                      /* removes current locus              */
      }
      else
      {
         p->p_prev = p_sav;             /* updates current locus' backwrd ptr */
         p_sav = p;                     /* keeps current locus                */
      }
   }
   return;
}

/*
 * removes non-selected marker loci from list of loci
 */
void shrink_prntmap (PRNTMAP_PTR p_pmap)
{
   PRDIST_PTR p_cur, p_nxt, p_sav;

   p_cur = p_pmap->p_locus;
   while (p_cur && p_cur->locus_typ == MARK_LOC && !p_cur->selectd)
   {
      p_nxt = p_cur->p_next;
      free (p_cur);
      p_cur = p_nxt;
   }
   p_pmap->p_locus = p_cur;
   p_sav = p_cur;
   if (p_cur)                           /* 1st selected marker          */
   {
      p_cur->p_prev = NULL;
      p_cur->dist = p_cur->sel_dist;
   }

   while (p_cur)
   {
      if (p_cur->locus_typ == MARK_LOC && !p_cur->selectd)
      {
         p_sav->p_next = p_cur->p_next;
         p_nxt = p_cur->p_next;
         free (p_cur);
         p_cur = p_nxt;
         if (p_cur)
            p_cur->p_prev = p_sav;
      }
      else
      {
         p_sav = p_cur;                 /* next selected marker        */
         p_cur->dist = p_cur->sel_dist;
         p_cur = p_cur->p_next;
      }
   }
}

int test_penetrnc (void)
{
   if (NTraits && TraitTyp == DISCRETE && PedDatTyp != DISCR_LIAB
       && PedDatTyp != DISCR_COVAR)
      if (NPenVals == 0)
      {
         err_typ = ERA_MPV;
         return FALSE;
      }
   if (MARKERDROP && PedDatTyp == INHERITANCE)
   {
      NPenVals = 0;
      err_typ = ERA_PNI;
      return FALSE;
   }

   return TRUE;
}

void prnt_penetrnc (void)
{
   if (!NPenVals)
      return;

   if (PrintLevel >= 2)
   {
      fprintf (stdout, " Incomplete penetrances for the trait are:\n");
      fprintf (stdout, "        genotype  code  penetrance\n");
      fprintf (stdout, "        --------  ----  ----------\n");
      fprintf (stdout, "           (1 1)     0       %5.3f\n", PenVal[0]);
      fprintf (stdout, "    (1 2), (2 1)     1       %5.3f\n", PenVal[1]);
      fprintf (stdout, "           (2 2)     2       %5.3f\n\n", PenVal[2]);
   }

   return;
}

/**
 * allocates and sets MarkNam[][] which contains the names of the
 * selected markers, either provided names or default names.
 * returns true if marker names given in par statements.
 */
int mak_marknam (SIM_PTR p_sel)
{
   MARKNAM_PTR p_mnm;
   int   i, no, nc;
   char *p_n = NULL;

   if (!NMarks)
      return FALSE;

   MarkNam = malloc (NMarks * sizeof (char *));
   alloc_chk (WHERE, MarkNam);
   for (i = 0; i < NMarks; ++i)
   {
      MarkNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, MarkNam[i]);
   }

   if ((p_mnm = first_marknam_node))    /* checks for names given for chrom   */
   {
      while (p_mnm->p_next && p_mnm->chrom_no < p_sel->chrom_no)
         p_mnm = p_mnm->p_next;
      if (p_mnm->chrom_no == p_sel->chrom_no)
         p_n = p_mnm->p_nams;
   }

   for (i = 0; i < NMarks; ++i)
   {
      if (!p_sel->p_mark)
         no = i + 1;
      else
         no = *(p_sel->p_mark + i);
      if (p_n)
      {
         if ((nc = get_marknam (MarkNam[i], p_n, no)) > LocNamLen)
            LocNamLen = nc;
      }
      else if ((nc = fmt_marknam (MarkNam[i], no)) > LocNamLen)
         LocNamLen = nc;
   }
   if (p_n)
      return TRUE;
   else
      return FALSE;
}

/**
 * allocates and sets TraitNam[][], which contains the name
 * of the selected trait.
 */
void mak_traitnam (TLLIST_PTR p_tr)
{
   int   i, nc;

   if (!NTraits)
      return;

   TraitNam = malloc (NTraits * sizeof (char *));
   alloc_chk (WHERE, TraitNam);

   for (i = 0; i < NTraits; ++i)
   {
      TraitNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, TraitNam[i]);
   }
   for (i = 0; i < NTraits; ++i)
   {
      if ((nc = fmt_traitnam (TraitNam[i], p_tr->no)) > LocNamLen)
         LocNamLen = nc;
   }
   return;
}

/**
 * allocates and sets TLocNam[][], which contains the name
 * of the selected tloc.
 */
void mak_tlocnam (int tln)
{
   int   i, nc;

   if (!NTLocs)
      return;

   TLocNam = malloc (NTLocs * sizeof (char *));
   alloc_chk (WHERE, TLocNam);

   for (i = 0; i < NTLocs; ++i)
   {
      TLocNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, TLocNam[i]);
   }
   for (i = 0; i < NTLocs; ++i)
   {
      if ((nc = fmt_tlocnam (TLocNam[i], tln)) > LocNamLen)
         LocNamLen = nc;
   }
   return;
}

/**
 * allocates global arrays for marker positions
 *   Marek:   2 sexes, no. markers recomb fracts 
 *            plus beginning and ending unlinked pos
 *   MarkPos: 2 sexes, no. marker positions
 */
void alloc_marek (void)
{
   int   sx;

   Marek = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, Marek);
   MarkPos = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, MarkPos);

   for (sx = FEM; sx <= MAL; ++sx)
   {
      Marek[sx] = malloc ((NMarks + 1) * sizeof (double));
      alloc_chk (WHERE, Marek[sx]);
      MarkPos[sx] = malloc (NMarks * sizeof (double));
      alloc_chk (WHERE, MarkPos[sx]);
   }
   return;
}

/**
 * stores recombination fractions for markers.
 * makes Marek[][] : sex-specific recombination fractions
 *                   (1st and last one set to .5)
 */
void mak_marek (void)
{
   int   sx, entry, i;
   double pos;
   PRNTMAP_PTR p_pm;
   PRDIST_PTR p_loc;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      entry = FALSE;
      if ((p_pm = nxt_prntmap (&entry, sx)))
      {
         p_loc = p_pm->p_locus;

         pos = 0.;
         MarkPos[sx][0] = pos;
         Marek[sx][0] = .5;             /* ignores tloc, if linked           */
         for (i = 1; p_loc; p_loc = p_loc->p_next) /* also 1st marker         */
            if (p_loc->locus_typ == MARK_LOC && p_loc->dist)
            {
               pos += p_loc->dist;
               MarkPos[sx][i] = pos;
               Marek[sx][i++] = dist2rf (p_loc->dist);
            }
         Marek[sx][i] = .5;
      }
   }
   return;
}

/**
 * stores recombination fractions for tlocs.
 * makes Int0, Rec0[2] - tloc position and sex-specific recomb fraction
 */
void mak_tarek (void)
{
   int   sx, entry, i, j;
   PRNTMAP_PTR p_pm;
   PRDIST_PTR p_loc;

   if (!NLnkdTL)
   {
      Loc0 = 0;
      Rec0[0] = .5;
      Rec0[1] = .5;
      TraitPos[0] = .5;
      TraitPos[1] = .5;
   }
   else
      for (sx = FEM; sx <= MAL; ++sx)
      {
         entry = FALSE;
         if ((p_pm = nxt_prntmap (&entry, sx)))
         {
            p_loc = p_pm->p_locus;
            for (i = 0, j = 0; p_loc; p_loc = p_loc->p_next)
            {
               if (p_loc->locus_typ == TLOC_LOC)
               {
                  Loc0 = j;
                  Rec0[sx] = dist2rf (p_loc->dist);
                  TraitPos[sx] = p_loc->dist;
               }
               else if (p_loc->locus_typ == MARK_LOC && p_loc->selectd)
                  ++j;
            }
         }
      }
   return;
}

/**
 * allocates arrays and stores allele frequencies for all selected loci.
 * on first entry, mallocs NAll[NLoci] and Qq[NLoci] (NLoci is NMarks+1)
 *    NAll: 0 .. NLoci-1
 *    Qq: 1st subscript:  0 .. NLoci-1
 *    Qq: 2nd: 1 .. NAll[1st subscript]
 *    (Note:  NAll[0], Qq[0] are tloc)
 */
void mak_allefrq (PRNTMAP_PTR p_pmap, PRDIST_PTR p_ploc, int no)
{
   int   j;
   double sum;
   static int m;

   LOCUS_PTR p_locus;

   if (!Qq)
   {
      NAll = malloc (NLoci * sizeof (int));
      alloc_chk (WHERE, NAll);
      Qq = malloc (NLoci * sizeof (double *));
      alloc_chk (WHERE, Qq);
      m = 1;
   }

   if (p_ploc && p_ploc->locus_typ == MARK_LOC) /* sets no. alleles and       */
   {                                    /* stores allele freqs for marker     */
      p_locus = first_mark_locus_node;
      while (((p_locus->p_next
               && (p_locus->chrom_no < p_pmap->chrom_no))
              || (p_locus->chrom_no == p_pmap->chrom_no
                  && p_locus->no < p_ploc->no)))
         p_locus = p_locus->p_next;

      NAll[m] = p_locus->no_alleles;

      Qq[m] = malloc ((NAll[m] + 1) * sizeof (double));
      alloc_chk (WHERE, Qq[m]);

      Qq[m][0] = 0.;                    /* 1st element is unused              */
      sum = p_locus->p_freq_node->sum;
      for (j = 0; j < NAll[m]; ++j)
      {
         Qq[m][j + 1] = *(p_locus->p_freq_node->p_freq + j);
         if (alle_frq_norm)             /* normalizes freqs, if required      */
            Qq[m][j + 1] /= sum;
      }
      ++m;
   }
   else if (no == 0)                    /* no trait locus; still need Qq[][]  */
   {
      NAll[0] = 2;

      Qq[0] = malloc (3 * sizeof (double));
      alloc_chk (WHERE, Qq[0]);

      Qq[0][0] = 0.;                    /* 1st element is unused              */
      Qq[0][1] = .5;
      Qq[0][2] = .5;
   }
   else
   {                                    /* sets no. alleles and stores        */
      p_locus = first_tloc_locus_node;  /* allele freqs for tloc              */
      while (p_locus->p_next && p_locus->no < no)
         p_locus = p_locus->p_next;

      NAll[0] = p_locus->no_alleles;

      Qq[0] = malloc ((NAll[0] + 1) * sizeof (double));
      alloc_chk (WHERE, Qq[0]);

      Qq[0][0] = 0.;                    /* 1st element is unused              */
      sum = p_locus->p_freq_node->sum;
      for (j = 0; j < NAll[0]; ++j)
      {
         Qq[0][j + 1] = *(p_locus->p_freq_node->p_freq + j);
         if (alle_frq_norm)             /* normalizes freqs, if required      */
            Qq[0][j + 1] /= sum;
      }
   }
   return;
}

void mak_markdat (SIM_PTR p_sel)
{
   int   i, j, k;
   int  *p_a;
   char *p_o;
   MARKDAT_PTR p_mdt;

   if (NObs == 0)
      return;

   if (!(p_mdt = first_markdat_node))   /* finds marker data for this chrom   */
      return;

   while (p_mdt->p_next && p_mdt->chrom_no < p_sel->chrom_no)
      p_mdt = p_mdt->p_next;
   if (p_mdt->chrom_no != p_sel->chrom_no)
      return;

   ObsNam = malloc (NObs * sizeof (char *)); /* mallocs:                     */
   alloc_chk (WHERE, ObsNam);           /*        ObsNam [NObs][MAXNAMLEN+1]  */
   ObsIx = malloc (NObs * sizeof (int)); /*      ObsIx  [NObs]               */
   alloc_chk (WHERE, ObsIx);
   ObsMark = malloc (NObs * sizeof (int **)); /* ObsMark[NObs][NMarks][2]    */
   alloc_chk (WHERE, ObsMark);

   for (i = 0; i < NObs; ++i)
   {
      ObsNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, ObsNam[i]);
      ObsMark[i] = malloc (NMarks * sizeof (int *));
      alloc_chk (WHERE, ObsMark[i]);
      for (j = 0; j < NMarks; ++j)
      {
         ObsMark[i][j] = malloc (2 * sizeof (int));
         alloc_chk (WHERE, ObsMark[i][j]);
      }
   }

   p_o = p_mdt->p_obs;
   p_a = p_mdt->p_alls;
   for (i = 0; i < NObs; ++i)           /* stores indices, names, alleles     */
   {
      ObsIx[i] = 0;
      strncpy (ObsNam[i], p_o, MAXNAMLEN + 1);
      p_o += MAXNAMLEN + 1;

      for (j = 0, k = 1; k <= (p_sel->no_mark_map ?
                               p_sel->no_mark_map : 1); ++k)
      {
         if (!p_sel->p_mark ||
             (j < p_sel->no_mark_sel && k == *(p_sel->p_mark + j)))
         {
            ObsMark[i][j][0] = *p_a;
            ObsMark[i][j][1] = *(p_a + 1);
            ++j;
         }
         p_a = p_a + 2;
      }
   }
   return;
}

void mak_probgam (void)
{
   int   c, s, i = 0;
   PROBGAM_PTR p_pg;
   PROBLIST_PTR p;

   p_pg = first_probgam_node;
   while (p_pg->p_next)                 /* sets no. of components with        */
      p_pg = p_pg->p_next;              /* proband gametes                    */
   NScCmp = p_pg->component;
   NScCmpAlloc = NScCmp;

   NScSet = malloc (NScCmp * sizeof (int)); /* allocs for components         */
   alloc_chk (WHERE, NScSet);
   ScSetNo = malloc (NScCmp * sizeof (int *));
   alloc_chk (WHERE, ScSetNo);
   NPrGam = malloc (NScCmp * sizeof (int *));
   alloc_chk (WHERE, NPrGam);
   ProbNam = malloc (NScCmp * sizeof (char ***));
   alloc_chk (WHERE, ProbNam);
   ProbIx = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, ProbIx);
   ProbGam = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, ProbGam);

   p_pg = first_probgam_node;
   for (c = 0; c < NScCmp; ++c)         /* sets no. of scoresets for ea comp  */
   {
      NScSet[c] = 0;
      while (p_pg && p_pg->component <= c + 1)
      {
         ++NScSet[c];
         p_pg = p_pg->p_next;
      }
      if (!NScSet[c])                   /* gets no. of scoresets for ea comp  */
      {
         ScSetNo[c] = NULL;
         NPrGam[c] = NULL;              /* none: sets this component to empty */
         ProbNam[c] = NULL;
         ProbIx[c] = NULL;
         ProbGam[c] = NULL;
      }
      else
      {
         ScSetNo[c] = malloc (NScSet[c] * sizeof (int)); /* allocs for sets  */
         alloc_chk (WHERE, ScSetNo[c]);
         NPrGam[c] = malloc (NScSet[c] * sizeof (int));
         alloc_chk (WHERE, NPrGam[c]);
         ProbNam[c] = malloc (NScSet[c] * sizeof (char **));
         alloc_chk (WHERE, ProbNam);
         ProbIx[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, ProbIx[c]);
         ProbGam[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, ProbGam[c]);
      }
   }
   p_pg = first_probgam_node;           /* sets no. of proband gametes for    */
   for (c = 0; c < NScCmp; ++c)         /* ea scoreset                        */
      for (s = 0; s < NScSet[c]; ++s)
      {
         ScSetNo[c][s] = p_pg->scoreset;
         NPrGam[c][s] = p_pg->no_pgams; /* allocs for proband gametes         */
         ProbNam[c][s] = malloc (NPrGam[c][s] * sizeof (char *));
         alloc_chk (WHERE, ProbNam[c][s]);
         ProbIx[c][s] = malloc (NPrGam[c][s] * sizeof (int));
         alloc_chk (WHERE, ProbIx[c][s]);
         ProbGam[c][s] = malloc (NPrGam[c][s] * sizeof (int));
         alloc_chk (WHERE, ProbGam[c][s]);

         p = p_pg->p_pglist;            /* stores names and haplotypes of     */
         for (i = 0; i < p_pg->no_pgams; ++i) /* proband gametes             */
         {
            ProbNam[c][s][i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
            alloc_chk (WHERE, ProbNam[c][s][i]);
            strncpy (ProbNam[c][s][i], p->nam, MAXNAMLEN + 1);
            ProbGam[c][s][i] = p->gamete;
            ++p;
         }
         if (NPrGam[c][s] > MxGam)      /* looks for max no. prob gametes     */
            MxGam = NPrGam[c][s];
         p_pg = p_pg->p_next;
      }
   return;
}

void mak_patlab (void)
{
   int   c, s, i;
   int  *p;
   WNDPAT_PTR p_wp;

   p_wp = first_windpat_node;

   NPtLab = malloc (NScCmp * sizeof (int *)); /* allocs for components       */
   alloc_chk (WHERE, NPtLab);
   PatLab = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, PatLab);

   for (c = 0; c < NScCmp; ++c)
      if (!NScSet[c])                   /* tests for probands for this scset  */
      {
         NPtLab[c] = NULL;
         PatLab[c] = NULL;
      }
      else
      {
         NPtLab[c] = malloc (NScSet[c] * sizeof (int)); /* allocs for scsets */
         alloc_chk (WHERE, NPtLab[c]);
         PatLab[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, PatLab);

         for (s = 0; s < NScSet[c]; ++s) /* sets no. of patterns for ea      */
         {                              /* component scoreset                 */
            while (p_wp
                   && ((p_wp->p_next && p_wp->component <= c)
                       || (p_wp->p_next && p_wp->component == c + 1
                           && p_wp->scoreset < ScSetNo[c][s])))
               p_wp = p_wp->p_next;

            if (p_wp && p_wp->component == c + 1
                && p_wp->scoreset == ScSetNo[c][s])
            {
               NPtLab[c][s] = p_wp->no_pats; /* allocs for pattern labels    */
               PatLab[c][s] = malloc (NPtLab[c][s] * sizeof (int));
               alloc_chk (WHERE, PatLab[c][s]);

               p = p_wp->p_pats;        /* stores pattern labels              */
               for (i = 0; i < p_wp->no_pats; ++i, ++p)
                  PatLab[c][s][i] = *p;
            }
            else
            {
               NPtLab[c][s] = 0;        /* sets to no patterns for component  */
               PatLab[c][s] = NULL;     /* and scoreset (or none at all)      */
            }
         }
      }
   return;
}

/** 
 * makes SelInt[], SelReal[] and NColData.
 * SelInt[0] ... SelInt[n], where n is the number of integers in the ped file
 * the ith integer is to be stored in data[][i]
 * NColData is the allocation: data[][NColData]
 * SelReal[0] ... SelReal[n], where n is the number of reals in the ped file
 * the ith real is to be stored in quant[][i]
 */
void mak_ped_dat (void)
{
   int   j, tno = 0;
   TRTPED_PTR p_p;

   alloc_sel ();

   SelInt[0] = 4;                       /* (gender)                           */
   NColData = 5;                        /* NColData is 5 or greater           */

   if (!sim_traits)                     /* tests for trait selected           */
      return;

   tno = *(sim_traits->p_trait);

   if ((p_p = find_trtped (tno)))       /* looks for ped record pos for data  */
   {
      while (p_p->p_next && p_p->no < tno)
         p_p = p_p->p_next;
      if (p_p->no != tno)
         p_p = NULL;
   }

   if (PedDatTyp != QUANTITATIVE)       /* sets up SelInt[]                   */
   {
      if (!p_p)
      {
         SelInt[tno] = NColData;        /* sets record pos from trait no.     */
         ++NColData;

         if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB
             || PedDatTyp == DISCR_COVAR)
         {
            SelInt[tno + 1] = NColData; /* sets 2nd record pos for data       */
            ++NColData;
         }
      }
      else
      {
         j = p_p->pos[0];               /* sets specified record position     */
         if (SexIn != N)                /* note: SexIn may be UNDEF           */
            --j;
         SelInt[j] = NColData;
         ++NColData;

         if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB
             || PedDatTyp == DISCR_COVAR)
         {
            j = p_p->pos[1];            /* sets 2nd record pos for data       */
            if (SexIn != N)
               --j;
            SelInt[j] = NColData;
            ++NColData;
         }
      }
   }
   else if (PedDatTyp == QUANTITATIVE)  /* sets up SelReal[]                  */
   {
      if (!p_p)
         SelReal[tno - 1] = 1;          /* sets record pos from trait no.     */
      else
         SelReal[p_p->pos[0] - 1] = 1;  /* sets specified record pos          */
   }

   return;
}

/**
  * prints the pedigree record position(s) for trait data
  */

void prnt_ped_dat (void)
{
   int   i, j, ic[2], rcol;

   j = 0;
   ic[0] = 0;
   ic[1] = 0;
   if (SexIn != N)                      /* note: SexIn may be UNDEF           */
      ++j;
   for (i = j; i < NInts; ++i)
      if (SelInt[i])
      {
         if (!ic[0])
            ic[0] = i;
         else if (SelInt[ic[0]] < SelInt[i])
            ic[1] = i;
         else
         {
            ic[1] = ic[0];
            ic[0] = i;
         }
      }

   for (i = 0; i < NReals; ++i)
      if (SelReal[i])
         rcol = i;

   switch (PedDatTyp)
   {
       case INHERITANCE:
          printf (" Inheritance indicators at trait locus are to be read\n"
                  "    from integers %d (pat) and %d (mat)"
                  " in the pedigree record\n\n", ic[0] + 1, ic[1] + 1);
          break;
       case GENOTYPIC:
          printf (" Genotypic data at trait locus are to be read"
                  " from integer %d\n"
                  "    in the pedigree record\n\n", ic[0] + 1);
          break;
       case DISCRETE:
          printf (" Discrete data at trait locus are to be read"
                  " from integer %d\n"
                  "    in the pedigree record\n\n", ic[0] + 1);
          break;
       case DISCR_LIAB:
          printf (" Discrete data and liability classes at trait locus"
                  " are to be read\n"
                  "    from integers %d and %d in the pedigree record\n\n",
                  ic[0] + 1, ic[1] + 1);
          break;
       case DISCR_COVAR:
          printf (" Discrete data and covariate at trait locus are to be read\n"
                  "    from integers %d and %d in the pedigree record\n\n",
                  ic[0] + 1, ic[1] + 1);
          break;
       case QUANTITATIVE:
          printf (" Quantitative data at trait locus are to be read"
                  " from real %d\n" "    in the pedigree record\n\n", rcol + 1);
          break;
   }

   return;
}

/**
 * prints the names of the selected markers, if input.
 */
void prnt_marknam (SIM_PTR p_sel)
{
   int   i, mw, nc, no;

   if (PrintLevel >= 5)
   {
      mw = intWid (1, p_sel->no_mark_map);
      nc = get_colfmt2 (79, mw, LocNamLen, 1);

      if (p_sel->chrom_no)
         fprintf (stdout, " Marker names for chromosome %d:\n",
                  p_sel->chrom_no);
      else
         fprintf (stdout, " Marker names:\n");

      for (i = 0; i < NMarks; ++i)
      {
         if (!p_sel->p_mark)
            no = i + 1;
         else
            no = *(p_sel->p_mark + i);

         if (!(i % nc))
            fprintf (stdout, "\n ");
         fprintf (stdout, " %*d %*s ", mw, no, LocNamLen, MarkNam[i]);
      }
      fprintf (stdout, "\n\n\n");
      fflush (stdout);
   }

   return;
}

/**
  prints the genotypes for the selected markers for observed individuals.
 */
void prnt_markdat (SIM_PTR p_sel)
{
   int   i, j, n, wa, na, nm, ns, nc = 0;

   if (PrintLevel >= 5)
   {
      if (!NObs)
         return;

      if (p_sel->chrom_no)
         fprintf (stdout, " Marker data for chromosome %d:\n\n",
                  p_sel->chrom_no);
      else
         fprintf (stdout, " Marker data:\n\n");

      na = 0;
      for (i = 1; i < NLoci; ++i)       /* gets no. chars for printng alleles */
         if (NAll[i] > na)
            na = NAll[i];
      wa = intWid (0, na);

      for (i = 0; i < NObs; ++i)
      {
         n = strlen (ObsNam[i]);        /* gets no. chars for printing names  */
         nc = n > nc ? n : nc;
      }

      nm = intWid (0, NMarks);          /* and no. chars for prntng mark no.s */
      ns = 2 + 2 * wa - nm;
      if (ns < 0)
         ns = 0;

      fprintf (stdout, "%*s ", nc, " ");
      for (i = 0; i < NMarks; ++i)
      {
         if (!p_sel->p_mark)
            fprintf (stdout, "%*sM%-*d", ns, " ", nm, i + 1);
         else
            fprintf (stdout, "%*sM%-*d", ns, " ", nm, *(p_sel->p_mark + i));
      }
      fprintf (stdout, "\n\n");

      for (i = 0; i < NObs; ++i)
      {
         fprintf (stdout, " %-*s", nc, ObsNam[i]);
         for (j = 0; j < NMarks; ++j)
         {
            if (!((j - 1) % 14) && j != 1)
               fprintf (stdout, " %*s\n", nc, " ");
            fprintf (stdout, "  %*d %*d", wa, ObsMark[i][j][0], wa,
                     ObsMark[i][j][1]);
         }
         fprintf (stdout, "\n");
      }
      fprintf (stdout, "\n\n");
      fflush (stdout);
   }

   return;
}

void prnt_probgam (void)
{
   int   c, s, i, n, nn, np, nd, nc = 0;

   if (NLocWndw)                        /* prints window info                 */
   {
      NWndw = NLoci + 1 - NLocWndw;
      fprintf (stdout, " Autozygosity will be scored for %d windows of "
               "%d loci\n\n\n", NWndw, NLocWndw);
   }

   for (c = 0; c < NScCmp; ++c)
   {
      for (s = 0; s < NScSet[c]; ++s)
      {
         for (i = 0; i < NPrGam[c][s]; ++i)
         {
            n = strlen (ProbNam[c][s][i]); /* gets no. chars in names for    */
            nc = n > nc ? n : nc;       /* printing                           */
         }
      }
   }
   nn = 80 / (nc + 4);
   nd = intWid (1, MxGam);
   np = 60 / (nd + 2);

   fprintf (stdout, " Proband gametes:\n\n");
   for (c = 0; c < NScCmp; ++c)
   {
      for (s = 0; s < NScSet[c]; ++s)   /* for ea component scoreset          */
      {
         for (i = 0; i < NPrGam[c][s]; ++i) /* prints proband gametes        */
         {
            if (!i)
            {
               fprintf (stdout, " Component %d, scoreset %d:\n\n  ",
                        c + 1, ScSetNo[c][s]);
            }
            else if (!(i % nn))
               fprintf (stdout, "\n  ");
            fprintf (stdout, " %*s", nc, ProbNam[c][s][i]);
            fprintf (stdout, " %d ", ProbGam[c][s][i]);
            if (i == NPrGam[c][s] - 1)
               fprintf (stdout, "\n\n");
         }
         if ((LM_AUTO) && NLocWndw && NPrGam[c][s] && NPtLab[c][s])
            for (i = 0; i < NPtLab[c][s]; ++i) /* prints pattern labels      */
            {
               if (!i)
                  fprintf (stdout, " Window patterns:  ");
               else if (!(i % np))
                  fprintf (stdout, "\n                   ");
               fprintf (stdout, " %*d ", nd, PatLab[c][s][i]);
               if (i == NPtLab[c][s] - 1)
                  fprintf (stdout, "\n\n");
            }
         if (NPrGam[c][s])
            fprintf (stdout, "\n");
      }
   }
   fflush (stdout);

   return;
}

void prnt_affind (void)
{
   fprintf (stdout, " Affected individuals to be identified by");
   if (TraitTyp == GENOTYPIC)
      fprintf (stdout, " genotypic");
   else if (TraitTyp == DISCRETE)
      fprintf (stdout, " discrete");
   fprintf (stdout, " data\n    for trait %d in the pedigree file\n\n",
            *(sim_traits->p_trait));

   fflush (stdout);

   return;
}

void prnt_trt_typ (void)
{
   fprintf (stdout, "\n");
   if (TraitTyp == INHERITANCE)
      fprintf (stdout, " Data are inheritance indicators at trait locus\n\n");
   else if (TraitTyp == GENOTYPIC)
      fprintf (stdout, " Trait data are genotypic\n\n");
   else if (TraitTyp == DISCRETE)
   {
      if (PedDatTyp == DISCR_LIAB)
         fprintf (stdout, " Trait data are discrete with liability classes"
                  " provided\n\n");
      else if (PedDatTyp == DISCR_COVAR)
         fprintf (stdout, " Trait data are discrete with covariates"
                  " provided\n\n");
      else
         fprintf (stdout, " Trait data are discrete\n\n");
   }
   else if (TraitTyp == QUANTITATIVE)
      fprintf (stdout, " Trait data are quantitative\n\n");
}

void prnt_auto_pars (void)
{
   int   n;

   if (IBDDROP)
   {
      fprintf (stdout, " Number of MC iterations requested:  %d\n\n", NIt);
      return;
   }

   if (LM_MAP || LM_PVAL || LM_IBDTESTS || LM_AUTO)
   {
      if (SmplSetup == INDEP_LOC)
         fprintf (stdout, " Locus-by-locus sampling to be used for setup\n\n");
      else if (SmplSetup == SEQIMP)
         fprintf (stdout, " Sequential imputation to be used for setup,"
                  " using %d samples \n\n", NSeqImp);
      if (PeelCmpnt)
         fprintf (stdout, " Peeling is to be by pedigree component\n\n");

      if (GlobalMcmc)
         fprintf (stdout,
                  " Lod scores for all components are estimated globally\n\n");

      if (Scan)
      {
         fprintf (stdout, " Sampling is to be by scan");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         else if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every scan\n");
         else
            fprintf (stdout, ", with scores estimated every %d scans\n",
                     ScorIt);
      }
      else
      {
         fprintf (stdout, " Sampling is to be by single step");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every step\n");
         else
            fprintf (stdout, ", with scores estimated every %d steps\n",
                     ScorIt);
      }
      fprintf (stdout, "\n");

      fprintf (stdout, " L-sampler probability set to %.3f\n\n", LSamplProb);

      if (NBrnIt != UNDEF)
      {
         fprintf (stdout, " The number of MC burn-in iterations is %d\n",
                  NBrnIt);
         fprintf (stdout, "\n");
      }
   }
   if (!LM_MAP && NIt != UNDEF)
   {
      fprintf (stdout, " Number of MC iterations requested is %d\n", NIt);
      if (ChkIt != UNDEF)
         fprintf (stdout, " with progress checked every %d iterations\n",
                  ChkIt);
   }

   if (LM_MAP)
   {
      if (MapEstMod != MISTYP)
      {
         fprintf (stdout, " \n Map estimation model is without mistyping\n");
         fprintf (stdout, " \n Number of iterations to compute likelihood"
                  " ratio test (LRT) statistics is %d\n", NChiSqIt);
         fprintf (stdout, "\n %d map-estimation %s\n", RedoEst,
                  RedoEst == 1 ? "run is to be made" : "passes are to be made");

         if (MaximAlgo == HYBRID)
         {
            fprintf (stdout, " \n The hybrid algorithm, using both SA and"
                     " MCEM for maximization, is to be used\n");
            fprintf (stdout, " The number of curvature iterations for the"
                     " hybrid algorithm is %d\n", NHybCurvIt);
            fprintf (stdout, " The number of ascent iterations for hybrid"
                     " algorithm is %d\n", NHybAscentIt);
            fprintf (stdout, " The number of gradient iterations for SA"
                     " is %d\n", NSAGradIt);
            fprintf (stdout, " SA convergence for successive estimates is set"
                     " to %6.4f\n", SA_bound);
         }
         else if (MaximAlgo == MCEM)
            fprintf (stdout, " \n MCEM only is to be used for maximization\n");
      }
      else if (MapEstMod == MISTYP)
      {
         fprintf (stdout, " \n Map estimation model is with possible"
                  " mistyping\n");
         fprintf (stdout, " The initial genotyping error rate is %6.4f\n",
                  GenoErrRat);
      }

      fprintf (stdout, " \n The number of MC iterations for the first MCEM"
               " step is %d\n", NIt);
      fprintf (stdout, " Recombination fraction estimates are restricted"
               " to the interval [%.2f, %.2f]\n", RF_bound, .5 - RF_bound);
      fprintf (stdout, " EM convergence for successive estimates is set"
               " to %.4f\n", EM_bound);

      switch (MapTyp)
      {
          case SEX_SPEC:
             fprintf (stdout, " Gender-specific");
             break;
          case SEX_AVG:
             fprintf (stdout, " Gender-averaged");
             break;
          case BOTH:
             fprintf (stdout, " Both gender-averaged and -specific");
             break;
      }
      fprintf (stdout, " maps will be estimated\n");
   }

   if (LM_PVAL)
   {
      fprintf (stdout, "\n IBD statistics are to be computed\n");
      fprintf (stdout, "    measures:   ");
      fprintf (stdout, "MaxFGL - %s   Spairs - %s   Srobdom - %s\n",
               MaxFGLOn ? "yes" : "no", SpairsOn ? "yes" : "no",
               RobdomOn ? "yes" : "no");
   }
   else if (LM_IBDTESTS)
   {
      if (IBDStats)
      {
         fprintf (stdout, "\n IBD statistics are to be computed\n");
         fprintf (stdout, "    measures:   ");
         fprintf (stdout, "Spairs - %s   Srobdom - %s   Saffect - %s"
                  "   Slambda - %s\n",
                  SpairsOn ? "yes" : "no", RobdomOn ? "yes" : "no",
                  SAffUnaffOn ? "yes" : "no", LambdaOn ? "yes" : "no");
         fprintf (stdout, "    tests:      ");
         fprintf (stdout, "normal - %s   permutation - %s\n",
                  NormalEnable ? "yes" : "no", PermutEnable ? "yes" : "no");
         if (PermutEnable)
            fprintf (stdout, "\n    the number of permutations is %d\n",
                     NPermutIt);
         fprintf (stdout, "\n");
      }
      else if (LikRatStats)
      {
         n = DeltaOn + Lambda_pOn;
         fprintf (stdout,
                  "\n Likelihood-ratio statistics are to be computed\n");
         fprintf (stdout, "    measures:   ");
         fprintf (stdout, "delta - %s   lambda-p - %s\n",
                  DeltaOn ? "yes" : "no", Lambda_pOn ? "yes" : "no");
         fprintf (stdout, "    tests:      ");
         fprintf (stdout, "chi-squared - %s\n", ChiSqEnable ? "yes" : "no");
         if (Lambda_pOn)
         {
            fprintf (stdout, "\n    lambda_a, lambda_u interval:"
                     "  gridpoints - %d  spacing %.4f\n",
                     NumOfSegL, IntervalSegL);
            fprintf (stdout, "    p interval:                 "
                     "  gridpoints - %d  spacing %.4f\n",
                     NumOfSegP, IntervalSegP);
         }
      }
   }
   fprintf (stdout, "\n");

   return;
}

int get_marknam (char *nam, char *p_nm, int imrk)
{
   strncpy (nam, p_nm + (imrk - 1) * (MAXNAMLEN + 1), MAXNAMLEN + 1);

   return strlen (nam);
}

/**
 * returns a pointer to the trait/inheritance pedigree data node.
 */
TRTPED_PTR find_trtped (int ti)
{
   int   found;
   TRTPED_PTR p;

   if ((p = first_trtped_node))
   {
      if (TRAIT_SELCTD)
      {
         while (p->p_next
                && (p->no < ti || (p->no == ti && p->trait_typ != SS_TRLIST)))
            p = p->p_next;
         if (p->no == ti && p->trait_typ == SS_TRLIST)
            found = TRUE;
      }

      else if (INHER_SELCTD)
      {
         while (p->p_next
                && (p->no < ti || (p->no == ti && p->trait_typ != SS_INHER)))
            p = p->p_next;
         if (p->no == ti && p->trait_typ == SS_INHER)
            found = TRUE;
      }
   }
   if (found)
      return p;
   else
      return NULL;
}
