
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* These functions are called by the parameter statement functions, appearing
 * in proc_pars_ss.c and named in parsedefs.h
 */

/**
 * tests for chrom no., marker no's out of range.
 * checks no. of markers to be sim/sel correct for program.
 * sorts marker list and checks for duplication of marker no's.
 */
int chk_sim (int chrom_num_included, int m)
{
   int   i, j = 0;
   int   sav, ix = 0;

   sav = err_typ;
   err_typ = 0;

   if (chrom_num_included == CH)
   {
      if (j_par[ix] < 1 || j_par[ix] > psx->max_chrom)
         err_typ = ERD_CHO;
      ++ix;
   }

   if (err_typ)
      return FALSE;

   if (SS_MKALL == m)                   /* does no checking for "all" markers */
      ;
   else if (SS_MKLIST == m)             /* checks marker no's in list         */
   {
      for (i = ix; i < nk_par; ++i, ++ix)
      {
         if (j_par[ix] < 1)
            err_typ = ERD_MKO;
         ++j;
      }
      if (j > psx->max_mark)            /* checks no. markers requested       */
         err_typ = ERD_MKX;

      if (!err_typ)                     /* sorts mark no's & checks for dups  */
      {
         qsort (&(j_par[ix]), nk_par - ix, sizeof (int), qfunc_int);

         for (i = ix; i < nk_par - 1; ++i, ++ix)
            if (j_par[i] == j_par[i + 1])
            {
               err_typ = ERD_DMK;
               sprintf (errnumb, "%d", j_par[i]);
               err_txt = errnumb;
            }
      }
   }
   if (err_typ)
      return FALSE;

   err_typ = sav;
   return TRUE;
}

/**
 * tests for chrom no., trait no's out of range.
 * checks no. of traits to be sim/sel correct for program.
 * sorts trait list and checks for duplication of marker no's.
 */
int chk_sim_tr (int m)
{
   int   i;
   int   sav, ix = 0;

   sav = err_typ;
   err_typ = 0;

   if (SS_TRLIST == m)                  /* checks list of trait no's          */
   {
      if (nk_par > psx->max_trait)
         err_typ = ERD_TRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTR)
               err_typ = ERD_TRO;
   }
   else if (SS_TLLIST == m)             /* checks list of tloc no's           */
   {
      if (nk_par > psx->max_tloc)
         err_typ = ERD_TRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTL)
               err_typ = ERD_TRO;
   }
   else if (SS_INHER == m)
   {
      if (nk_par > psx->max_trait)
         err_typ = ERD_HRX;
      else
         for (i = 0; i < nk_par; ++i)
            if (j_par[i] < 1 || j_par[i] > MAXTR)
               err_typ = ERD_IHO;
   }

   if (!err_typ && nk_par > 1)          /* sorts trait no's & checks for dups */
   {
      qsort (&(j_par[0]), nk_par, sizeof (int), qfunc_int);

      for (i = 0; i < nk_par - 1; ++i, ++ix)
         if (j_par[i] == j_par[i + 1])
         {
            if (SS_TRLIST == m)
               err_typ = ERD_DTR;
            else
               err_typ = ERD_DTW;
            sprintf (errnumb, "%d", j_par[i]);
            err_txt = errnumb;
         }
   }

   if (err_typ)
      return FALSE;

   err_typ = sav;
   return TRUE;
}

/**
 * Checks for this chromosome already sim/sel,
 * and for markers already specified.
 */

SIM_PTR chk_sim_node (int chrom_num_included, int m)
{
   SIM_PTR p_node;

   int   n = 0;

   if (chrom_num_included == CH)
      n = j_par[0];

   if ((p_node = first_sim_chrom_node))
   {
      while (p_node->p_next && p_node->chrom_no < n)
         p_node = p_node->p_next;

      if (p_node->chrom_no == n)
      {
         if (SS_MKALL == m || SS_MKLIST == m)
         {
            if (p_node->no_mark_sel)    /* checks for existing marker list    */
            {
               if (n == 0)
                  err_typ = ERD_RSM;
               else
                  err_typ = ERD_RCM;
            }
         }
         return p_node;
      }
   }
   return NULL;
}

/**
 * checks for sim/sel of traits or of inheritance already specified
 */

SIMTRAIT_PTR chk_sim_tr_node (void)
{
   SIMTRAIT_PTR p_node;

   if ((p_node = sim_traits))
   {
      if (p_node->trait_typ == SS_TRLIST)
         err_typ = ERD_RST;
      else if (p_node->trait_typ == SS_TLLIST)
         err_typ = ERD_RSL;
      else if (p_node->trait_typ == SS_INHER)
         err_typ = ERD_RSI;
   }
   return p_node;
}

/**
 * Checks set effects for a specified trait.
 */

int chk_set_tr_eff ()
{
   int   ix;

   if (j_par[0] < 1 || j_par[0] > MAXTR) /* Checks for a valid trait number.  */
   {
      err_typ = ERD_TRO;
      return FALSE;
   }

   if (nk_par > psx->max_fef)           /* Checks maximum number of effects.  */
   {
      err_typ = ERR_SFN;
      err_txt = psx->pnam;
      return FALSE;
   }

   for (ix = 1; ix <= nk_par; ++ix)     /* Checks for valid effect numbers.   */
      if (j_par[ix] < 1 || j_par[ix] > FMAX_FEF)
      {
         err_typ = ERR_FFO;
         return FALSE;
      }

   /* Sort the fixed effect numbers into ascending order.                     */
   qsort (&(j_par[1]), nk_par, sizeof (int), qfunc_int);

   for (ix = 1; ix < nk_par; ++ix)      /* Checks for repeat effect numbers.  */
      if (j_par[ix] == j_par[ix + 1])
      {
         err_typ = ERR_FFD;
         sprintf (errnumb, "%d", j_par[ix]);
         err_txt = errnumb;
      }

   return TRUE;
}

/**
 * Checks the set tlocs statement for the trait.
 */

int chk_set_tr_tl (int trait_set_option)
{
   int   ix, nt, nl;
   int  *tab;

   if (trait_set_option == TRLIST_TLLIST) /* Get number of traits and tlocs.  */
   {
      nt = nk_par / 2;
      nl = nt;
      if (nt + nl != nk_par)            /* Same number of traits and tlocs?   */
         err_typ = ERD_TTL;
   }
   else if (trait_set_option == TR_TLLIST)
   {
      nt = 1;
      nl = nk_par - 1;
      if (nl > psx->max_tloc)           /* Checks number of tlocs for trait.  */
      {
         err_typ = ERD_MTL;
         sprintf (errnumb, "%d", psx->max_tloc);
         err_txt = errnumb;
      }
   }

   if (err_typ)
      return FALSE;

   for (ix = 0; ix < nt; ++ix)          /* Checks trait numbers are valid.    */
      if (j_par[ix] < 1 || j_par[ix] > MAXTR)
      {
         err_typ = ERD_TRO;
         return FALSE;
      }

   for (ix = nt; ix < nt + nl; ++ix)    /* Checks tloc numbers are valid.     */
      if (j_par[ix] < 1 || j_par[ix] > MAXTL)
         err_typ = ERD_TLO;

   if (err_typ)
      return FALSE;

   if (nt > 1)                          /* Reorders array by trait numbers.   */
   {
      tab = malloc (nt * 2 * sizeof (int));
      alloc_chk (WHERE, tab);
      for (ix = 0; ix < nt; ++ix)
      {
         tab[2 * ix] = j_par[ix];
         tab[2 * ix + 1] = j_par[nt + ix];
      }
      qsort (tab, nt, 2 * sizeof (int), qfunc_int); /* Sort trt or inh numbs. */

      for (ix = 0; ix < nt - 1; ++ix)   /* Checks for duplicate trait numbers. */
         if (tab[2 * ix] == tab[2 * ix + 2])
         {
            err_typ = ERD_DTR;
            sprintf (errnumb, "%d", tab[2 * ix]);
            err_txt = errnumb;
         }

      for (ix = 0; ix < 2 * nt; ++ix)   /* Copy sorted trait numbers back.    */
         j_par[ix] = tab[ix];
      free (tab);
   }

   if (err_typ)
      return FALSE;

   return TRUE;
}

/**
 * Checks for a trait_effect_node already present for this trait.
 */
TRAITEFF_PTR chk_tr_eff_node (int it)
{
   TRAITEFF_PTR p_node;

   if ((p_node = first_trait_eff_node))
   {
      while (p_node->p_next && p_node->trait_num < j_par[it])
         p_node = p_node->p_next;

      if (p_node->trait_num == j_par[it])
      {
         if (p_node->set_trait_eff_inp)
         {
            err_typ = ERD_REF;          /* Replacing effects for this trait.  */
            sprintf (errnumb, "%d", j_par[it]);
            err_txt = errnumb;
         }
         return p_node;
      }
   }
   return NULL;
}

/**
 * Checks for a trait/tloc node already present for this trait.
 */
TRAITTLOC_PTR chk_tr_tl_node (int it)
{
   TRAITTLOC_PTR p_node;

   if ((p_node = first_trait_tloc_node))
   {
      while (p_node->p_next && p_node->trait_no < j_par[it])
         p_node = p_node->p_next;

      if (p_node->trait_no == j_par[it])
      {
         err_typ = ERD_RTL;
         return p_node;
      }
   }
   return NULL;
}

/**
 * Checks set tlocs for inheritances.
 */

int chk_set_inh_tl (void)
{
   int   i, ni, nl;
   int  *tab;

   ni = nk_par / 2;                     /* gets no. inherits and no. tlocs    */
   nl = ni;
   if (ni + nl != nk_par)               /* checks same no.                    */
   {
      err_typ = ERD_ITL;
      return FALSE;
   }

   for (i = 0; i < ni; ++i)             /* checks inheritance no's            */
      if (j_par[i] < 1 || j_par[i] > MAXTR)
      {
         err_typ = ERD_IHO;
         return FALSE;
      }

   for (i = ni; i < ni + nl; ++i)       /* checks tloc no's                   */
      if (j_par[i] < 1 || j_par[i] > MAXTL)
      {
         err_typ = ERD_TLO;
         return FALSE;
      }

   if (ni > 1)                          /* reorders array by inheritance no's */
   {
      tab = malloc (ni * 2 * sizeof (int));
      alloc_chk (WHERE, tab);
      for (i = 0; i < ni; ++i)
      {
         tab[2 * i] = j_par[i];
         tab[2 * i + 1] = j_par[ni + i];
      }
      qsort (tab, ni, 2 * sizeof (int), qfunc_int);

      for (i = 0; i < ni - 1; ++i)      /* checks for dup inheritance no's   */
         if (tab[2 * i] == tab[2 * i + 2])
         {
            err_typ = ERD_DTR;
            sprintf (errnumb, "%d", tab[2 * i]);
            err_txt = errnumb;
         }
      for (i = 0; i < 2 * ni; ++i)      /* copies back                        */
         j_par[i] = tab[i];
      free (tab);
   }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks for inher/tloc node already present for this inheritance no.
 */
INHERTLOC_PTR chk_inh_tl_node (int it)
{
   INHERTLOC_PTR p_node;

   if ((p_node = first_inher_tloc_node))
   {
      while (p_node->p_next && p_node->inher_no < j_par[it])
         p_node = p_node->p_next;

      if (p_node->inher_no == j_par[it])
      {
         err_typ = ERD_RIL;
         return p_node;
      }
   }
   return NULL;
}

/**
 * Allocates a new sim_node.  Then inserts the new sim_node in the linked list
 * in ascending chromosome number order.
 */
SIM_PTR alloc_sim_node (int chrom_num_included)
{
   SIM_PTR new_node, cur_node;
   int   n = 0;

   if (chrom_num_included == CH)
      n = j_par[0];

   new_node = malloc (sizeof (SIM));
   alloc_chk (WHERE, new_node);
   new_node->chrom_no = n;
   new_node->mark_typ = 0;
   new_node->no_mark_sel = 0;
   new_node->no_mark_map = 0;
   new_node->p_mark = NULL;
   new_node->no_tloc_sel = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_sim_chrom_node || first_sim_chrom_node->chrom_no > n)
   {
      new_node->p_next = first_sim_chrom_node; /* Puts at start of list.      */
      first_sim_chrom_node = new_node;  /* (may be only node)                 */
   }
   else
   {
      cur_node = first_sim_chrom_node;  /* Inserts node in list               */
      /*                                   or adds to end of list.            */
      while (cur_node->p_next && cur_node->p_next->chrom_no < n)
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 * Allocates the trait/tloc/inher sim/sel node.
 */
SIMTRAIT_PTR alloc_sim_tr_node (void)
{
   sim_traits = malloc (sizeof (SIMTRAIT));
   alloc_chk (WHERE, sim_traits);
   sim_traits->trait_typ = 0;
   sim_traits->no_traits = 0;
   sim_traits->p_trait = NULL;

   return sim_traits;
}

/**
 *  Allocate a trait_effects_node for this trait and add to list.
 */
TRAITEFF_PTR alloc_tr_eff_node (int it)
{
   TRAITEFF_PTR new_node, cur_node;

   new_node = malloc (sizeof (TRAITEFF));
   alloc_chk (WHERE, new_node);

   new_node->trait_num = j_par[it];
   new_node->set_trait_eff_inp = 0;
   new_node->start_tr_mean_inp = 0;
   new_node->start_tr_mean = 0.0;
   new_node->num_eff = 0;
   new_node->eff_nums = NULL;
   new_node->p_eff_list = NULL;
   new_node->p_next = NULL;

   if (!first_trait_eff_node || first_trait_eff_node->trait_num > j_par[it])
   {
      new_node->p_next = first_trait_eff_node; /* Puts at start of list.      */
      first_trait_eff_node = new_node;  /* (may be only node)                 */
   }
   else
   {
      cur_node = first_trait_eff_node;  /* Inserts in or adds to end of list. */
      while (cur_node->p_next && cur_node->p_next->trait_num < j_par[it])
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 *  Allocate a fixed_eff_node for the selected fixed effect and add to the
 *  linked list of fixed effects for this trait with user specified start
 *  values.
 */
FIXEDEFF_PTR alloc_fixed_eff_node (TRAITEFF_PTR p_t, int it)
{
   FIXEDEFF_PTR new_node, cur_node;

   new_node = malloc (sizeof (FIXEDEFF));
   alloc_chk (WHERE, new_node);

   new_node->eff_num = j_par[it];
   new_node->nstartff = 0;
   new_node->startff = NULL;
   new_node->p_next = NULL;

   cur_node = p_t->p_eff_list;
   if (!cur_node || cur_node->eff_num > j_par[it])
   {
      new_node->p_next = cur_node;      /* Puts at start of linked list.      */
      p_t->p_eff_list = new_node;       /* This may be the only node.         */
   }
   else
   {
      /* Inserts in or adds to end of the linked list.                        */
      while (cur_node->p_next && cur_node->p_next->eff_num < j_par[it])
         cur_node = cur_node->p_next;

      if (cur_node->eff_num == j_par[it])
      {
         /* If trait_num and eff_num in cur_node both equal those from the    */
         /* new "start trait K1 effect M1 X1 X2 ..." statement, the previous  */
         /* set of start values should be replaced by this new set of start   */
         /* values.  In this case, the new fixed_eff_node is not needed.      */
         if (cur_node->startff)
            free_fixed_eff_array (cur_node);
         free (new_node);
         new_node = cur_node;
         err_typ = ERR_FFR;
      }
      else
      {
         new_node->p_next = cur_node->p_next;
         cur_node->p_next = new_node;
      }
   }
   return new_node;
}

/**
 *  Allocate the trait/tloc node for this trait and add to list.
 */
TRAITTLOC_PTR alloc_tr_tl_node (int it)
{
   TRAITTLOC_PTR new_node, cur_node;

   new_node = malloc (sizeof (TRAITTLOC));
   alloc_chk (WHERE, new_node);

   new_node->trait_no = j_par[it];
   new_node->list_typ = 0;
   new_node->selectd = 0;
   new_node->no_loci = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_trait_tloc_node || first_trait_tloc_node->trait_no > j_par[it])
   {
      new_node->p_next = first_trait_tloc_node; /* puts at start of list      */
      first_trait_tloc_node = new_node; /* (may be only node)                 */
   }
   else
   {
      cur_node = first_trait_tloc_node; /* inserts in or adds to end of list  */
      while (cur_node->p_next && cur_node->p_next->trait_no < j_par[it])
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 *  Allocate the inher/tloc node for this inheritance and add to list.
 */
INHERTLOC_PTR alloc_inh_tl_node (int it)
{
   INHERTLOC_PTR new_node, cur_node;

   new_node = malloc (sizeof (INHERTLOC));
   alloc_chk (WHERE, new_node);

   new_node->inher_no = j_par[it];
   new_node->list_typ = 0;
   new_node->selectd = 0;
   new_node->no_loci = 0;
   new_node->p_tloc = NULL;
   new_node->p_next = NULL;

   if (!first_inher_tloc_node || first_inher_tloc_node->inher_no > j_par[it])
   {
      new_node->p_next = first_inher_tloc_node; /* puts at start of list      */
      first_inher_tloc_node = new_node; /* (may be only node)                 */
   }
   else
   {
      cur_node = first_inher_tloc_node; /* inserts in or adds to end of list  */
      while (cur_node->p_next && cur_node->p_next->inher_no < j_par[it])
         cur_node = cur_node->p_next;

      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return new_node;
}

/**
 *  Allocates the array of markers to simulate or select.
 */
void alloc_mark_array (SIM_PTR p_node)
{
   p_node->p_mark = malloc (p_node->no_mark_sel * sizeof (int));
   alloc_chk (WHERE, p_node->p_mark);

   return;
}

/**
 *  Allocates the array of traits to simulate or select.
 */
void alloc_trait_array (SIMTRAIT_PTR p_node)
{
   p_node->p_trait = malloc (p_node->no_traits * sizeof (int));
   alloc_chk (WHERE, p_node->p_trait);

   return;
}

/**
 *  Allocates the array of start values for the selected fixed effect.
 */
void alloc_fixed_eff_array (FIXEDEFF_PTR p_node)
{
   p_node->nstartff = nx_par;

   p_node->startff = malloc (p_node->nstartff * sizeof (double));
   alloc_chk (WHERE, p_node->startff);

   return;
}

/**
 *  Allocates the array of fixed effect numbers for the trait.
 */
void alloc_tr_eff_array (TRAITEFF_PTR p_node)
{
   p_node->num_eff = nk_par;

   p_node->eff_nums = malloc (p_node->num_eff * sizeof (int));
   alloc_chk (WHERE, p_node->eff_nums);

   return;
}

/**
 *  Allocates the array of tloc numbers for the trait.
 */
void alloc_tr_tl_array (TRAITTLOC_PTR p_node, int trait_set_option)
{
   if (trait_set_option == TRLIST_TLLIST)
      p_node->no_loci = 1;
   else if (trait_set_option == TR_TLLIST)
      p_node->no_loci = nk_par - 1;

   p_node->p_tloc = malloc (p_node->no_loci * sizeof (int));
   alloc_chk (WHERE, p_node->p_tloc);

   return;
}

/**
 *  Allocates the array of tloc numbers for the inheritance.
 */
void alloc_inh_tl_array (INHERTLOC_PTR p_node)
{
   p_node->no_loci = nk_par - 1;

   p_node->p_tloc = malloc (p_node->no_loci * sizeof (int));
   alloc_chk (WHERE, p_node->p_tloc);

   return;
}

/**
 *  Free the array of markers chosen for simulation or selection.
 */
void free_mark_array (SIM_PTR p_s)
{
   if (p_s && p_s->p_mark)
   {
      p_s->no_mark_sel = 0;
      free (p_s->p_mark);
      p_s->p_mark = NULL;
   }
   return;
}

/**
 *  Free the array of traits chosen for simulation or selection.
 */
void free_trait_array (SIMTRAIT_PTR p_s)
{
   if (p_s && p_s->p_trait)
   {
      p_s->no_traits = 0;
      free (p_s->p_trait);
      p_s->p_trait = NULL;
   }
   return;
}

/**
 *  Free the array of start values for this fixed effect.
 */
void free_fixed_eff_array (FIXEDEFF_PTR p_f)
{
   if (p_f && p_f->startff)
   {
      p_f->nstartff = 0;
      free (p_f->startff);
      p_f->startff = NULL;
   }
   return;
}

/**
 * Free the array of fixed effect numbers for this trait.
 */
void free_tr_eff_array (TRAITEFF_PTR p_t)
{
   if (p_t && p_t->eff_nums)
   {
      p_t->num_eff = 0;
      free (p_t->eff_nums);
      p_t->eff_nums = NULL;
   }
   return;
}

/**
 * Free the array of tloc numbers for this trait.
 */
void free_tr_tl_array (TRAITTLOC_PTR p_t)
{
   if (p_t && p_t->p_tloc)
   {
      p_t->no_loci = 0;
      free (p_t->p_tloc);
      p_t->p_tloc = NULL;
   }
   return;
}

/**
 * Free the array of tloc numbers for this inheritance.
 */
void free_inh_tl_array (INHERTLOC_PTR p_i)
{
   if (p_i && p_i->p_tloc)
   {
      p_i->no_loci = 0;
      free (p_i->p_tloc);
      p_i->p_tloc = NULL;
   }
   return;
}

/**
 *  Sets the marker specification type, chromosome number and the number of
 *  markers.
 */
void stor_sim_data (SIM_PTR p_node, int chrom_num_included, int m)
{
   if (chrom_num_included == CH)
      p_node->chrom_no = j_par[0];
   else
      p_node->chrom_no = 0;

   if (SS_MKALL == m || SS_MKLIST == m) /* set sim/sel marker statement type  */
   {
      p_node->mark_typ = m;
      if (SS_MKALL == m)                /* set all markers sim/sel            */
         p_node->no_mark_sel = -1;
      else if (SS_MKLIST == m)          /* or set a list of markers sim/sel   */
         p_node->no_mark_sel = nk_par;
   }

   return;
}

/**
 *  Sets the trait/tloc/inher type and the number of traits or trait loci.
 */
void stor_sim_tr_data (SIMTRAIT_PTR p_node, int m)
{
   p_node->trait_typ = m;               /* Set sim/sel trait/tloc/inher type. */
   p_node->no_traits = nk_par;

   return;
}

/**
 *  Stores the array of markers chosen for simulation or selection, sorted in
 *  ascending order by marker number.
 */
void stor_mark_array (SIM_PTR p_node, int chrom_num_included)
{
   int   ix;
   int  *p_m;

   ix = (chrom_num_included == CH ? 1 : 0);

   for (p_m = p_node->p_mark; p_m < p_node->p_mark + p_node->no_mark_sel; ++p_m)
      *p_m = j_par[ix++];

   return;
}

/**
 *  Stores the array of traits, inheritances or tlocs chosen for simulation or
 *  selection.
 */
void stor_trait_array (SIMTRAIT_PTR p_node)
{
   int   ix = 0;
   int  *p_t;

   for (p_t = p_node->p_trait; p_t < p_node->p_trait + p_node->no_traits; ++p_t)
      *p_t = j_par[ix++];

   return;
}

/**
 *  Stores the array of start values for the selected fixed effect.  Note that
 *  the start values have been entered in increasing level number order.  This
 *  means that the order of the start values has to be retained as the user
 *  entered them.  They should not be sorted in any way.
 */
void stor_fixed_eff_array (FIXEDEFF_PTR p_node)
{
   int   ix = 0;
   double *p_f;

   for (p_f = p_node->startff; p_f < p_node->startff + p_node->nstartff; ++p_f)
      *p_f = d_par[ix++];

   return;
}

/**
 *  Stores the array of fixed effect numbers for the trait.  Note that the fixed
 *  effect numbers were already sorted into ascending order in the
 *  chk_set_tr_eff subroutine, so they do not need to be sorted in this
 *  subroutine.
 */
void stor_tr_eff_array (TRAITEFF_PTR p_node)
{
   int   ix = 1;
   int  *p_t;

   for (p_t = p_node->eff_nums; p_t < p_node->eff_nums + p_node->num_eff; ++p_t)
      *p_t = j_par[ix++];
/* qsort (p_node->eff_nums, p_node->num_eff, sizeof (int), qfunc_int);        */

   return;
}

/**
 *  Stores the array of tloc numbers for the trait.
 */
void stor_tr_tl_array (TRAITTLOC_PTR p_node, int it, int trait_set_option)
{
   int  *p_t;

   if (trait_set_option == TRLIST_TLLIST)
      *(p_node->p_tloc) = j_par[it + 1];

   else if (trait_set_option == TR_TLLIST)
   {
      for (p_t = p_node->p_tloc; p_t < p_node->p_tloc + p_node->no_loci; ++p_t)
         *p_t = j_par[++it];
      qsort (p_node->p_tloc, p_node->no_loci, sizeof (int), qfunc_int);
   }

   return;
}

/**
 *  Stores the array of tloc numbers for the inheritance.
 */
void stor_inh_tl_array (INHERTLOC_PTR p_node, int it)
{
   *(p_node->p_tloc) = j_par[it + 1];

   return;
}
