/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* Parameter statement functions for "simulate ..." and "select ..."
 * statements, both for markers and for traits, are included here.
 * Also included are the "set trait .. effects ..", "set trait .. tloc .."
 * and "set inheritance .. tloc .." statements.
 */

int sim_mk (void)                       /* simulate markers (all)             */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, SS_MKALL))
      {
         if (!(p_sim = chk_sim_node (NCH, SS_MKALL))) /* tests for exist node */
            p_sim = alloc_sim_node (NCH);

         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);

         stor_sim_data (p_sim, NCH, SS_MKALL); /* stores info in sim node     */
      }
   }
   return err_typ;
}

int sim_mk_c (void)                     /* simulate markers (all) on chrom    */
{
   SIM_PTR p_sim;

   if (chk_stat ())
      if (chk_sim (CH, SS_MKALL))
      {
         if (!(p_sim = chk_sim_node (CH, SS_MKALL))) /* tests for exist node */
            p_sim = alloc_sim_node (CH);

         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);

         stor_sim_data (p_sim, CH, SS_MKALL); /* stores info in sim node      */
      }
   return err_typ;
}

int sim_tr (void)                       /* simulate traits                    */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TRLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing trait           */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TRLIST); /* stores info in node       */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }
   return err_typ;
}

int sim_tl (void)                       /* simulate tloc                      */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TLLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing tloc            */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TLLIST); /* stores info in node       */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }
   return err_typ;
}

int sel_inh (void)                      /* select inheritance  REV*****       */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_INHER))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing tloc            */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_INHER); /* stores info in node        */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }
   return err_typ;
}

int sel_mk_all (void)                   /* select all markers                 */
{                                       /* SAME CODE as sim_mk:  REPLACE???   */
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, SS_MKALL))
      {
         if (!(p_sim = chk_sim_node (NCH, SS_MKALL))) /* tests for exist node */
            p_sim = alloc_sim_node (NCH);

         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);

         stor_sim_data (p_sim, NCH, SS_MKALL); /* stores sel info in sim node */
      }
   }
   return err_typ;
}

int sel_mk_c_all (void)                 /* select all markers on chromosome   */
{                                       /* same code as sim_mk_c:  REPLACE??  */
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, SS_MKALL))
      {
         if (!(p_sim = chk_sim_node (CH, SS_MKALL))) /* tests for exist node  */
            p_sim = alloc_sim_node (CH);

         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);

         stor_sim_data (p_sim, CH, SS_MKALL); /* stores sel info in sim node  */
      }
   }
   return err_typ;
}

int sel_mk (void)                       /* select markers (list)              */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, SS_MKLIST))
      {
         if (!(p_sim = chk_sim_node (NCH, SS_MKLIST))) /* tests for exist nod */
            p_sim = alloc_sim_node (NCH);
         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);
         stor_sim_data (p_sim, NCH, SS_MKLIST); /* stores sel info in sim nod */
         alloc_mark_array (p_sim);
         stor_mark_array (p_sim, NCH);  /* and sets marker list               */
      }
   }
   return err_typ;
}

int sel_mk_c (void)                     /* select markers (list) on chrom     */
{
   SIM_PTR p_sim;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, SS_MKLIST))
      {
         if (!(p_sim = chk_sim_node (CH, SS_MKLIST))) /* tests for exist node */
            p_sim = alloc_sim_node (CH);
         if (p_sim->no_mark_sel)        /* tests for existing marker list     */
            free_mark_array (p_sim);
         stor_sim_data (p_sim, CH, SS_MKLIST); /* stores sel info in sim node */
         alloc_mark_array (p_sim);
         stor_mark_array (p_sim, CH);   /* and sets marker list               */
      }
   }
   return err_typ;
}

int sel_tr (void)                       /* select traits                      */
{                                       /* SAME CODE as sim_tr:  RENAME??     */
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TRLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* Tests for an existing node. */
            p_sim_tr = alloc_sim_tr_node ();
         if (p_sim_tr->no_traits)       /* Tests for already existing traits. */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TRLIST); /* stores select info in nod */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores traits into p_trait array.  */
      }
   }
   return err_typ;
}

int set_inh_tl (void)                   /* set inheritances .. tlocs ..       */
{
   int   i;
   INHERTLOC_PTR p_inhtl;

   if (chk_stat ())
      if (chk_set_inh_tl ())
         for (i = 0; i < nk_par / 2; ++i)
         {
            if ((p_inhtl = chk_inh_tl_node (2 * i))) /* Checks existing node  */
               free_inh_tl_array (p_inhtl); /* for inheritance.               */
            else
               p_inhtl = alloc_inh_tl_node (2 * i);
            alloc_inh_tl_array (p_inhtl); /* Adds tloc numbers array.         */
            stor_inh_tl_array (p_inhtl, 2 * i);
         }
   return err_typ;
}

int set_tr_eff (void)                   /* set trait .. effects ..            */
{
   TRAITEFF_PTR p_t;

   if (chk_stat ())
      if (chk_set_tr_eff ())
      {
         if ((p_t = chk_tr_eff_node (0))) /* Checks for exist node for trait. */
            free_tr_eff_array (p_t);
         else
            p_t = alloc_tr_eff_node (0);
         p_t->set_trait_eff_inp = Y;
         alloc_tr_eff_array (p_t);
         stor_tr_eff_array (p_t);       /* Store effects into eff_nums array. */
      }

   return err_typ;
}

int set_tr_tl (void)                    /* set traits .. tlocs ..             */
{
   int   i;
   TRAITTLOC_PTR p_trtl;

   if (chk_stat ())
      if (chk_set_tr_tl (TRLIST_TLLIST))
         for (i = 0; i < nk_par / 2; ++i)
         {
            /* Checks for an existing trait_tloc_node.                        */
            if ((p_trtl = chk_tr_tl_node (2 * i)))
               free_tr_tl_array (p_trtl);
            else
               p_trtl = alloc_tr_tl_node (2 * i);
            alloc_tr_tl_array (p_trtl, TRLIST_TLLIST); /* Adds tlocs array.   */
            stor_tr_tl_array (p_trtl, 2 * i, TRLIST_TLLIST);
         }
   return err_typ;
}

int set_tr_tl_mlt (void)                /* set trait .. mult tlocs ..         */
{
   TRAITTLOC_PTR p_trtl;

   if (chk_stat ())
      if (chk_set_tr_tl (TR_TLLIST))
      {
         /* Checks for an existing node for the trait.                        */
         if ((p_trtl = chk_tr_tl_node (0)))
            free_tr_tl_array (p_trtl);
         else
            p_trtl = alloc_tr_tl_node (0);
         alloc_tr_tl_array (p_trtl, TR_TLLIST);
         stor_tr_tl_array (p_trtl, 0, TR_TLLIST); /* Stores tloc numbers.     */
      }

   return err_typ;
}
