/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "lsample.h"

void allocate_loci (int nl, int *nall, double **Qq, int pen_flag)
{
   struct Locus *loc;
   int   i, j, k;
   int  *tp1;

   /* Allocate locus structures */
   loci = calloc (nl, sizeof (struct Locus));
   alloc_chk (WHERE, loci);
   tp1 = calloc (nl * (2 * pedsize + n_compnts), sizeof (int));
   alloc_chk (WHERE, tp1);

   for (i = 0; i < nl; i++)
   {
      loc = loci + i;
      /* set basic stuff */
      loc->g_mu = 0;                    /* will be reset if necessary */
      loc->index = i;
      loc->data_index = i;
      loc->n_all = nall[i];
      loc->chromosome = 1;
      loc->malepos = loc->fempos = 0.0;
      loc->freq = Qq[i] + 1;

      if ((i == 0) && (TraitTyp == QUANTITATIVE)) /* TRAIT LOCUS; locus 0; 
                                                     quantitative data */
      {
         loc->locus_type = TRAIT_LOCUS;

         if ((pen_flag == 4) || (IS_LM_SCHNELL))
            /* The following section moved from schnell.c, EAT 6/2003       */
         {
            /* qtl penetrance */
            loc->penetrance = qtlpen;
            loc->data_index = -1;       /* Ignore the trait y[][0][] */
            printf (" Using qtl penetrance for locus %d \n\n", i);

            /* the following may not be necessary -- per Na Li, 2002 */
            loc->sparse_flag = 0;
            loc->allele_trans = 0;
            loc->all_set = 0;
            loc->peel = 0;
            loc->max_alternates = 0;
            loc->genes1[0] = loc->genes1[1] = 0;
            loc->req_set[0] = loc->req_set[1] = loc->req_set[2] = 0;
            loc->r_func = 0;
            loc->n_rfuncs = 0;
         }
      }
      else if ((i == 0) && (TraitTyp == DISCRETE)) /* TRAIT LOCUS; locus 0; 
                                                      phenotypic data */
      {
         loc->locus_type = TRAIT_LOCUS;
         /* Several choices of penetrance model are available  now */
         /* indicated by the flag pen_flag, but pen_flag must be     */
         /* set "by hand" in the appropriate main programs,        */
         /* An improvement on the previous hardwiring: EAT 12/2003 */

         /* "RECESSIVE" -- equiv to genotypic data--for checking    */
         if (pen_flag == 0)
            loc->penetrance = recessive;

         /* INCOMPLETE (Added by AWG) is now the default for phenotypic data */
         /* values are read from the ped file, NOT hardwired:  EAT 6/2003    */
         if (pen_flag == 1)
            loc->penetrance = incomplete;

         /* ORDINAL. Used in GAW13 Analysis  */
         /* Trait is an ordinal trait with 5 levels */
         if (pen_flag == 2)
            loc->penetrance = ordinal;

         /* LIABILITY. Used in AD Analysis   */
         if (pen_flag == 3)
            loc->penetrance = liability;

         /* Logistic Penetrance function */
         if (pen_flag == 5)
            loc->penetrance = logisticpen;

      }
      else if ((i == 0 && TraitTyp == GENOTYPIC) || (i != 0)) /* MARKER or TRAIT
                                                                 genotypic    */
      {
         /* Locus is genotypic trait or marker locus */
         loc->locus_type = MARKER_LOCUS;
         loc->penetrance = 0;
      }
      else                              /* FIXME:  no trait locus */
         ;

      if ((loc->penetrance == incomplete) || (loc->penetrance == qtlpen))
      {
         j = loc->n_all;                /* number of alleles  */
         j = j * (j + 1) / 2;           /* number of (unordered) genotypes */
         /* allocate memory for the penetrances for each genotype */
         loc->g_mu = calloc (j, sizeof (double));
         alloc_chk (WHERE, loc->g_mu);
         if (loc->penetrance == incomplete)
         {
            for (k = 0; k < j; ++k)
               loci[i].g_mu[k] = PenVal[k];
         }
      }
      loc->n_alls = tp1;                /* No. alleles per component */
      tp1 += n_compnts;
      loc->gt[0] = tp1;
      loc->gt[1] = tp1 + pedsize;
      tp1 += 2 * pedsize;
      loc->seg[0] = xx0[i];             /* Set up seg stuff */
      loc->seg[1] = xx1[i];
      loc->genes[0] = gg0[i];
      loc->genes[1] = gg1[i];
   }
   fflush (stdout);
}

void free_allocate_loci (void)
{
   if (loci)
   {
      if (loci[0].g_mu)
         free (loci[0].g_mu);
      if (loci[0].n_alls)
         free (loci[0].n_alls);
      free (loci);
      loci = NULL;
   }
}
