/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/* do_msmpl(), alloc_msmpl(), free_msmpl()  */

/**********************************************************************
*  M-Sample Step     EAT 1997                                         *
*      to subroutine   EAT March 2000                                 *
*  Multiple meiosis version   fwd comput LPT March 2006               *
*      to sub-subroutine  so can do iid samples  EAT May 2006         *
*  Associated alloc/free routines,          added EAT May 2006        *
*  exact_lods modified to take loc_left and rec_left info :           *
*                                                 EAT Sept 2006       *
*  exact_lods modified to do marker computations once only,           *
*      for all trait locus positions:             EAT April 2007      *
*                                                                     *
***********************************************************************
*/

#include "msample.h"

static int allocated = 0;
static int allocated2 = 0;

/* declare the relevant fwd-comput variables as STATIC */

static int nmeio, nm2;
static int *sx, *indx;

static double **pp2;                    /* [NLOC][2^nmeio]  
                                           penetrance probabilities 
                                           at each locus under proposed values 
                                           for switching given meiosis in 
                                           given component                  */
static double **pcf;                    /* [NLOC][2^nmeio]  cumulative peeling 
                                           probs along chromosome: now markers
                                           only                */
static double **pcb;                    /* [NLOC][2^nmeio]  backwards peeling 
                                           probs along chromosome: now markers
                                           only                */
static double *mklf, *mklb;             /* cumulated marker likelihood
                                           fwd (f) and back (b)         */
static int **new_segs;                  /* [NLOC][nmeio], NLOC = nl + 1     */

/* separate out the allocation of all the static variables */
/*  if allocated, and NOT freed, "allocated == nmeio alloc size"           */

void alloc_exact (int nloc)
{
   al_int_vector (&indx, nmeio);
   al_int_vector (&sx, nmeio);
   al_double_vector (&mklf, nloc + 1);
   al_double_vector (&mklb, nloc + 1);
   pp2 = alloc_dbl2d (nloc + 1, nm2);
   pcf = alloc_dbl2d (nloc + 1, nm2);   /* fwds joint: from left */
   pcb = alloc_dbl2d (nloc + 1, nm2);   /*  bk joint : from right */

   allocated = nmeio;
   return;

}

/* and here is the corresponding set of frees */

void free_exact (int nloc)
{
   free_1d ((void *) &indx);
   free_1d ((void *) &sx);
   free_1d ((void *) &mklf);
   free_1d ((void *) &mklb);
   free_2d ((void *) &pp2);
   free_2d ((void *) &pcf);
   free_2d ((void *) &pcb);
   free_2d ((void *) &new_segs);

   allocated = 0;
   allocated2 = 0;
   return;
}

/* set a vector to a constant */
void set_unif (double *vec, int nn, double vv)
{
   int   j;
   for (j = 0; j < nn; ++j)
      vec[j] = vv;
   return;
}

/* then need shell to call the fwd-comput, and, if needed multiple iid samples:
   Separate the sampling from the fwd compute:  EAT May 2006
 */
void exact_smpl (int fwd, int bck,
                 int c, int mm, int nloc, int ngenes, double **prob,
                 int ntp, double *lod)
/* mm=0 if all loci are calculated; mm=1 if markers only */
/* if fwd==1, then do the forward computation, setting pcf */
/* If bck==1, then generate a sample using the current pcf */
/* If fwd==bck==1, this is equivalent to a single fullupdate */
{
   int   i, k, j, jj;
   double rr[2];

   nmeio = cmplist[c].nmeioses;
   nm2 = pow2 (nmeio);

   if (!nmeio)
   {
      for (k = 0; k < ntp; k++)
         lod[k] = 0;
      return;
   }

   if (nmeio > allocated)
   {
      if (allocated)
         free_exact (nloc);
      alloc_exact (nloc);
   }

   for (i = 0; i < nmeio; i++)
   {
      k = cmplist[c].meioses[i];
      indx[i] = (meiosis[k].ind).index;
      sx[i] = meiosis[k].sex;
   }

   /* First store all penetrances, including trait locus in pp2[NLOC][nm2] */

   if (fwd == 1)
   {
      store_pen (c, nloc, mm, indx, sx, nmeio, ngenes, pp2);

      /* We will first compute and store the partial sums up to each locus   */
      /*  In **pcf store P(Sk |Y1,...Yk) and in mklf[k] store log P(Y1,..Yk) */

      if (mm == 0)
      {
         j = 0;
         jj = chromo[0][j];
         mklf[0] = log (fwd_compute (nmeio, j, mm, NULL, sx,
                                     pp2[jj], NULL, pcf[j]));
      }
      for (j = 1; j <= nloc; j++)
      {
         jj = chromo[0][j];
         rr[0] = xecomo[0][0][j - 1];
         rr[1] = xecomo[1][0][j - 1];
         mklf[j] = mklf[j - 1] +
            log (fwd_compute (nmeio, j, 1, rr, sx, pp2[j], pcf[j - 1], pcf[j]));
      }
   }

   /* forward computation is complete  */

   if (bck == 1)
      iid_sample (c, mm, nloc, ngenes, prob);

   /* set xx0, xx1, gg0, gg1 -- this is done at end of iid_sample  */

   return;

}

/* calculate exact probs of marker inheritance on small components */

void exact_ivprob (int c, int mm, int nloc, int ngenes, double **pcf2)
{
   int   i, k, j, jj, *w2;
   double rr[2];

   nmeio = cmplist[c].nmeioses;
   al_int_vector (&w2, nmeio);
   nm2 = pow2 (nmeio);

   if (!nmeio)
      return;

   if (nmeio > allocated)
   {
      if (allocated)
         free_exact (nloc);
      alloc_exact (nloc);
   }

   for (i = 0; i < nmeio; i++)
   {
      k = cmplist[c].meioses[i];
      indx[i] = (meiosis[k].ind).index;
      sx[i] = meiosis[k].sex;
   }

   /* First store all penetrances, including trait locus in pp2[NLOC][nm2] */

   store_pen (c, nloc, mm, indx, sx, nmeio, ngenes, pp2);

/* check some pp2 probs  */
   if (DebugSample)
   {
      for (k = 0; k < nmeio; ++k)
         w2[k] = 0;
      for (i = 0; i < nm2; ++i)
      {
         printf ("\n");
         printf (" %d ", i);
         for (k = 0; k < nmeio; ++k)
            printf (" %d", w2[k]);
         for (j = mm; j <= nloc; ++j)
            printf (" %9.6f", pp2[j][i]);
         biadd (w2, nmeio, 1);
      }
      printf ("\n");
      fflush (stdout);
   }
   /* We will first compute and store the partial sums up to each marker */
   /*  In **pcf store P(Sk |Y1,...Yk) and in mklf[k] store log P(Y1,..Yk) */

   if (mm == 0)
   {
      j = 0;
      jj = chromo[0][j];
      mklf[0] = log (fwd_compute (nmeio, j, mm, NULL, sx,
                                  pp2[jj], NULL, pcf[j]));
   }
   for (j = 1; j <= nloc; j++)
   {
      jj = chromo[0][j];
      rr[0] = xecomo[0][0][j - 1];
      rr[1] = xecomo[1][0][j - 1];
      mklf[j] = mklf[j - 1] +
         log (fwd_compute (nmeio, j, 1, rr, sx, pp2[jj], pcf[j - 1], pcf[j]));
   }

   /* We will next compute and store the partial sums beyond each marker    */
   /*  In **pcb store P(Sk |Yk,...YL) and in mklb[k] store log P(Yk,...,Yl) */

   jj = chromo[0][nloc];
   mklb[nloc] = log (fwd_compute (nmeio, nloc, nloc, NULL, sx, pp2[jj],
                                  NULL, pcb[nloc]));
   for (j = nloc - 1; j >= mm; --j)
   {
      jj = chromo[0][j];
      rr[0] = xecomo[0][0][j];
      rr[1] = xecomo[1][0][j];
      mklb[j] = mklb[j + 1] +
         log (fwd_compute (nmeio, nloc + 1 - j, mm, rr, sx, pp2[jj],
                           pcb[j + 1], pcb[j]));
   }

/* check some probs  */
   if (DebugSample)
   {
      for (k = 0; k < nmeio; ++k)
         w2[k] = 0;
      for (i = 0; i < nm2; ++i)
      {
         printf ("\n");
         printf (" %d ", i);
         for (k = 0; k < nmeio; ++k)
            printf (" %d", w2[k]);
         for (j = mm; j <= nloc; ++j)
            printf (" %9.6f", pcf[j][i]);
      }

      for (i = 0; i < nm2; ++i)
      {
         printf ("\n");
         printf (" %d ", i);
         for (k = 0; k < nmeio; ++k)
            printf (" %d", w2[k]);
         for (j = mm; j <= nloc; ++j)
            printf (" %9.6f", pcb[j][i]);
         biadd (w2, nmeio, 1);
      }
      printf ("\n");
      fflush (stdout);
   }

   /* Then we combine these, but where should we put them: in pcf I guess */
   /*  P(Sk | Y) : pcf[k] * pcb[k+1] * P(S_k | S_k+1) sum over S_k+1      */

   fwd_bk_combine (nmeio, nloc, mm, mm, rr, sx, pcf[mm], pcb[mm]);
   /* case j=mm */
   for (j = mm + 1; j < nloc; j++)
   {
      jj = chromo[0][j];
      rr[0] = xecomo[0][0][j];
      rr[1] = xecomo[1][0][j];
      fwd_bk_combine (nmeio, nloc, j, mm, rr, sx, pcf[j], pcb[j + 1]);
   }
   /* note case j=nloc is already set in pcf */

/* exact probabilities for locus j are now in pcf[j]  */

   if (DebugSample)
   {
      for (k = 0; k < nmeio; ++k)
         w2[k] = 0;
      for (i = 0; i < nm2; ++i)
      {
         printf ("\n");
         printf (" %d ", i);
         for (k = 0; k < nmeio; ++k)
            printf (" %d", w2[k]);
         for (j = mm; j <= nloc; ++j)
            printf (" %9.6f", pcf[j][i]);
         biadd (w2, nmeio, 1);
      }
      printf ("\n");
      fflush (stdout);
   }

   for (i = 0; i < nm2; ++i)
      for (j = mm; j <= nloc; ++j)
         pcf2[j][i] = pcf[j][i];
   return;
}

/* calculate the exact lods for small components*/

void exact_lods (int c, int nloc, int ngenes, int ntp, double *lod,
                 int *lleft, double *rlm, double *rlf,
                 int *lrght, double *rrm, double *rrf)
{
   int   i, k, j;
   double rr[2], cc;

   nmeio = cmplist[c].nmeioses;
   nm2 = pow2 (nmeio);

   if (!nmeio)
   {
      for (k = 0; k < ntp; k++)
         lod[k] = 0;
      return;
   }

   if (nmeio > allocated)
   {
      if (allocated)
         free_exact (nloc);
      alloc_exact (nloc);
   }

   for (i = 0; i < nmeio; i++)
   {
      k = cmplist[c].meioses[i];
      indx[i] = (meiosis[k].ind).index;
      sx[i] = meiosis[k].sex;
   }

   /* First store all penetrances, including trait locus in pp2[NLOC][nm2] */

   store_pen (c, nloc, 0, indx, sx, nmeio, ngenes, pp2);

   /* We will first compute and store the partial sums up to each marker */
   /*  In **pcf store P(Sk |Y1,...Yk) and in mklf[k] store log P(Y1,..Yk) */

   mklf[1] = log (fwd_compute (nmeio, 1, 1, NULL, sx, pp2[1], NULL, pcf[1]));
   for (j = 2; j <= nloc; ++j)
   {
      rr[0] = Marek[0][j - 1];
      rr[1] = Marek[1][j - 1];
      mklf[j] = mklf[j - 1] +
         log (fwd_compute (nmeio, j, 1, rr, sx, pp2[j], pcf[j - 1], pcf[j]));
   }

   /* We will next compute and store the partial sums beyond each marker    */
   /*  In **pcb store P(Sk |Yk,...YL) and in mklb[k] store log P(Yk,...,Yl) */

   mklb[nloc] = log (fwd_compute (nmeio, 1, 1, NULL, sx, pp2[nloc],
                                  NULL, pcb[nloc]));
   for (j = nloc - 1; j >= 1; --j)
   {
      rr[0] = Marek[0][j];
      rr[1] = Marek[1][j];
      mklb[j] = mklb[j + 1] +
         log (fwd_compute (nmeio, nloc + 1 - j, 1, rr, sx, pp2[j],
                           pcb[j + 1], pcb[j]));
   }

   /* Then we will peel in the trait locus in each requested position   */
   /*  using the loc-left and rec-left etc info -- not globals          */

   lod[0] = 0.0;                        /* multiply by 1/2^m     */
   for (k = 0; k < nm2; ++k)
      lod[0] += pp2[0][k];
   lod[0] = log (lod[0]) - log (nm2);   /* multiply by 1/2^m   */

   for (i = 1; i < ntp; i++)
   {
      lod[i] = 0.0;                     /* initialize lod  */
      cc = -mklf[nloc];                 /*  -log P(Y_M)      */

      k = lleft[i];
      if (k != -1)
      {
         rr[0] = rlf[i];
         rr[1] = rlm[i];
         (void) fwd_compute (nmeio, 0, 1, rr, sx, NULL, pcf[k], pcf[0]);
         cc += mklf[k];
      }
      else
         set_unif (pcf[0], nm2, 1.0 / nm2); /* before first marker */

      k = lrght[i];
      /* beyond or at last marker */
      if ((k == -1) || ((k == lleft[i]) && (k == nloc)))
         set_unif (pcb[0], nm2, 1.0 / nm2);
      else if (k == lleft[i])           /* marker location (not last) */
      {
         rr[0] = Marek[0][k];
         rr[1] = Marek[1][k];
         (void) fwd_compute (nmeio, 0, 1, rr, sx, NULL, pcb[k + 1], pcb[0]);
         cc += mklb[k + 1];
      }
      else                              /* inter-marker location */
      {
         rr[0] = rrf[i];
         rr[1] = rrm[i];
         (void) fwd_compute (nmeio, 0, 1, rr, sx, NULL, pcb[k], pcb[0]);
         cc += mklb[k];
      }

      cc += log (nm2);                  /* divide by P(S_T) = 1/nm2  */

      for (k = 0; k < nm2; ++k)
         lod[i] += pp2[0][k] * pcf[0][k] * pcb[0][k];
      lod[i] = log (lod[i]) + cc;
   }

   lod[0] = lod[0] / log (10.0);
   for (i = 1; i < ntp; i++)
      lod[i] = lod[i] / log (10.0) - lod[0];

   return;
}

void iid_sample (int c, int mm, int nloc, int ngenes, double **prob)
{
   int   i, k, mmm;

   nmeio = cmplist[c].nmeioses;
   if (nmeio > allocated2)
   {
      if (allocated2)
      {
         free_2d ((void *) &new_segs);
         allocated2 = nmeio;
      }
      new_segs = alloc_int2d (nloc + 1, nmeio);
      allocated2 = nmeio;
   }

   mmm = mm;
   if (chromo[0][0] != 0)
      mmm = 0;

   for (i = 0; i < nmeio; i++)
   {
      k = cmplist[c].meioses[i];
      indx[i] = (meiosis[k].ind).index;
      sx[i] = meiosis[k].sex;
   }

   /* sample new indicators by "reverse peeling" */

   bck_smpl (nloc, mmm, nmeio, sx, pcf, new_segs);

   /* assign new inheritance indicators to xx0 or xx1 and re-set genes */

   assign_reset (c, nloc, mmm, nmeio, indx, sx, ngenes, prob, new_segs);

   return;
}
