/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/* do_msmpl(), alloc_msmpl(), free_msmpl()  */

/*******************************************************************
*    M-Sample Step     EAT 1997                                    *
*      to subroutine   EAT March 2000                              *
*    Multiple meiosis version   LPT  2004?                         *
*    Separate out allocs and frees etc   EAT May 2006              *
*    Split the big do_msmpl routine to smaller bits  EAT June 2006 *
*      (so can use forward and back steps separately)              *
*    This file contains the 4 subroutines only                     *
********************************************************************/

#include "lsample.h"
#include "msample.h"

static int err = 0;

/* routine to store the locus-by-locus penetrances values  */

void store_pen (int c, int nl, int mmm, int *indx, int *sx, int nmeio,
                int ng, double **spp2)
{
   int   jj, i, ii, nm2;
   int  *ss;                            /* ss[nmeio] inheritance indicator vector
                                           at some location j */
   al_int_vector (&ss, nmeio);
   nm2 = pow2 (nmeio);

   for (jj = mmm; jj <= nl; jj++)
   {
      for (i = 0; i < nmeio; i++)
         ss[i] = 0;
      for (ii = 0; ii < nm2; ii++)
      {
         for (i = 0; i < nmeio; i++)
         {
            if (sx[i] == 0)
               xx0[jj][indx[i]] = ss[i];
            else
               xx1[jj][indx[i]] = ss[i];
         }
         reset_FGL (jj, c);
         spp2[jj][ii] = seg_pen2 (c - 1, loci[jj].freq, jj, ng, &err);
         biadd (ss, nmeio, 1);
      }
   }
   free_1d ((void *) &ss);
   return;

}

/* routine to do the forwards computation */

double calc_fwd (int nl, int mmm, int nmeio, int *sx,
                 double **spp2, double **spcf)
{
   int   j, jj;
   double rr[2];                        /* rr is the recombination frequency 
                                           for female/male */
   double slklhd = 0.0;                 /* lod score (?) to be returned */

   if (mmm == 0)
   {
      j = 0;
      jj = chromo[0][j];
      slklhd += log (fwd_compute (nmeio, j, mmm, NULL, sx,
                                  spp2[jj], NULL, spcf[j]));
   }
   for (j = 1; j <= nl; j++)
   {
      jj = chromo[0][j];
      rr[0] = xecomo[0][0][j - 1];
      rr[1] = xecomo[1][0][j - 1];
      slklhd += log (fwd_compute (nmeio, j, mmm, rr, sx,
                                  spp2[jj], spcf[j - 1], spcf[j]));
   }

   return (slklhd);
}

/* routine to back sample accross all the loci */
void bck_smpl (int nl, int mmm, int nmeio, int *sx,
               double **spcf, int **sn_segs)
{
   int   j;
   double rr[2];                        /* rr is the recombination frequency 
                                           for female/male */
   j = nl;
   back_sample (nmeio, nl, j, NULL, NULL, sx, spcf[j], sn_segs[j]);
   for (j = nl - 1; j >= mmm; j--)
   {
      rr[0] = xecomo[0][0][j];
      rr[1] = xecomo[1][0][j];
      back_sample (nmeio, nl, j, sn_segs[j + 1], rr, sx, spcf[j], sn_segs[j]);
   }

   return;

}

/* routine to assign neg segs and reset genes */
void assign_reset (int c, int nl, int mmm, int nmeio, int *indx, int *sx,
                   int ng, double **prob, int **sn_segs)
{
   int   j, jj, i;
   for (j = mmm; j <= nl; j++)
   {
      jj = chromo[0][j];
      for (i = 0; i < nmeio; i++)
      {
         if (sx[i] == 0)
            xx0[jj][indx[i]] = sn_segs[j][i];
         else
            xx1[jj][indx[i]] = sn_segs[j][i];
      }
      reset_FGL (jj, c);
      prob[c - 1][jj] = seg_pen2 (c - 1, loci[jj].freq, jj, ng, &err);
   }
   return;
}

/*********************************************************/
/* This function is first defined by Liping in Nov, 2004 */
/* purpose: calculate w+n, where w is binomial vector, and n is a number
   input: w[0...m-1], n
   output: none, but w[1...m-1] will be modified by this subroutine
*/

void biadd (int *w, int m, int n)
{
   int   i, j;
   for (i = 0; i < n; i++)
   {
      j = m - 1;
      if (w[j] == 1)
      {
         while (j >= 0 && w[j] == 1)
         {
            w[j] = 0;
            j--;
         }
      }
      if (j >= 0)
         w[j] = 1;
   }
   return;
}

/*********************************************************/
/* purpose: calculate 2^m, obtained from Myrna, 2004 */
long int pow2 (int m)
{
   unsigned long l = LONG_MAX;
   static int max_m = 0;                /* static variable */
   unsigned long a = 1;                 /* automatic variable */

   if (!max_m)                          /* on first entry, determine max m */
   {
      while (l)
      {
         l = l >> 1;
         ++max_m;
      }
      --max_m;
   }

   if (m >= 0 && m <= max_m)
      return a << m;                    /* shift "1" m bit positions to left */

   else
      return 0;
}
